/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
//#define INCLUDE_DRIVINIT_H
#define INCLUDE_COMMDLG_H       1
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <dos.h>
#include <string.h>
#include <malloc.h>
#include <strsafe.h>
#include "tibw.h"
#include "tform.h"
#include "tibform.ext"

void display_prev ( void ); 
void CreateBitmapInfoStruct ( void );
void CreateBMPFile ( void );
void DrawBitmapXY ( HDC, HBITMAP, int, int );

// Key state definitions:

#define SHIFT_DOWN                1
#define CONTROL_DOWN              2
#define ALT_DOWN                  4
#define F9_DOWN                   8
#define F10_DOWN                 16

extern RECT deskrc;
RECT prev_rc;  
HDC pag_DC; // = CreateCompatibleDC(NULL);
int cur_mem_pag;
int max_mem_pag;
HBRUSH pag_brush; // = CreateSolidBrush(RGB(0, 0xff, 0));
HBRUSH pag_brush2;
HBITMAP pag_bmp;  
PBITMAPINFO pbmi;
int bmpfilhedsiz, bmppbisiz, bmpbitssiz; // File component sizes,
int bmpfilsiz; // Size of saved BMP files.
int key_state;  
float zoom_factor = 0.5; 
int tgt_cx, tgt_cy;
int makingpreview = 0;

// Preview window scaling data:

int src_xmax, src_ymax, src_cenx, src_ceny; // Source image data
int dis_cenx, dis_ceny, dis_xmax, dis_ymax; // Client area values
int proj_xmax, proj_ymax; // Width, height of scaled image

extern HWND Tibw_hwnd; 
extern HANDLE Tibw_inst; 
HWND prev_hwnd;
HDC prev_hdc; 
char fnam[10];

extern char *tf37buf;
extern int tf37_xpt, tf37_ypt, tf37_fc, tf37_lc;
extern HBITMAP tf37_hbm[94];

extern char *tf38buf;
extern int tf38_xpt, tf38_ypt, tf38_fc, tf38_lc;
extern HBITMAP tf38_hbm[60];

extern char *tf39buf;
extern int tf39_xpt, tf39_ypt, tf39_fc, tf39_lc;
extern HBITMAP tf39_hbm[94];

extern char *tf40buf;
extern int tf40_xpt, tf40_ypt, tf40_fc, tf40_lc;
extern HBITMAP tf40_hbm[60];

extern char *tf41buf;
extern int tf41_xpt, tf41_ypt, tf41_fc, tf41_lc;
extern HBITMAP tf41_hbm[94];

extern char *tf42buf;
extern int tf42_xpt, tf42_ypt, tf42_fc, tf42_lc;
extern HBITMAP tf42_hbm[60];

char *pgBits;
int pgxln, pgbufsiz;

void display_prev ( void )         
  {
    int x1, y1, w1, h1;
    POINT Pt[3];
  
    x1 = tgt_cx - (int) ( zoom_factor * (float) src_cenx );
    y1 = tgt_cy - (int) ( zoom_factor * (float) src_ceny );
    w1 = (int) ( zoom_factor * (float) pag_bmpWidth );
    h1 = (int) ( zoom_factor * (float) pag_bmpHeight );    
    
    SelectObject( prev_hdc, pag_brush2 );
    Rectangle( prev_hdc, prev_rc.left, prev_rc.top, prev_rc.right+1, prev_rc.bottom+1 );  
    if ( portprn )
      StretchBlt( prev_hdc, x1, y1, w1, h1, 
                pag_DC, 
                0, 0, src_xmax, src_ymax, 
                SRCCOPY );        
    else
      {                
        y1 = tgt_cy - (int) ( zoom_factor * (float) src_cenx );
        x1 = tgt_cx - (int) ( zoom_factor * (float) src_ceny );      
        h1 = (int) ( zoom_factor * (float) pag_bmpWidth );
        w1 = (int) ( zoom_factor * (float) pag_bmpHeight );       
        Pt[0].x = x1 + w1;
        Pt[0].y = y1;
        Pt[1].x = x1 + w1;
        Pt[1].y = y1 + h1;
        Pt[2].x = x1;
        Pt[2].y = y1;       
        PlgBlt ( prev_hdc, Pt, pag_DC, 0, 0, src_xmax, src_ymax, NULL, 0, 0 );                 
      }         
  } // END - display_prev ()     

LONG _EXPORT FAR PASCAL PrevWindowProc( HWND hwnd, unsigned msg,
                                     UINT wparam, LONG lparam )
  {
    int i;
    BOOL res;
    PAINTSTRUCT ps;    
      switch( msg ) {
      case WM_CLOSE:
        DestroyWindow( hwnd );
        break;
      case WM_CREATE:
        pag_brush2 = CreateSolidBrush (RGB(0x7f, 0x7f, 0x7f)); 
        break;
      case WM_DESTROY:
        SetCurrentDirectory ( cur_dir ); // Not really needed - just to be sure!
        for ( i = 0; i <= max_mem_pag; ++i )
          {
            sprintf ( fnam, "temp%.3d.bmp", i );
            res = DeleteFile ( fnam ); 
          }      
        DeleteObject(pag_brush2);   
        DeleteObject(pag_bmp);
        DeleteObject(pag_brush);
        DeleteDC(pag_DC);             
        free ( pgBits );
        DeleteObject( pag_DC );
        prev_on = 0; // Allow printing; preview mode off.          
        break;
      case WM_PAINT:   
      doredraw:
        BeginPaint ( prev_hwnd, &ps);
        dis_xmax = prev_rc.right-prev_rc.left;
        dis_ymax = prev_rc.bottom-prev_rc.top;
        dis_cenx = dis_xmax / 2;
        dis_ceny = dis_ymax / 2;   
        if ( !portprn ) // Landscape
          {
            proj_xmax = (int) ( zoom_factor * (float) pag_bmpHeight );          
            proj_ymax = (int) ( zoom_factor * (float) pag_bmpWidth );                      
          }
        else // Portrait
          {
            proj_ymax = (int) ( zoom_factor * (float) pag_bmpHeight );          
            proj_xmax = (int) ( zoom_factor * (float) pag_bmpWidth );                      
          }        
        if ( proj_xmax < dis_xmax )
          tgt_cx = dis_cenx;
        if ( proj_ymax < dis_ymax )
          tgt_cy = dis_ceny;          
        display_prev ( );         
        EndPaint (hwnd, &ps);      
        break;
      case WM_SIZE:
        prev_rc.left = 0;  
        prev_rc.right = LOWORD( lparam );
        prev_rc.bottom = HIWORD( lparam );
        prev_rc.top = 0;
        dis_xmax = prev_rc.right-prev_rc.left;
        dis_ymax = prev_rc.bottom-prev_rc.top;
        dis_cenx = dis_xmax / 2;
        dis_ceny = dis_ymax / 2;             
        proj_xmax = (int) ( zoom_factor * (float) pag_bmpWidth );
        if ( proj_xmax < dis_xmax )
          tgt_cx = dis_cenx;
        proj_ymax = (int) ( zoom_factor * (float) pag_bmpHeight ); 
        if ( proj_ymax < dis_ymax )
          tgt_cy = dis_ceny;
        break;
      case WM_COMMAND:
        switch( LOWORD( wparam ) ) 
          {
//          case MENU_ABOUT:
//            ;
//            break;
//          case MENU_CLEAR:
//            ;       
//          break;
         }
        break;

      case WM_SYSKEYDOWN:
      case WM_KEYDOWN:

        key_state = 0L;
        if ( HIWORD(lparam) & KF_ALTDOWN /* 0x2000 */ )
          key_state = key_state + ALT_DOWN;
        if ( GetKeyState ( VK_SHIFT) < 0 )
          key_state = key_state + SHIFT_DOWN;
        if ( GetKeyState ( VK_CONTROL ) < 0 )
          key_state = key_state + CONTROL_DOWN;
        if ( GetKeyState ( VK_F9 ) < 0 )
          key_state = key_state + F9_DOWN;
        else if ( GetKeyState ( VK_F10 ) < 0 )
          key_state = key_state + F10_DOWN;
 
      switch ( wparam ) {
// Page Up:
        case VK_NUMPAD9:
        case VK_PRIOR:     
          if ( cur_mem_pag > 0 && !makingpreview )
            {
              --cur_mem_pag;       
              read_mempag ( cur_mem_pag );
              goto doredraw;                
            }    
          break;
// Page Down:
        case VK_NUMPAD3:
        case VK_NEXT:
          if ( cur_mem_pag < max_mem_pag && !makingpreview )
            {
              ++cur_mem_pag;    
              read_mempag ( cur_mem_pag );            
              goto doredraw;               
            }
          break;  
        case VK_NUMPAD1:
        case VK_END:    
          cur_mem_pag = max_mem_pag;
          read_mempag ( cur_mem_pag );                      
          goto doredraw;          
          break;
        case VK_NUMPAD7:
        case VK_HOME:      
          cur_mem_pag = 0;
          read_mempag ( cur_mem_pag );                                
          goto doredraw;          
          break;                
// Arrows:
        case VK_DOWN:   
          tgt_cy = tgt_cy + 75;            
          goto doredraw;                    
          break;
        case VK_UP:        
          tgt_cy = tgt_cy - 75;                 
          goto doredraw;          
          break;
        case VK_LEFT:
          tgt_cx = tgt_cx - 75;            
          goto doredraw;          
        case VK_RIGHT:
          tgt_cx = tgt_cx + 75;            
          goto doredraw;                    
          break;          
        case VK_ESCAPE:    DestroyWindow( prev_hwnd );        break;
        case 187: 
          zoom_factor = zoom_factor * 1.25;
          GetClientRect( prev_hwnd, &prev_rc );              
          goto doredraw;        
          break;
        case VK_ADD:    
          if ( key_state & CONTROL_DOWN )
            {          
              zoom_factor = zoom_factor * 1.25;
              GetClientRect( prev_hwnd, &prev_rc );              
              goto doredraw;
            }
          break;
        case 189: 
          zoom_factor = zoom_factor / 1.25;       
          GetClientRect( prev_hwnd, &prev_rc );  
          goto doredraw;                      
          break;                  
        case VK_SUBTRACT:              
          if ( key_state & CONTROL_DOWN )
            {
              zoom_factor = zoom_factor / 1.25;       
              GetClientRect( prev_hwnd, &prev_rc );  
              goto doredraw;              
            }
          break;                
        default: ;
    } // END of switch.
    ;	
      break; // END - case WM_SYSKEYDOWN: and case WM_KEYDOWN:
     ; 
   default:
         return( DefWindowProc( hwnd, msg, wparam, lparam ) );
    }
    return( 0L );
  } // END - PrevWindowProc

// Routine to print character in preview device context.
int wrt_chr ( unsigned char c ) // Always returns zero - could change this
  {
    int x, y;
    if ( c < 33 )  // Remove this later, or keep as error check.
      {
        sprintf ( d_bugstr, "Spurious character: %d.", (int) c );
        wrt_debug ();
        return (0);
      }
    if ( portprn )
      {
        if ( smlfnt )
          y = pag_y + 7;
        else
          y = pag_y;
        if ( cur_fnt == 37 )
          DrawBitmapXY ( pag_DC, tf37_hbm[(int) c - tf37_fc], pag_x, y );        
        else if ( cur_fnt == 38 )
          DrawBitmapXY ( pag_DC, tf38_hbm[(int) c - tf38_fc], pag_x, y );         
        else if ( cur_fnt == 39 )
          DrawBitmapXY ( pag_DC, tf39_hbm[(int) c - tf39_fc], pag_x, y );         
        else if ( cur_fnt == 40 )
          DrawBitmapXY ( pag_DC, tf40_hbm[(int) c - tf40_fc], pag_x, y );                       
      }
    else // if landscape
      {
        if ( smlfnt )  
          x = pag_x + 7;
        else
          x = pag_x;
        y = pag_y;
        if ( cur_fnt == 37 )
          DrawBitmapXY ( pag_DC, tf37_hbm[(int) c - tf37_fc], x, y - tf37_ypt ); 
        else if ( cur_fnt == 38 )
          DrawBitmapXY ( pag_DC, tf38_hbm[(int) c - tf38_fc], x, y - tf38_ypt );         
        else if ( cur_fnt == 39 )
          DrawBitmapXY ( pag_DC, tf39_hbm[(int) c - tf39_fc], x, y - tf39_ypt );         
        else if ( cur_fnt == 40 )
          DrawBitmapXY ( pag_DC, tf40_hbm[(int) c - tf40_fc], x, y - tf40_ypt );                       
        else if ( cur_fnt == 41 ) 
          DrawBitmapXY ( pag_DC, tf41_hbm[(int) c - tf41_fc], x, y - tf41_ypt );         
        else if ( cur_fnt == 42 )
          DrawBitmapXY ( pag_DC, tf42_hbm[(int) c - tf42_fc], x, y - tf42_ypt );                                 
      }
    return (0);
  }  // END wrt_chr ()  
  
void do_preview ( void )
  {
    int initx, inity, initwidth, initheight;  
    int scaling_needed_save, pr_vertres_save;
    DWORD lsterror;  
    prev_on = 1; // Preview mode flag
    pr_vertres_save = pr_vertres; // This has to be used in following routines:
    set_pms ();  // This is in TF1.C - does not set paper size parameters except line spacing - move that???
    if ( paper_size == A4_PAPER )
      {
        set_pms_a4 (); // In TF1.C -- need to do this to set bitmap size!  
        pr_vertres = pag_bmpHeight; 
      }
    else if ( paper_size == A3_PAPER )
      {
        portprn = 0;
        set_pms_a3 ();
        pr_vertres = pag_bmpHeight; // 730 + 200 + 200          
      }      
    else if ( paper_size == Tabloid_PAPER )
      {
        portprn = 0;
        set_pms_Tabloid ();
        pr_vertres = pag_bmpHeight; // 730 + 200 + 200          
      }      
    else if ( paper_size == LEGAL_PAPER )
      {
        set_pms_legal ();
        pr_vertres = pag_bmpHeight; // 730 + 200 + 200          
      }      
    else if ( paper_size == LETTER_PAPER )
      {
        set_pms_letter ();
        pr_vertres = pag_bmpHeight; // 730 + 200 + 200          
      }                        
    else 
      {
        prev_on = 0;
        paper_size = paper_size_save;        
        return;  
      }
// Create preview Window:
    if ( portprn )
      {
        inity = 0;      
        initwidth = deskrc.right - deskrc.left; // 1000;
        initheight = deskrc.bottom - deskrc.top; // 700;
        initwidth = ( initheight * pag_bmpWidth ) / pag_bmpHeight + 2;
        initx = ( deskrc.right - deskrc.left - initwidth ) / 2;
      }
    else  
      {
        initx = 0;
        inity = 40;      
        initwidth = deskrc.right - deskrc.left; // 1000;
        initheight = (deskrc.bottom - deskrc.top) * 2 / 3; // 700;
      }    
    prev_hwnd = CreateWindow (
        "prvwClass", 	
        "Test screen writing", WS_OVERLAPPEDWINDOW,
        initx, 		
        inity, 		
        initwidth,    	
        initheight,   
        Tibw_hwnd,   
        NULL,       
        Tibw_inst,   
        NULL  
       );
    if( !prev_hwnd ) 
      {
        Msg ("Creating pop_up failed!");
        lsterror = GetLastError ();
        sprintf ( d_bugstr, "Last error: %d.", lsterror );
        wrt_debug ();
        return;
      }
    ShowWindow( prev_hwnd, SW_SHOW );       
    GetClientRect( prev_hwnd, &prev_rc);  
    scaling_needed_save = scaling_needed;     
    scaling_needed = 0;  
    prev_hdc = GetDC ( prev_hwnd ); // This is for test screen writing
    cur_mem_pag = 0;
    max_mem_pag = 0;
    pag_brush = CreateSolidBrush (RGB(0xff, 0xff, 0xff));    
    pag_DC = CreateCompatibleDC (NULL);
    pgxln = pag_bmpWidth / 8; 
    if ( pgxln * 8 < pag_bmpWidth )
      ++pgxln;  
    if ( pgxln % 2 )
      ++pgxln;
    pgbufsiz = pgxln * pag_bmpHeight;
    pgBits = malloc ( pgbufsiz );  
    if ( pgBits == NULL )
      {
        sprintf ( d_bugstr, "Memory allocation failure, size: %d.", pgbufsiz );
        Msg ( d_bugstr );
        wrt_debug ();
        return;
      }
    pag_bmp = CreateBitmap ( pag_bmpWidth, pag_bmpHeight, 1, 1, pgBits );
    if ( pag_bmp == NULL ) 
      {
        Msg ( "Create bitmap failed!" );
        sprintf ( d_bugstr, "Bit map creation failure:" );
        wrt_debug ();
        sprintf ( d_bugstr, "Width: %d, Height: %d.", pag_bmpWidth, pag_bmpHeight );
        wrt_debug ();  
        clos_debug ();
        exit (0);      
      }
    SelectObject( pag_DC, pag_bmp );
    SelectObject( pag_DC, pag_brush );
    makingpreview = 1;
    main_print (); // Sets the A3/A4 parameters - in TF1, calls port_rout, land_rout   
    makingpreview = 0;    
    cur_mem_pag = 0;          
    read_mempag ( cur_mem_pag );
    display_prev ( );    
        
    src_xmax = pag_bmpWidth;
    src_ymax = pag_bmpHeight;
    src_cenx = src_xmax / 2;
    src_ceny = src_ymax / 2;    

    dis_xmax = prev_rc.right-prev_rc.left;
    dis_ymax = prev_rc.bottom-prev_rc.top;
    dis_cenx = dis_xmax / 2;
    dis_ceny = dis_ymax / 2;     
    if ( portprn )
      zoom_factor = (float) dis_ymax / (float) src_ymax;      
    else
      zoom_factor = (float) dis_xmax / (float) src_ymax;
    tgt_cx = dis_cenx;
    tgt_cy = dis_ceny;    
    scaling_needed = scaling_needed_save;  
    pr_vertres = pr_vertres_save;
    paper_size = paper_size_save;
  } // END - do_preview
  
void start_mempag ( void )
  {
    if ( pag_DC )
      DeleteDC(pag_DC);       
    pag_DC = CreateCompatibleDC (NULL);    
    SelectObject( pag_DC, pag_bmp );
    SelectObject( pag_DC, pag_brush );    
    Rectangle( pag_DC, 0, 0, pag_bmpWidth, pag_bmpHeight );    
  } // END - start_mempag 

void save_mempag ( int x )
  {
    BITMAP bmp;
    HANDLE hf;                  // file handle
    BITMAPFILEHEADER hdr;       // bitmap file-header
    PBITMAPINFOHEADER pbih;     // bitmap info-header
    LPBYTE lpBits;              // memory pointer
    DWORD dwTotal;              // total count of bytes
    DWORD cb;                   // incremental count of bytes
    BYTE *hp;                   // byte pointer
    DWORD dwTmp;    
    WORD    cClrBits;
    int i;
    BOOL res;

    if ( x >=999 )
      {
        Msg ( "Too many errors in the source file!" );
        clos_debug ();
        for ( i = 0; i < x; ++i )
          {
            sprintf ( fnam, "temp%.3d.bmp", i );
            res = DeleteFile ( fnam ); 
          }   
        exit (0);   
      }
    if ( cur_mem_pag > max_mem_pag ) // Do this here because in pecha
      max_mem_pag = cur_mem_pag;     // mode they are created out of order
    if ( d_bug > 1 )
      {  
        sprintf ( d_bugstr, "Saving file: %.3d", x );
        wrt_debug ();    
      }
    sprintf ( fnam, "temp%.3d.bmp", x );
// Following code based on:
// http://msdn.microsoft.com/en-us/library/windows/desktop/dd145119%28v=vs.85%29.aspx    
// Retrieve the bitmap colour format, width, and height.
    if (!GetObject( pag_bmp, sizeof(BITMAP), (LPSTR)&bmp))
        Msg ( "GetObject" );
// Allocate memory for the BITMAPINFO structure. (This structure
// contains a BITMAPINFOHEADER structure and an array of RGBQUAD
// data structures.)
    cClrBits = 1;
    pbmi = (PBITMAPINFO) LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER) +
                    sizeof(RGBQUAD) * (1<< cClrBits));
// Initialize the fields in the BITMAPINFO structure.
    pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    pbmi->bmiHeader.biWidth = bmp.bmWidth;
    pbmi->bmiHeader.biHeight = bmp.bmHeight;
    pbmi->bmiHeader.biPlanes = bmp.bmPlanes;
    pbmi->bmiHeader.biBitCount = bmp.bmBitsPixel;
//    if (cClrBits < 24)
    pbmi->bmiHeader.biClrUsed = (1<<cClrBits);
    if ( d_bug > 1 )
      {      
        sprintf ( d_bugstr, "Write, size: %d, wid: %d, h: %d, plan: %d, cnt: %d",
              pbmi->bmiHeader.biSize, pbmi->bmiHeader.biWidth, pbmi->bmiHeader.biHeight,
              pbmi->bmiHeader.biPlanes, pbmi->bmiHeader.biBitCount );
        wrt_debug ();
      }
// If the bitmap is not compressed, set the BI_RGB flag.
    pbmi->bmiHeader.biCompression = BI_RGB;
// Compute the number of bytes in the array of colour
// indices and store the result in biSizeImage.
// For Windows NT, the width must be DWORD aligned unless
// the bitmap is RLE compressed. This example shows this.
// For Windows 95/98/Me, the width must be WORD aligned unless the
// bitmap is RLE compressed.
    pbmi->bmiHeader.biSizeImage = ((pbmi->bmiHeader.biWidth * cClrBits +31) & ~31) /8
                                  * pbmi->bmiHeader.biHeight;
// Set biClrImportant to 0, indicating that all of the
// device colours are important.
    pbmi->bmiHeader.biClrImportant = 0;
    pbih = (PBITMAPINFOHEADER) pbmi;
    lpBits = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbih->biSizeImage);
    if (!lpBits) Msg ( "GlobalAlloc fail!" );
// Retrieve the colour table (RGBQUAD array) and the bits
// (array of palette indices) from the DIB.
    if (!GetDIBits(pag_DC, pag_bmp, 0, (WORD) pbih->biHeight, lpBits, pbmi, DIB_RGB_COLORS))
      Msg ( "GetDIBits" );
// Create the .BMP file.
    hf = CreateFile( fnam,
                   GENERIC_READ | GENERIC_WRITE,
                   (DWORD) 0,
                    NULL,
                   CREATE_ALWAYS,
                   FILE_ATTRIBUTE_NORMAL,
                   (HANDLE) NULL);
    if (hf == INVALID_HANDLE_VALUE) Msg ( "CreateFile" );
    hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M"
    // Compute the size of the entire file.
    hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) +
                 pbih->biSize + pbih->biClrUsed
                 * sizeof(RGBQUAD) + pbih->biSizeImage);
    bmpfilsiz = hdr.bfSize;                
    if ( d_bug > 1 )
      {      
        sprintf ( d_bugstr, "Sizes: Head: %d, Info: %d, RGBQUAD: %d, Image: %d.",                 
              sizeof(BITMAPFILEHEADER),
              pbih->biSize + pbih->biClrUsed,
              sizeof(RGBQUAD), pbih->biSizeImage );
        wrt_debug ();              
      }
    hdr.bfReserved1 = 0;
    hdr.bfReserved2 = 0;
    // Compute the offset to the array of colour indices.
    hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) +
                    pbih->biSize + pbih->biClrUsed
                    * sizeof (RGBQUAD);
    // Copy the BITMAPFILEHEADER into the .BMP file.
    if (!WriteFile(hf, (LPVOID) &hdr, sizeof(BITMAPFILEHEADER),
        (LPDWORD) &dwTmp,  NULL))
       Msg ( "WriteFile" );
// Copy the BITMAPINFOHEADER and RGBQUAD array into the file.
    if (!WriteFile(hf, (LPVOID) pbih, sizeof(BITMAPINFOHEADER)
                  + pbih->biClrUsed * sizeof (RGBQUAD),
                  (LPDWORD) &dwTmp, ( NULL)))
        Msg ( "WriteFile" );
    bmpfilhedsiz = (DWORD) sizeof (BITMAPFILEHEADER);
    bmppbisiz = sizeof(BITMAPINFOHEADER) + pbih->biClrUsed * sizeof (RGBQUAD);
    bmpbitssiz = pbih->biSizeImage;
    // Copy the array of colour indices into the .BMP file.
    dwTotal = cb = pbih->biSizeImage;
    if ( d_bug > 1 )
      {      
        sprintf ( d_bugstr, "Writing %d bytes.", (int) cb );
        wrt_debug ();
      }
    hp = lpBits;
    if (!WriteFile(hf, (LPSTR) hp, (int) cb, (LPDWORD) &dwTmp, NULL))
           Msg ( "WriteFile2" );

    // Close the .BMP file.
     if (!CloseHandle(hf)) Msg ( "CloseHandle fail" );

    // Free memory.
    GlobalFree((HGLOBAL)lpBits);
    // Free pbmi ????
  } // END - save_mempag ()

void read_mempag ( int x )
  {
    int xpt;
    HANDLE hf;
    unsigned i, j;
    char *bmp_buf;
    int Bytes;
    if ( pbmi )
      free ( pbmi );
    if ( bmp_buf )      
      free ( bmp_buf );  
    if ( d_bug > 1 )
      {        
        sprintf ( d_bugstr, "Reading file: %d", x );
        wrt_debug ();    
      }
    sprintf ( fnam, "temp%.3d.bmp", x );    
    bmp_buf = malloc (bmpfilsiz+1);
    if ( bmp_buf == NULL )
      {
        Msg ( "Read file memory allocation failed!" );
        return;
      }
      
    pbmi = (PBITMAPINFO) malloc (bmpfilsiz-bmpfilhedsiz);
    if ( pbmi == NULL )
      {
        Msg ( "Bitmap memory allocation failed!" );
        return;
      }      

    hf = CreateFile ( fnam, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( hf == INVALID_HANDLE_VALUE )
      Msg ( "Open file failed." );                  
    if ( FALSE == ReadFile( hf, bmp_buf, bmpfilsiz, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( hf );
        free ( pbmi );
        free ( bmp_buf );        
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, bmpfilsiz );
        wrt_debug (); 
        return;
      }
    if ( d_bug > 1 )
      {            
        sprintf ( d_bugstr, "Bytes read from file: %d", Bytes );
        wrt_debug ();  
      }  
    j = bmppbisiz+bmpbitssiz;
    for ( i = 0; i < j; ++i )
      *( (char*) pbmi+i) = *(bmp_buf+i+bmpfilhedsiz);
   
    if ( d_bug > 1 )
      {      
        sprintf ( d_bugstr, "Read, size: %d, wid: %d, h: %d, plan: %d, cnt: %d",
              pbmi->bmiHeader.biSize, pbmi->bmiHeader.biWidth, pbmi->bmiHeader.biHeight,
              pbmi->bmiHeader.biPlanes, pbmi->bmiHeader.biBitCount );
        wrt_debug ();
      }  

    free ( bmp_buf );  
    if ( pag_DC )
      DeleteDC(pag_DC);       
    pag_DC = CreateCompatibleDC (NULL);        
    if (pgBits)
      free ( pgBits );
    xpt = bmpbitssiz / pbmi->bmiHeader.biHeight * 8;
    pgxln = xpt / 8;
    pag_bmpHeight = pbmi->bmiHeader.biHeight;
    pgbufsiz = pgxln * pag_bmpHeight;
    pgBits = malloc ( pgbufsiz );        
    if ( pgBits == NULL )
      Msg ( "Memory allocation failed!" );

    for ( i = 0; i < pag_bmpHeight; ++i )
      for ( j = 0; j < pgxln; ++j )
        *(pgBits + i * pgxln + j ) = *( (char*) pbmi + bmppbisiz + pgxln * (pag_bmpHeight-i-1) + j );

    pag_bmp = CreateBitmap ( xpt, pag_bmpHeight, 1, 1, pgBits );
    SelectObject( pag_DC, pag_bmp );   
// Close the .BMP file.
     if (!CloseHandle(hf)) Msg ( "CloseHandle" );
  }
  