/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
/**** TF1.C ****/

#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>  
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <dos.h>
#include "tform.h"
#include "tibform.def"
#include "tibform.ids"

/**** SET MAIN PARAMETERS ****/

void set_pms ( void ) // Called immediately after printer selected.
  {
    intcspc = 1;     // Inter-character spacing - not over ridden
    intwspc = 2;     // Inter-word spacing - before/after tshegs
    strpnum = 1;     /** First Tibetan page number **/
    shadspc[0] = 24; /** Shad spacing ***/
    shadspc[1] = ( shadspc[0] * 2 ) / 3;
    bfrshad[0] = 3;  /** Before-Shad spacing ***/
    bfrshad[1] = 2;
    aftshad[0] = 5;  /** After-Shad spacing ***/
    aftshad[1] = 3;
    noprint = 0;
    if ( paper_size == A3_PAPER || paper_size == Tabloid_PAPER )
      linspc = 115;  
    else
      linspc = 100;     
    linespp = 6;    /** Number of lines per "pecha" page **/
    chrwd[0] = 48;  // Width of characters in dots 
    chrwd[1] = 32;  // Width of characters in dots 
    
    titrgtx = 0;
    frmtmgn = 17; /** Top and bottom margins for frames **/
    frmbmgn = 5;
    bas2top = 17; // Distance from top of character cell to Tibetan line
// The following could be in a configuration file:
    boxgap = 8;     // Make these two configurable
    boxbottomgap = 27; // For characters overlapping lower frame lines     
    titpageon = 0; // These two here to fix a bug
    titnumlines = 0;       
  } // END - set_pms
  
void main_print ( void )
  {    
    if ( paper_size == A4_PAPER ) // Portrait and landscape modes
      set_pms_a4 ();
    else if ( paper_size == A3_PAPER ) // Only landscape pecha mode
      set_pms_a3 ();
    else if ( paper_size == Tabloid_PAPER ) // Only landscape pecha mode
      set_pms_Tabloid ();      
    else if ( paper_size == LEGAL_PAPER )
      set_pms_legal ();    
    else if ( paper_size == LETTER_PAPER )
      set_pms_letter ();          
    maxqptr = 0; // Not really still needed; part of old debug code
    if ( !portprn )
      land_rout ();
    else
      port_rout ();   
    if ( d_bug )
      {
        sprintf ( d_bugstr, "Max Sanskrit string = %d", max_sktstr );
          wrt_debug ( );
      }
  } // END - main_print ()

void setpmstop ( void )
  {
    box.left = lndrct.left;
    box.right = lndrct.right;    
    box.top = midtop - boxhgt / 2;
    box.bottom = box.top + boxhgt;
    txtrct.left = box.left + 115; /** including a margin **/
    txtrct.right = box.right - 105;   
    txtrct.top = box.top; 
    txtrct.bottom = box.bottom;
    sidbr_l[0].left = box.left;
    sidbr_l[0].right = box.left + 100;
    sidbr_l[0].top = box.top; 
    sidbr_l[0].bottom = box.bottom;
    sidbr_r[0].left = box.right - 100;
    sidbr_r[0].right = box.right;
    sidbr_r[0].top = box.top;
    sidbr_r[0].bottom = box.bottom;    
    if ( d_bug == 6 )
      {
        sprintf ( d_bugstr, "setpmstop:" );      
        wrt_debug ();        
        sprintf ( d_bugstr, "box: left: %d, right: %d, top: %d, bottom: %d.",
                  box.left, box.right, box.top, box.bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "txtrct: left: %d, right: %d, top: %d, bottom: %d.",
                  txtrct.left, txtrct.right, txtrct.top, txtrct.bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "sidbr_l 0: left: %d, right: %d, top: %d, bottom: %d.",
                  sidbr_l[0].left, sidbr_l[0].right, sidbr_l[0].top, sidbr_l[0].bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "sidbr_r 0: left: %d, right: %d, top: %d, bottom: %d.",
                  sidbr_r[0].left, sidbr_r[0].right, sidbr_r[0].top, sidbr_r[0].bottom );
        wrt_debug ();        
      }        
  } // END - setpmstop ()

void setpmstit ( void )
  {
    int y;
    int x;
    y = bas2top + linspc * fstistit + frmtmgn + frmbmgn;
    x = txtrct.right - titrgtx;
    if ( d_bug == 6 ) 
      {
        sprintf ( d_bugstr, 
          "setpmstit, y: %d, linspc: %d, fstistit: %d, frmtmgn: %d, frmbmgn: %d.",
          y, linspc, fstistit, frmtmgn, frmbmgn );
        wrt_debug ( );
      }
    if ( x > 150 )
      {
        x = x - 150;
        titleft = lndrct.left + x / 2;
        titright = lndrct.right - x / 2;        
      }
    else
      {
        titleft = lndrct.left;
        titright = lndrct.right;        
      }
    box.top = midtop - y / 2;
    box.bottom = box.top + y + 6; // Added 6 just to ease it down a little.
    txtrct.left = titleft + 115;  // including a margin 
    txtrct.right = titright - 105;  
    txtrct.top = box.top;
    txtrct.bottom = box.bottom;
    sidbr_l[0].left = titleft; 
    sidbr_l[0].right = titleft + 100; 
    sidbr_l[0].top = box.top;
    sidbr_l[0].bottom = box.bottom;
    sidbr_r[0].left = titright - 100; 
    sidbr_r[0].right = titright; 
    sidbr_r[0].top = box.top;
    sidbr_r[0].bottom = box.bottom;    
    if ( d_bug == 6 ) 
      {
        sprintf ( d_bugstr, "setpmstit:" );
        wrt_debug ();        
        sprintf ( d_bugstr, "box: left: %d, right: %d, top: %d, bottom: %d.",
                  box.left, box.right, box.top, box.bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "txtrct: left: %d, right: %d, top: %d, bottom: %d.",
                  txtrct.left, txtrct.right, txtrct.top, txtrct.bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "sidbr_l 1: left: %d, right: %d, top: %d, bottom: %d.",
                  sidbr_l[0].left, sidbr_l[0].right, sidbr_l[0].top, sidbr_l[0].bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "sidbr_r 1: left: %d, right: %d, top: %d, bottom: %d.",
                  sidbr_r[0].left, sidbr_r[0].right, sidbr_r[0].top, sidbr_r[0].bottom );
        wrt_debug ();        
      }     
  } // END - setpmstit ()

void setpmsbot ( void )
  {
    box_b.left = lndrct.left;
    box_b.right = lndrct.right;    
    box_b.top = midbot - boxhgt / 2;
    box_b.bottom = box_b.top + boxhgt;
    txtrct.left = box_b.left + 115; /** including a margin **/
    txtrct.right = box_b.right - 105; 
    txtrct.top = box_b.top;
    txtrct.bottom = box.bottom;
    sidbr_l[1].left = box_b.left;
    sidbr_l[1].right = box_b.left + 100;
    sidbr_l[1].top = box_b.top;
    sidbr_l[1].bottom = box_b.bottom;
    sidbr_r[1].left = box_b.right - 100;
    sidbr_r[1].right = box_b.right;
    sidbr_r[1].top = box_b.top;
    sidbr_r[1].bottom = box_b.bottom;    
    if ( d_bug == 6 )
      {
        sprintf ( d_bugstr, "setpmsbot:" );      
        wrt_debug ();        
        sprintf ( d_bugstr, "box_b: left: %d, right: %d, top: %d, bottom: %d.",
                  box_b.left, box_b.right, box_b.top, box_b.bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "txtrct_b: left: %d, right: %d, top: %d, bottom: %d.",
                  txtrct_b.left, txtrct_b.right, txtrct_b.top, txtrct_b.bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "sidbr_l 1: left: %d, right: %d, top: %d, bottom: %d.",
                  sidbr_l[1].left, sidbr_l[1].right, sidbr_l[1].top, sidbr_l[1].bottom );
        wrt_debug ();
        sprintf ( d_bugstr, "sidbr_r 1: left: %d, right: %d, top: %d, bottom: %d.",
                  sidbr_r[1].left, sidbr_r[1].right, sidbr_r[1].top, sidbr_r[1].bottom );
        wrt_debug ();        
      }    
  } // END - setpmsbot
  
void sid_barst ( int p1, int f ) // f indicates recto (1) or verso (0) pages
  {
    int sx1, sx2, sy, z;
    z = 0;
    sidprn = 1;
    smlfnt = 1;
    sx1 = sidbr_l[0].top + 3;
    sx2 = sidbr_l[0].bottom - 3;
    sy = sidbr_l[0].left + 20;
/** UPPER PAGES ON EITHER LANDSCAPE SIDE, 0 AND 3 **/
    if ( f ) /** Page 0 **/
      {
        if ( lastpg == 0 || lastpg == 1 )
          z = 1;           /** Make sure 'byon' is added **/
        // sx1 and sx2 are x-coordinates from point of view of sidebar, y is similar          
        prn_num ( p1, sx1, sx2, sy, z ); // tf7.c
      }
    else if ( !skiptop && !page3blank ) /** Page 3 **/
      {
        prn_titb ( sx1, sx2, sy );
      }
    sidprn = 0;      
  } // END - sid_barst ()
  
void sid_barsb ( int p2, int f ) // f f indicates recto (1) or verso (0) pages
  {
    int sx1, sx2, sy, z;
    z = 0;
    sidprn = 1;
    smlfnt = 1;
    sx1 = sidbr_l[0].top + 3;
    sx2 = sidbr_l[0].bottom - 3;
    sy = sidbr_l[0].left + 20;
// LOWER PAGES ON EITHER LANDSCAPE SIDE, 2 AND 1 
    sx1 = sidbr_l[1].top + 3;
    sx2 = sidbr_l[1].bottom - 3;
    sy = sidbr_l[1].left + 20;

    if ( f && !skipbot ) /** Page 2 **/
      {
        if ( lastpg == 2 || lastpg == 3 )
          z = 1;           /** Make sure 'byon' is added **/
        prn_num ( p2, sx1, sx2, sy, z );
      }
    else if ( !skipbot && !page1blank ) /** Page 1 **/
      {
        prn_titb ( sx1, sx2, sy );
      }
    sidprn = 0;
  } // END - sid_barsb ()    

void an_pags ( void )
  {
    int pg, i, l2s, endfil;
    endfil = 0; /** End of file flag **/
    strtpg = 1;
    lastpg = 20;  /** Just to get it out of the way **/
    if ( curwpnum )
      {
        for ( i = 0; i < 4; ++i )
          pd[i].wstpnum = curwpnum++;
      }
    l2s = 0;
    obblck = 1; /** Allow adding more to obscure queue **/
    pg = 0;
    do
      {
        pd[pg].finalpg = 0; /** Final page **/
        tpnum = pg;
        obblck = 1; /** Allow adding more to obscure queue **/
        if ( pg == 0 || pg == 2 )
          strtpg = 1;
        pcurly = txtrct.top + bas2top + frmtmgn;
        obptr = 0;
        prvobptr = 0;
        if ( fstistit && pg == 1 )
          strtpg = 1;
        for ( i = 0; i < linespp; ++i )
          {
            tplinnum = i;
            if ( d_bug == 4 )
              {
                sprintf ( d_bugstr, "AN_PAGS: ENDFIL = %d, TITPEND = %d\n",
                        endfil, titpend );
                wrt_debug ();
              }
            if ( !endfil && !titpend )
              {
                nxtobj = 0;
                pd[pg].lindts[i].strtq = l2s;
                pd[pg].lindts[i].finq = anlslin ( l2s );
                strtpg = 0; /** Turn off page flag **/
                l2s = pd[pg].lindts[i].finq + 1; /** Next line, next word **/
                pcurly += linspc; /** line feed **/
                sort_obs ( pcurly );
                que_mxprn = pd[pg].lindts[i].finq;
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "AN_PAGS: Line %d, First = %d, Last = %d\n",
                         i, pd[pg].lindts[i].strtq, pd[pg].lindts[i].finq ); 
                    wrt_debug (); 
                  }
                if ( titpfnd )
                  fstistit += 1;
                if ( newpage )
                  i = linespp;
              }
            else
              {
                pd[pg].lindts[i].strtq = 0;
                pd[pg].lindts[i].finq = 0;
              }
            if ( lastpg > 4 && l2s == qu_ptr )
              {
                endfil = 1; /** This is first line after end of file **/
                pd[pg].finalpg = 1; /** Final page **/
                lastpg = pg;
                pd[pg].lastlin = i;
              }
          } // END - for ( i = 0; i < linespp; ++i )
        if ( fstistit )
          { // Clear the flags
            titpend = 0;
            titpfnd = 0;
          }
        if ( newpage )
          {
            if ( pg == 3 )
              pg = 4;
            else if ( pg == 2 )
              {
                pg = 4;
                page3blank = 1;
              }
            else if ( pg == 1 )
              pg = 2;
            else if ( pg == 0 )
              {
                pg = 2;
                page1blank = 1;
              }
            newpage = 0;
          }
        else
          ++pg;
      } while ( pg < 4 ); // END of page loop 
  }

/** Print a page, at given vertical position **/
void prn_pag ( int pg )
  {
    int i, endfil, dotit, kilit;
    pageinprogress = 1;
    tpnum = pg;
    if ( lastpg < pg )
      return;     // Check this, it's too simple !
    if ( pg == 0 || pg == 2 )
      strtpg = 1;
    else if ( titpon && pg == 1 )
      strtpg = 1;
    endfil = 0;   // Local variable only.
    dotit = 0;
    kilit = 0;
    pcurly = txtrct.top + bas2top + frmtmgn;
    mincurpos = txtrct.top;
    obblck = 1; /** Allow adding more to obscure queue **/
    obptr = 0;
    prvobptr = 0;
    for ( i = 0; i < linespp; ++i )
      {
        tplinnum = i;
        if ( pd[pg].lindts[i].finq == 0 )
          endfil = 1; /** This is first line after end of file **/
        if ( pg == 0 && titpon )
          dotit += 1;
        if ( !endfil && !kilit )  
          {
            nxtobj = 0;
            prn_nlin ( pd[pg].lindts[i].strtq, pd[pg].lindts[i].finq );
            if ( newpage )
              {
                i = linespp;
                newpage = 0;
              }
            strtpg = 0;
            pcurly += linspc; /** Effective line feed **/
            sort_obs ( pcurly );
            if ( dotit && dotit == fstistit )
              kilit = 1;
          }
      }
    sort_obs ( box.bottom + bas2top );
    pd[pg].optr = obptr;
    if ( obptr )
      {
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "COPYING %d elements, for page %d.\n", obptr, pg );
            wrt_debug ();
          }
        for ( i = 0; i < obptr; ++i )
          { // Do these ever get used now? 
            pd[pg].o_d[i].left = ob_det[i].left;
            pd[pg].o_d[i].right = ob_det[i].right;
            pd[pg].o_d[i].depth = ob_det[i].depth;
          }
      }
    pageinprogress = 0;
  } // END - prn_pag ()

void prn_boxt ( int sd ) // Print landscape mode frames - top pages
  {
// If sd == 0, pages are 0, 2; if sd == 1, pages are 3, 1 
// This is old code - method no longer needed
    int more, ptr, last_was_join; 
    int  x1, x2, y1, y2; 
    int upp; /** Upper pages flag **/
    if ( !doall ) // Flag for printing frames.
      return; 
    hor_lin ( sidbr_l[0].left, sidbr_l[0].top, sidbr_r[0].right, 3 );
    hor_lin ( sidbr_l[0].left, sidbr_l[0].bottom, sidbr_l[0].right, 3 );    
    hor_lin ( sidbr_r[0].left, sidbr_l[0].bottom, sidbr_r[0].right, 3 );    
// Left hand side bars:
    ver_lin ( sidbr_l[0].top, sidbr_l[0].bottom, sidbr_l[0].left, 3 );
    ver_lin ( sidbr_l[0].top, sidbr_l[0].bottom, sidbr_l[0].right, 3 );            
// Right hand side bars:
    ver_lin ( sidbr_r[0].top, sidbr_r[0].bottom, sidbr_r[0].left, 3 );
    ver_lin ( sidbr_r[0].top, sidbr_r[0].bottom, sidbr_r[0].right, 3 );        
    if ( sd == 0 )
      upp = 0;
    else if ( sd == 1 )
      upp = 3;
    if ( !pd[upp].optr ) 
      hor_lin ( sidbr_l[0].left, sidbr_l[0].bottom, sidbr_r[0].right, 3 );   
    else
      { 
        more = 1;
        ptr = 0;
        x1 = box.left + 100;        
        y1 = sidbr_l[0].bottom;    
        last_was_join = 0;    
        do
          {
            if ( d_bug == 6 )
              {
                sprintf ( d_bugstr, "Upper, Ptr: %d, Left: %d, Bottom: %d, Right: %d.",
                  ptr, pd[upp].o_d[ptr].left, pd[upp].o_d[ptr].depth, pd[upp].o_d[ptr].right );
                wrt_debug ();        
              }                    
            x2 = pd[upp].o_d[ptr].left - boxgap;
            if ( x2 < box.left + 100 ) // In case we are right at beginning of line
              x2 = box.left + 100 + 2;
            if ( last_was_join )
              last_was_join = 0;                            
            else
              hor_lin ( x1, y1, x2, 3 );   
            y2 = pd[upp].o_d[ptr].depth + boxbottomgap;
            if ( y2 < sidbr_l[0].bottom ) // To fix a bug on test pages
              y2 = sidbr_l[0].bottom;
            ver_lin ( y1, y2, x2, 3 );              
            x1 = x2;
            x2 = pd[upp].o_d[ptr].right + boxgap;
            if ( x2 > sidbr_r[0].right ) // In case we are right at end of line
              x2 = sidbr_r[0].right - 2;            
           
            if ( ptr + 1 < pd[upp].optr && pd[upp].o_d[ptr+1].left < x2 + 30 )
              {
                if ( d_bug == 6 )
                  {
                    sprintf ( d_bugstr, "Joining:" );
                    wrt_debug ();        
                  }                    
                x2 = pd[upp].o_d[ptr+1].left - boxgap;
                hor_lin ( x1, y2, x2, 3 ); 
                y1 = y2;
                last_was_join = 1;
              }
            else
              {
                y1 = sidbr_l[0].bottom;              
                hor_lin ( x1, y2, x2, 3 ); 
                ver_lin ( y1, y2, x2, 3 );                 
              }
            x1 = x2;
            ++ptr;
            if ( ptr >= pd[upp].optr )
              more = 0;
          } while ( more );  
        hor_lin ( x1, sidbr_l[0].bottom, sidbr_r[0].right, 3 );               
      }
  } // END prn_boxt ()

void prn_boxb ( int sd ) // Print landscape mode frames - bottom pages
  {
// If sd == 0, pages are 0, 2; if sd == 1, pages are 3, 1 
    int more, ptr, last_was_join; 
    int  x1, x2, y1, y2; 
    int lwp; // Lower page flag
    if ( !doall ) // Flag for printing frames.
      return; 
    hor_lin ( sidbr_l[1].left, sidbr_l[1].top, sidbr_r[1].right, 3 );
    hor_lin ( sidbr_l[1].left, sidbr_l[1].bottom, sidbr_l[1].right, 3 );    
    hor_lin ( sidbr_r[1].left, sidbr_l[1].bottom, sidbr_r[1].right, 3 );        
// Left hand side bars:
    ver_lin ( sidbr_l[1].top, sidbr_l[1].bottom, sidbr_l[1].left, 3 );
    ver_lin ( sidbr_l[1].top, sidbr_l[1].bottom, sidbr_l[1].right, 3 );              
// Right hand side bars:    
    ver_lin ( sidbr_r[1].top, sidbr_r[1].bottom, sidbr_r[1].left, 3 );
    ver_lin ( sidbr_r[1].top, sidbr_r[1].bottom, sidbr_r[1].right, 3 );            
    if ( sd == 0 )
      lwp = 2;
    else if ( sd == 1 )
      lwp = 1;
    if ( !pd[lwp].optr ) 
      hor_lin ( sidbr_l[1].left, sidbr_l[1].bottom, sidbr_r[1].right, 3 );   
    else 
      { 
        more = 1;
        ptr = 0;
        x1 = box.left + 100;        
        y1 = sidbr_l[1].bottom;    
        last_was_join = 0;    
        do
          {      
            if ( d_bug == 6 )
              {
                sprintf ( d_bugstr, "Bottom, Ptr: %d, Left: %d, Bottom: %d, Right: %d.",
                  ptr, pd[lwp].o_d[ptr].left, pd[lwp].o_d[ptr].depth, pd[lwp].o_d[ptr].right );
                wrt_debug ();        
              }                    
            x2 = pd[lwp].o_d[ptr].left - boxgap;
            if ( x2 < box.left + 100 ) // In case we are right at beginning of line
              x2 = box.left + 100 + 2;
            if ( last_was_join )
              last_was_join = 0;                            
            else
              hor_lin ( x1, y1, x2, 3 );   
            y2 = pd[lwp].o_d[ptr].depth + boxbottomgap;
            if ( y2 < sidbr_l[1].bottom ) // To fix a bug on test pages
              y2 = sidbr_l[1].bottom;            
            ver_lin ( y1, y2, x2, 3 );              
            x1 = x2;
            x2 = pd[lwp].o_d[ptr].right + boxgap;
            if ( x2 > box.right - 100 ) // In case we are right at end of line
              x2 = box.right - 100 - 2;            
            if ( ptr + 1 < pd[lwp].optr && pd[lwp].o_d[ptr+1].left < x2 + 30 )
              {
                if ( d_bug == 6 )
                  {
                    sprintf ( d_bugstr, "Joining:" );
                    wrt_debug ();        
                  }                    
                x2 = pd[lwp].o_d[ptr+1].left - boxgap;
                hor_lin ( x1, y2, x2, 3 ); 
                y1 = y2;
                last_was_join = 1;
              }
            else
              {
                y1 = sidbr_l[1].bottom;              
                hor_lin ( x1, y2, x2, 3 ); 
                ver_lin ( y1, y2, x2, 3 );                 
              }
            x1 = x2;
            ++ptr;
            if ( ptr >= pd[lwp].optr )
              more = 0;
          } while ( more );       
        hor_lin ( x1, sidbr_l[1].bottom, sidbr_r[1].right, 3 );               
      }    
  } // END prn_boxb ()

void set_pms_a4 ( void )
    { // 3507x2480
      porrct.left = 250 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
// Assume no justification in portrait mode:
      porrct.right = porrct.left + 2130; // was 2050 -> 2480-250-100
      porrct.top = 45 - (int) ( (float) pr_physicaloffsety / pr_scaley );
      porrct.bottom = porrct.top + 3275; 
// Total length of A4 pecha frames = 3164 
// A4 page length = 3507, so we want margin of 171 each end.
// Each pecha box is 640 in height, with 600 between them.
      lndrct.left = 171 - (int) ( (float) pr_physicaloffsety / pr_scaley );        
      lndrct.right = lndrct.left + 3164; 
      lndrct.top = 300 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
      lndrct.bottom = lndrct.top + 1880; // Changed - is this ever used ???
      if ( !portprn ) // Landscape, if A4 pecha
        { 
// Each page box is 640 in height, with 600 between them.
          midtop = lndrct.top + 320; 
          midbot = lndrct.top + 1560;      
          boxhgt = 640;
          paper_margin = 200; // Distance of trim marks from page frames 
          if ( prev_on ) // Preview mode flag
            {
              lndrct.left = 171; // - (int) ( (float) pr_physicaloffsety / pr_scaley );    
              lndrct.right = lndrct.left + 3164;
              pag_bmpWidth = 1040; // 640+200+200
              pag_bmpHeight = 3507; 
              lndrct.top = paper_margin; // 200
              midtop = lndrct.top + 320; // 640/2
              lndrct.bottom = lndrct.top + 640;    
              midbot = midtop;                 
            }                          
        }        
      else // For portrait A4 printing
        {
          txtrct.left = 250;
          txtrct.right = porrct.left + 2130;
          txtrct.top = 195; // was +120
          txtrct.bottom = porrct.top + 3275;               
          if ( prev_on ) // Preview mode flag
            {
              pag_bmpWidth = 2480;
              pag_bmpHeight = 3507;        
            }                  
        }
    } // END - set_pms_a4 ()

void set_pms_a3 ( void )
    { // 4960.5x3507.5
// boxhgt: 730 == 61.8 mms. Given greater line spacing than A4
// Frame width = 4335 and centre it.
// A3 page width = 3507.5, length = 4960.5 - border either end => 313, so:
      lndrct.left = 313 - (int) ( (float) pr_physicaloffsety / pr_scaley );    
      lndrct.right = lndrct.left + 4335;
// For box heights, 3507.5 - 2x730 => 2047.5 div. 4 = 511.875,  So:     
      lndrct.top = 512 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
      lndrct.bottom = lndrct.top + 2484; 
      boxhgt = 730;
      midtop = lndrct.top + 365; // 730/2 
      midbot = lndrct.top + 2119; // 730 + 2x511.875 + 730/2   
      paper_margin = 200; // Distance of trim marks from page frames                      
      if ( prev_on ) // Preview mode flag
        {
          lndrct.left = 313;     
          lndrct.right = lndrct.left + 4335; // 4961-313-313
          pag_bmpWidth = 1130;  // 730 + 200 + 200
          pag_bmpHeight = 4960;           
          lndrct.top = paper_margin; // 200
          midtop = lndrct.top + 365; // 730/2
          lndrct.bottom = lndrct.top + 730;    
          midbot = midtop;                 
        }                
   } // END - set_pms_a3 ()
   
void set_pms_Tabloid ( void ) // 17x11
  { // 5100x3300
// boxhgt: 730 == 61.8 mms. Greater line spacing than A4
// Frame width = 4380 = 4365 and centre it.
      lndrct.left = 360 - (int) ( (float) pr_physicaloffsety / pr_scaley );    
      lndrct.right = lndrct.left + 4380;
// 3300 - 2x730 => 1840 div. 4 = 460, so:  
      lndrct.top = 460 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
      lndrct.bottom = lndrct.top + 2380; // 2x730 + 2x460
      boxhgt = 730;
      midtop = lndrct.top + 365; // 730/2 
      midbot = lndrct.top + 2015; // 730 + 2x460 + 730/2   
      paper_margin = 200; // Distance of trim marks from page frames   
      if ( prev_on ) // Preview mode flag
        {
          lndrct.left = 313;      
          lndrct.right = lndrct.left + 4474; // 5100-313-313
          pag_bmpWidth = 1130;  // 730 + 200 + 200
          pag_bmpHeight = 5100;           
          lndrct.top = paper_margin; // 200
          midtop = lndrct.top + 365; // 730/2
          lndrct.bottom = lndrct.top + 730;    
          midbot = midtop;                 
        }                                       
  } // END - set_pms_11X17 ()
     
void set_pms_legal ( void )      
    { // 4200x2550
      porrct.left = 250 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
// Assume no justification in portrait mode:      
      porrct.right = porrct.left + 2200; // 2550-250-100
      porrct.top = 45 - (int) ( (float) pr_physicaloffsety / pr_scaley );
      porrct.bottom = porrct.top + 3968; 
// Total length of Legal pecha frames = 3800
// Legal page length = 4200, so we want margin of 200 each end.
// Each pecha box is 640 in height, with 2x317.5 between them.
      lndrct.left = 200 - (int) ( (float) pr_physicaloffsety / pr_scaley );        
      lndrct.right = lndrct.left + 4661; 
      lndrct.top = 317 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
      lndrct.bottom = lndrct.top + 1915; 
      if ( !portprn ) // Landscape, if Legal pecha
        { 
// Each page box is 640 in height, with 635 between them.
          midtop = lndrct.top + 320; 
          midbot = lndrct.top + 1595;
          boxhgt = 640;
          paper_margin = 200; // Distance of trim marks from page frames   
          if ( prev_on ) // Preview mode flag
            {
              lndrct.left = 200; 
              lndrct.right = lndrct.left + 3800; // 4200-200-200
              pag_bmpWidth = 1040; // 640 + 200 + 200
              pag_bmpHeight = 4200; 
              lndrct.top = paper_margin; // 200
              midtop = lndrct.top + 320; // 640/2
              lndrct.bottom = lndrct.top + 640;    
              midbot = midtop;                 
            }                                             
        }        
      else // For portrait Legal printing 
        {
          txtrct.left = 250; 
          txtrct.right = porrct.left + 2200; 
          txtrct.top = 165; 
          txtrct.bottom = porrct.top + 3968;   
          if ( prev_on ) // Preview mode flag
            {
              pag_bmpWidth = 2550;
              pag_bmpHeight = 4200;        
            }                                     
        }      
   } // END - set_pms_legal ()     
   
void set_pms_letter ( void )
    { // 3300x2550
// Legal page width = 2550    
      porrct.left = 250 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
// Assume no justification in portrait mode:      
      porrct.right = porrct.left + 2200; // 2550-250-100
      porrct.top = 45 - (int) ( (float) pr_physicaloffsety / pr_scaley );
      porrct.bottom = porrct.top + 3068; 
// Total length of Letter pecha frames = 2960 
// Letter page length = 3300, so we want margin of 170 each end.
// Each pecha box is 640 in height, with 2x317.5 between them.
      lndrct.left = 170 - (int) ( (float) pr_physicaloffsety / pr_scaley );        
      lndrct.right = lndrct.left + 2960; 
      lndrct.top = 317 - (int) ( (float) pr_physicaloffsetx / pr_scalex );
      lndrct.bottom = lndrct.top + 1915; 
      if ( !portprn ) // Landscape, if Letter pecha
        { 
// Each page box is 640 in height, with 600 between them.
          midtop = lndrct.top + 320; 
          midbot = lndrct.top + 1595;      
          boxhgt = 640;
          paper_margin = 200; // Distance of trim marks from page frames            
          if ( prev_on ) // Preview mode flag
            {
              lndrct.left = 170; // - (int) ( (float) pr_physicaloffsety / pr_scaley );    
              lndrct.right = lndrct.left + 2960; // 3300-170-170
              pag_bmpWidth = 1040; // 640+200+200
              pag_bmpHeight = 3300; 
              lndrct.top = paper_margin; // 200
              midtop = lndrct.top + 320; // 640/2
              lndrct.bottom = lndrct.top + 640;    
              midbot = midtop;                 
            }                                    
        }        
      else // For portrait Letter printing
        {
          txtrct.left = 250; 
          txtrct.right = porrct.left + 2200; 
          txtrct.top = 165; 
          txtrct.bottom = porrct.top + 3068;   
          if ( prev_on ) // Preview mode flag
            {
              pag_bmpWidth = 2550;
              pag_bmpHeight = 3300;        
            }                                    
        }
    } // END - set_pms_letter ()
    