/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
// Main word parsing routines

#include <io.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <dos.h>
#include "tform.h"
#include "tibform.ext"
#include "tibform.ids"

// Locals:

int     vwlsl[2], vwlsr[2]; // Left and right spacing for plain vowel 
int     dotsl[2], dotsr[2]; // Left and right spacing for dot 
int     shdsl[2], shdsr[2]; // Left and right spacing for 'shad'
int     consl[2], consr[2]; // Left and right for cont-shad, bsdus rtags.

FILE *dbg_fil = NULL;

char vwlsgn; /* Vowel, if needed - actual sign of vowel **/
char vwlltr; /* Vowel character, 'a, l, r **/
char vwlltr2; /* Vowel character, 'a, l, r **/
char con[10]; /* Consonants required **/
int conshft[10];  // Array for character centre shifts.
int vwl1shft;     // Same for vowels.
int vwl2shft;
int vwlsgnshft;
int numcon; /* Number of consonants */
int skthld, sktptr;      
int anusvar; /* Anusvara, if needed */ 
int visarg;                            
int ignore_tsheg = 0;
int dwn;

/*** ROUTINE TO DETERMINE SPACING FOR PREFIX, ROOT AND SUFFIXES ***/

void getbasspc ( void )
  {
    int l, r;
    if ( prf )
      getchrspc ( prf, &prfsl, &prfsr );
    getchrspc ( rot, &rotsl, &rotsr );
    if ( sup )
      {
        getchrspc ( sup, &l, &r );
        if ( l < rotsl )
          rotsl = l;
        if ( r < rotsr )
          rotsr = r;
      }
    if ( sub )
      {
        getchrspc ( sub, &l, &r );
        if ( l < rotsl )
          rotsl = l;
        if ( r < rotsr )
          rotsr = r;
      }
    if ( suf1 )
      getchrspc ( suf1, &suf1sl, &suf1sr );
    if ( suf2 )
      getchrspc ( suf2, &suf2sl, &suf2sr );
  }

/*** ROUTINE TO DETERMINE GLOBAL SPACINGS ***/

void getglbspc ( void )
  {
    smlfnt = 1;
    getchrspc ( ID_NOR_A, &vwlsl[smlfnt], &vwlsr[smlfnt] ); /** Plain vowel **/
    getchrspc ( ID_SHAD, &shdsl[smlfnt], &shdsr[smlfnt] ); /** Shad **/
    getchrspc ( ID_DOT, &dotsl[smlfnt], &dotsr[smlfnt] ); /** Dot **/
    getchrspc ( ID_CONTSH, &consl[smlfnt], &consr[smlfnt] ); // Cont shad
    smlfnt = 0;
    getchrspc ( ID_NOR_A, &vwlsl[smlfnt], &vwlsr[smlfnt] ); /** Plain vowel **/
    getchrspc ( ID_SHAD, &shdsl[smlfnt], &shdsr[smlfnt] ); /** Shad **/
    getchrspc ( ID_DOT, &dotsl[smlfnt], &dotsr[smlfnt] ); /** Dot **/
    getchrspc ( ID_CONTSH, &consl[smlfnt], &consr[smlfnt] ); // Cont shad
  }

// Get the ID for the root when there is a superjoin:
int gsup_chr ( char chr )
  {
    int j;
    j = -1;
    switch ( chr )
      {
        case ID_NOR_K:  j = ID_SQU_K; break;
        case ID_NOR_G:  j = ID_SQU_G; break;
        case ID_NOR_NG: j = ID_SQU_NG; break;
        case ID_NOR_C:  j = ID_SQU_C; break;
        case ID_NOR_J:  j = ID_RSP_J; break;
        case ID_NOR_NY: j = ID_RSP_NY; break;
        case ID_NOR_T:  j = ID_SQU_T; break;
        case ID_NOR_TH: j = ID_SQU_TH; break;
        case ID_NOR_D:  j = ID_RSP_D; break;
        case ID_NOR_N:  j = ID_SQU_N; break;
        case ID_NOR_P:  j = ID_SQU_P; break;
        case ID_NOR_PH: j = ID_SQU_PH; break;
        case ID_NOR_B:  j = ID_SQU_B; break;
        case ID_NOR_M:  j = ID_SQU_M; break;
//      case ID_NOR_TS: j = ID_SQU_TS; break;   // CHECK - 9/9/97
        case ID_NOR_TSH: j = ID_SQU_TSH; break;
        case ID_NOR_DZ: j = ID_SQU_DZ; break;
        case ID_NOR_H:  j = ID_SQU_H; break;
        default: j = chr; break;
      }
    return ( j );
  }

int sub_chr ( char chr )
  {
    int    j;
    switch ( chr )
      {
        case ID_NOR_K:  j = ID_RSB_K; break;
        case ID_NOR_KH: j = ID_NOR_KH; break;
        case ID_NOR_G:  j = ID_RSB_G; break;
        case ID_NOR_NG: j = ID_NOR_NG; break;
        case ID_NOR_C:  j = ID_NOR_C; break;
        case ID_NOR_CH: j = ID_NOR_CH; break;
        case ID_NOR_J:  j = ID_NOR_J; break;
        case ID_NOR_NY: j = ID_NOR_NY; break;
        case ID_NOR_T:  j = ID_NOR_T; break;
        case ID_NOR_TH: j = ID_NOR_TH; break;
        case ID_NOR_D:  j = ID_RSB_D; break;
        case ID_NOR_N:  j = ID_RSB_N; break;
        case ID_NOR_P:  j = ID_NOR_P; break;
        case ID_NOR_PH: j = ID_NOR_PH; break;
        case ID_NOR_B:  j = ID_NOR_B; break;
        case ID_NOR_M:  j = ID_NOR_M; break;
        case ID_NOR_TS: j = ID_NOR_TS; break;
        case ID_NOR_TSH: j = ID_NOR_TSH; break;
        case ID_NOR_DZ: j = ID_NOR_DZ; break;
        case ID_NOR_W:  j = ID_NOR_W; break;
        case ID_NOR_ZH: j = ID_RSB_ZH; break;
        case ID_NOR_Z:  j = ID_NOR_Z; break;
        case ID_NOR_R:  j = ID_NOR_R; break;
        case ID_NOR_L:  j = ID_NOR_L; break;
        case ID_NOR_SH: j = ID_RSB_SH; break;
        case ID_NOR_S:  j = ID_NOR_S; break;
        case ID_NOR_H:  j = ID_SQU_H; break;
        default: j = 0; break;
      }
    return ( j );
  }

int gssp_chr ( char chr )
  {
    int    j;
    switch ( chr )
      {
        case ID_NOR_K:  j = ID_SQU_K; break;
        case ID_NOR_G:  j = ID_SQU_G; break;
        case ID_NOR_T:  j = ID_SQU_T; break;
        case ID_NOR_TH: j = ID_SQU_TH; break;
        case ID_NOR_D:  j = ID_SQU_D; break;
        case ID_NOR_N:  j = ID_SQU_N; break;
        case ID_NOR_P:  j = ID_SQU_P; break;
        case ID_NOR_B:  j = ID_SQU_B; break;
        case ID_NOR_M:  j = ID_SQU_M; break;
        case ID_NOR_TS: j = ID_SQU_TS; break;
        case ID_NOR_H:  j = ID_SQU_H; break;
        default: j = 0; break;
      }
    return ( j );
  }

/**** ROUTINE TO RETURN CHR ID FOR SQUASHED SANSKRIT CHARACTER ****/
int squsktid ( int n ) /** SKT **/
  {
    int j;
    switch ( n )
      {
        case ID_NOR_K: j = ID_SQU_K; break;    
        case ID_NOR_KH: j = ID_SQU_KH; break; 
        case ID_NOR_G: j = ID_SQU_G; break;     
        case ID_NOR_GH: j = ID_SQU_GH; break;   
        case ID_NOR_NG: j = ID_SQU_NG; break;  
        case ID_NOR_C: j = ID_SQU_C; break;    
        case ID_NOR_CH: j = ID_SQU_CH; break;   
        case ID_NOR_J: j = ID_SQU_J; break;    
        case ID_NOR_NY: j = ID_SQU_NY; break;   
        case ID_NOR_TR: j = ID_SQU_TR; break;   
        case ID_NOR_TRH: j = ID_SQU_TRH; break;
        case ID_NOR_DR: j = ID_SQU_DR; break;   
        case ID_NOR_DRH: j = ID_SQU_DRH; break; 
        case ID_NOR_NR: j = ID_SQU_NR; break;  
        case ID_NOR_T: j = ID_SQU_T; break;     
        case ID_NOR_TH: j = ID_SQU_TH; break;  
        case ID_NOR_D: j = ID_SQU_D; break;    
        case ID_NOR_DH: j = ID_SQU_DH; break;   
        case ID_NOR_N: j = ID_SQU_N; break;     
        case ID_NOR_P: j = ID_SQU_P; break;    
        case ID_NOR_PH: j = ID_SQU_PH; break;   
        case ID_NOR_B: j = ID_SQU_B; break;     
        case ID_NOR_BH: j = ID_SQU_BH; break;   
        case ID_NOR_M: j = ID_SQU_M; break;    
        case ID_NOR_TS: j = ID_SQU_TS; break;  
        case ID_NOR_TSH: j = ID_SQU_TSH; break; 
        case ID_NOR_DZ: j = ID_SQU_DZ; break;   
        case ID_NOR_DZH: j = ID_SQU_DZH; break; 
        case ID_NOR_A: j = ID_SQU_A; break;   
        case ID_NOR_Y: j = ID_SQU_Y; break;  
        case ID_NOR_R: j = ID_SQU_R; break;    
        case ID_NOR_L: j = ID_SQU_L; break;    
        case ID_NOR_ZH: j = ID_SQU_ZH; break;   
        case ID_NOR_V: j = ID_SQU_V; break;    
//      case ID_NOR_W: j = ID_SQU_V; break;    
        case ID_NOR_SH: j = ID_SQU_SH; break;   
        case ID_SKT_SZH: j = ID_SQU_SZH; break;
        case ID_NOR_S: j = ID_SQU_S; break;    
        case ID_NOR_H: j = ID_SQU_H; break;    
        case ID_SKT_LI: j = ID_SQU_L; break;    
        case ID_SKT_R: j = ID_SQU_R; break;     
        case ID_SKT_HR: j = ID_SQU_HR; break;   
        case ID_SKT_KZH: j = ID_SQU_KZ; break;  
        case ID_SKT_Q: j = ID_SKT_QS; break;
        default: j = n; break;
      }
    return ( j );
  }

/** SLIDE SANSKRIT SYLLABLE **/
void sldsktsyl ( int i ) /** SKT **/
  {
    int  x;
    // if numcon == 4
    // if i < 2, say 1
    if ( i < numcon - 1 )  // CHANGED FROM - 2, 9/5/94
      {
        // for  x = 1, x < 3
        for ( x = i; x < numcon - 1; ++x )
          // con[1] = con[2]
          con[x] = con[x+1];
      }
    --numcon;
  }

int smlconv ( int x )
{
  if ( smlfnt )
    x = ( ( x * 2 ) + 1 ) / 3;
  return ( x );
}

int vwlshft ( void ) // ONLY CALLED TO ADJUST ID_SQU_A
  {
    if ( con[0] == ID_SKT_SZH && numcon == 1 )
      return ( smlconv ( -3 ) );
    if ( con[0] == ID_SQU_N && numcon == 1 )
      return ( smlconv ( 0 ) );
    return ( smlconv ( 6 ) );  // THIS NEEDS A LOT OF WORK.
  }

// CHECK FOR SPECIAL CASE SANSKRIT ALIGNMENTS
int chk_sktshift ( int i, int c )
  {
  if ( i == 0 )
    {
      if ( con[0] == ID_NOR_SH && numcon == 1 )
        {
          return ( c - smlconv ( 3 ) ); // MOVE TO LEFT
          // THIS DOES NOT WORK !!
        }
      return ( c );
    }
  else if ( i == 1 )
    {
      switch ( con[i] )
        {
          case ID_SQU_TS:
// kca ( wkcitta )
            if ( con[0] == ID_SQU_K )
              return ( conshft[0] + smlconv ( 9 ) );  // MOVE TO RIGHT
// ca
            else if ( con[0] == ID_SQU_NY )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
// rca, rtsa
            else if ( con[0] == ID_SUP_R )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
// ca
            else if ( con[0] == ID_SQU_SZH )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
// c
            else if ( con[0] == ID_SQU_SH )  // THIS MAY NOT BE NEEDED
              return ( conshft[0] + smlconv ( 9 ) );  // MOVE TO RIGHT
// cc
            else if ( con[0] == ID_SQU_TS )
              return ( conshft[0] + smlconv ( 6 ) );  // MOVE TO RIGHT
            break;
          case ID_SLO_TS: // NUMCON == 2
// cc
            if ( con[0] == ID_SQU_TS )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
// c
            else if ( con[0] == ID_SQU_SH )
              return ( conshft[0] + smlconv ( 6 ) );  // MOVE TO RIGHT
            break;
          case ID_SLO_TSH: // NUMCON == 2
// cch, chch
            if ( con[0] == ID_SQU_TS || con[0] == ID_SQU_TSH )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
            break;
          case ID_SQU_TSH:
// cch, chch
            if ( con[0] == ID_SQU_TS || con[0] == ID_SQU_TSH )
              return ( conshft[0] + smlconv ( 6 ) );  // MOVE TO RIGHT
            break;
          case ID_SQU_DZ:
// rj
            if ( con[0] == ID_SUP_R )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
// ja
            else if ( con[0] == ID_SQU_NY )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
// jja
            else if ( con[0] == ID_SQU_DZ )
              return ( conshft[0] + smlconv ( 6 ) );  // MOVE TO RIGHT
            break;
          case ID_RSP_DR:
// n
            if ( con[0] == ID_SQU_N )
                return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
            break;
          case ID_SQU_NR:
// h
            if ( con[0] == ID_SQU_H )
                return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
            break;
          case ID_SKT_TR:
// ntr
            if ( con[0] == ID_SQU_N )
                return ( conshft[0] + smlconv ( 6 ) );  // MOVE TO RIGHT
            break;
          case ID_SQU_TRH:
// ha
            if ( con[0] == ID_SQU_SH )
                return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT

          case ID_SQU_T:
// nt
            if ( con[0] == ID_SQU_N )
              return ( conshft[0] + smlconv ( 6 ) );  // MOVE TO RIGHT
            break;
          case ID_SQU_TH:
// rth
            if ( con[0] == ID_SUP_R )
              return ( conshft[0] - smlconv ( 6 ) );  // MOVE TO LEFT
            break;
          case ID_NOR_TH:
// rth
            if ( con[0] == ID_SUP_R )
              return ( conshft[0] - smlconv ( 6 ) );  // MOVE TO LEFT
            break;
          case ID_SQU_DH:
// ddh
            if ( con[0] == ID_SQU_D )
              return ( conshft[0] + smlconv ( 3 ) );  // MOVE TO RIGHT
// ndh
            else if ( con[0] == ID_SQU_N )
              return ( conshft[0] + smlconv ( 6 ) );  // MOVE TO RIGHT
            break;
          case ID_SQU_N:
// rn
            if ( con[0] == ID_SUP_R )
              return ( conshft[0] - smlconv ( 6 ) );  // MOVE TO LEFT
// hn
            else if ( con[0] == ID_SQU_H )
              return ( conshft[0] - smlconv ( 6 ) );  // MOVE TO LEFT
// ghn
            else if ( con[0] == ID_SQU_GH )
              return ( conshft[0] - smlconv ( 6 ) );  // MOVE TO LEFT
            break;
          case ID_SQU_B:
// rb
            if ( con[0] == ID_SUP_R )
              return ( conshft[0] - smlconv ( 3 ) );  // MOVE TO LEFT
            break;
          case ID_SUB_W:
// hve, etc.
            if ( con[0] == ID_NOR_H ) // 14/9/97
              return ( conshft[0] - smlconv ( 12 ) ); // LEFT
// dhva, etc.
            else if ( con[0] == ID_SQU_DH ) // 14/9/97
              return ( conshft[0] - smlconv ( 9 ) ); // LEFT
            break;
          case ID_SQU_HR:
// dhr
            if ( con[0] == ID_SQU_D )
              return ( conshft[0] + smlconv ( 6 ) );  // RIGHT
// ghr
            else if ( con[0] == ID_SQU_G )
              return ( conshft[0] + smlconv ( 6 ) );  // RIGHT
            break;
          default:
            return ( conshft[0] );
            break;
        }
    } // END - else if ( i == 1 )
  else
    {
      if ( con[i] == ID_SUB_R )
        return ( conshft[i-1] );
      else if ( con[i] == ID_SUB_Y )
        return ( conshft[i-1] );
      else if ( con[i] == ID_SUB_W )
        {
          if ( con[i-1] == ID_NOR_DH )
            return ( conshft[i-1] - smlconv ( 12 ) );
           else if ( con[i-1] == ID_SQU_DH )
             return ( conshft[i-1] - smlconv ( 10 ) );
          else if ( con[i-1] == ID_SKT_TR )
            return ( conshft[i-1] - smlconv ( 10 ) );
          else
            return ( conshft[i-1] );
        }
      else if ( con[i] == ID_SQU_M && con[i-1] == ID_SUP_R )
        return ( conshft[i-1] + smlconv ( 3 ) );
      else if ( con[i] == ID_SQU_TSH && con[i-1] == ID_SQU_TS )
        return ( conshft[i-1] + smlconv ( 7 ) );
      else if ( con[i] == ID_SQU_TS || con[i] == ID_SQU_TSH || con[i] ==
                          ID_SQU_DZ )
        {
          if ( con[i-1] == ID_SQU_NY )
            return ( conshft[i-1] + smlconv ( 5 ) );
        }
    }
  return ( conshft[i-1] );
  } // END - chk_sktshift ( int i, int c )

void fixskt ( void ) // FIX SOME SANSKRIT PROBLEMS - MAINLY FOR NEATNESS
  {
// THIS ROUTINE IS IMMEDIATELY BEFORE PRINTING - AFTER CHK_SPCSKT AND SQUASHING
// FIRST, UNSQUASHING FOR SIMPLE COMPOUNDS
    if ( d_bug == 5 )
      {
        sprintf ( d_bugstr,
                  "VWLSGN = %s, VWLLTR = %s, VWLLTR2 = %s NUMCON = %d",
                  chrs[(int)vwlsgn], chrs[(int)vwlltr], chrs[(int)vwlltr2], numcon );
        wrt_debug ( );
      }
    if ( numcon == 1 )
      {
        if ( con[0] == ID_SKT_HY )
          {
            if ( strcmp ( nxt_wrd1, "hy" ) == 0 && !vwlltr )
              {
                con[0] = ID_SQU_H;
                con[1] = ID_SQU_Y;
                numcon = 2;
                return;
              }
            else if ( strcmp ( lst_wrd, "hya" ) == 0 && vwlltr == ID_SQU_A )
              {
                con[0] = ID_SQU_H;
                con[1] = ID_SQU_Y;
                numcon = 2;
                return;
              }
          }
        else if ( vwlsgn != ID_VOW_U )
          {
            switch ( con[0] )
              {
                case ID_SQU_TS: con[0] = ID_NOR_TS; break; // 27/9/97
                case ID_SQU_DZ: con[0] = ID_NOR_DZ; break; // 27/9/97
                case ID_SQU_TH: con[0] = ID_NOR_TH; break; // 27/9/97
                case ID_SQU_D: con[0] = ID_NOR_D; break;
                case ID_SQU_N: con[0] = ID_NOR_N; break; // 27/9/97
                case ID_SQU_DR: con[0] = ID_RSP_DR; break;
                case ID_SQU_P: con[0] = ID_NOR_P; break;
                case ID_SQU_B: con[0] = ID_NOR_B; break;
                case ID_SQU_M: con[0] = ID_NOR_M; break;
                case ID_SQU_W: con[0] = ID_NOR_W; break;
                case ID_SQU_Y: con[0] = ID_NOR_Y; break;
                case ID_SQU_R: con[0] = ID_NOR_R; break;
                case ID_SQU_L: con[0] = ID_NOR_L; break;  // l
                case ID_SQU_SZH: con[0] = ID_SKT_SZH; break;
                case ID_SQU_S: con[0] = ID_NOR_S; break;  //
                default: ; break;
              }
          }
        else if ( vwlltr2 == ID_SQU_A && vwlltr == 0 )  // CAREFUL
          {
            switch ( con[0] )
              {
//  with long vowel.
                case ID_SQU_DR:
                  con[0] = ID_RSP_DR;
                  break;
// D with long vowel.
                case ID_SQU_D:
                  con[0] = ID_RSP_D;
                  break;
// i
                case ID_SQU_R:
                  if ( vwlsgn == ID_VOW_IR )
                    con[0] = ID_NOR_R;
                  break;
// i, l, etc
                case ID_SQU_L:
                  con[0] = ID_NOR_L;
                  break;
// y, y, etc
                case ID_SQU_Y:
                  con[0] = ID_NOR_Y;
                  break;
// w
                case ID_SQU_W:
                  if ( vwlsgn == ID_VOW_U )
                    con[0] = ID_NOR_W;
                  break;
// s
                case ID_SQU_S:
                  if ( vwlsgn == ID_VOW_I )
                    con[0] = ID_NOR_S;
                  break;
                default: ; break;
              }
          }
        else if ( vwlltr == ID_SSB_R )
          {
// yi, yi
            if ( con[0] == ID_SQU_Y )
              con[0] = ID_NOR_Y;
          }
        else if ( vwlltr == ID_SSB_L )
          {
// yi, yi
            if ( con[0] == ID_SQU_Y )
              con[0] = ID_NOR_Y;
          }
      } // END - if ( numcon == 1 )
    else if ( numcon == 2 )
      {
        switch ( con[0] )
          {
            case ID_SQU_TS:
// cca
              if ( con[1] == ID_SLO_TS )
                con[0] = ID_NOR_TS;
// cya
              else if ( con[1] == ID_SUB_Y )
                {
                  if ( vwlsgn == ID_VOW_AI || vwlsgn == ID_VOW_E )
                    con[0] = ID_SLO_TS;
                  else
                    con[0] = ID_NOR_TS;
                }
              break;
            case ID_SQU_TSH:
// chcha
              if ( con[1] == ID_SLO_TSH )
                con[0] = ID_NOR_TSH;
              break;
            case ID_SQU_DZ:
// jva
              if ( con[1] == ID_SUB_W && vwlsgn != ID_VOW_U )
                con[0] = ID_NOR_DZ;
              break;
            case ID_SQU_NG:
              if ( con[1] == ID_SQU_NG )
                {
                  con[0] = ID_NOR_NG;
                  con[1] = ID_NOR_NG;
                }
              break;
            case ID_SQU_DR:
// a
              if ( con[1] == ID_SQU_DR && vwlsgn != ID_VOW_U )
                {
                  con[0] = ID_RSP_DR;
                  con[1] = ID_RSP_DR;
                }
              break;
            case ID_SQU_NR:
// a
              if ( con[1] == ID_SQU_DR && vwlsgn != ID_VOW_U )
                con[1] = ID_RSP_DR;
              break;
            case ID_SQU_D:
// dka
              if ( con[1] == ID_SQU_K && vwlsgn != ID_VOW_U )
                  con[0] = ID_RSP_D;
// dga
              else if ( con[1] == ID_SQU_G && vwlsgn != ID_VOW_U )
                con[0] = ID_RSP_D;
// dda
              else if ( con[1] == ID_SQU_D )
//            else if ( con[1] == ID_SQU_D && vwlsgn != ID_VOW_U )
                {
                  con[0] = ID_RSP_D;
                  con[1] = ID_RSP_D;
                }
// dya, dyi
              else if ( con[1] == ID_SUB_Y && vwlsgn != ID_VOW_U )
                  con[0] = ID_RSP_D;
// dva, dvi
              else if ( con[1] == ID_SUB_W && vwlsgn != ID_VOW_U )
                  con[0] = ID_RSP_D;
              break;
            case ID_SQU_N:
// nda
              if ( con[1] == ID_SQU_D && vwlsgn != ID_VOW_U )
                con[1] = ID_RSP_D;
// na
              else if ( con[1] == ID_SQU_DR && vwlsgn != ID_VOW_U )
                con[1] = ID_RSP_DR;
// When do we ever use ID_RSS_D, 109 ????
              break;
            case ID_SQU_P:
// apda
              if ( con[1] == ID_SQU_D )
                con[1] = ID_RSP_D;
              break;
            case ID_SQU_M:
// mya
              if ( con[1] == ID_SUB_Y && vwlsgn != ID_VOW_U )
                con[0] = ID_NOR_M;
              break;
            case ID_SQU_Y:
              if ( con[1] == ID_SUB_W )  /** Wazur **/
                con[0] = ID_NOR_Y;
              else if ( con[1] == ID_SQU_Y )
                {
                  con[0] = ID_NOR_Y;
                  con[1] = ID_NOR_Y;
                }
              break;
            case ID_SQU_R:
// rra
              if ( con[1] == ID_SQU_R && vwlsgn == 0 )
                {
                  con[0] = ID_NOR_R;
                  con[1] = ID_NOR_R;
                }
// rva
              else if ( con[1] == ID_SUB_W )  /** Wazur **/
                con[0] = ID_NOR_R;
              break;
            case ID_SUP_R:  // Rago
// rba
              if ( con[1] == ID_SQU_B )
                con[1] = ID_NOR_B;
// rma
              else if ( con[1] == ID_SQU_M && vwlsgn != ID_VOW_U )
                con[1] = ID_NOR_M;
// rca
              else if ( con[1] == ID_SQU_TS )
                con[1] = ID_SLO_TS;
// rth
              else if ( con[1] == ID_SQU_TH && vwlsgn != ID_VOW_U )
                con[1] = ID_NOR_TH;
              break;
            case ID_SQU_L:
              if ( con[1] == ID_SUB_W )  /** Wazur **/
                con[0] = ID_NOR_L;
              break;
            case ID_SQU_S:
// sva
              if ( con[1] == ID_SUB_W && !vwlltr )
                con[0] = ID_NOR_S;
// ssa
              else if ( con[1] == ID_SQU_S && vwlsgn != ID_VOW_U )
                {
                  con[0] = ID_NOR_S;
                  con[1] = ID_NOR_S;
                }
              break;
            case ID_SQU_H:
              if ( con[1] == ID_SUB_W )  /** Wazur **/
                con[0] = ID_NOR_H;
              break;
            default: ; break;
          }
      } // END - else if ( numcon == 2 )
// sya
    else if ( con[numcon-2] == ID_SQU_S && con[numcon-1] == ID_SUB_Y )
      {
        con[numcon-2] = ID_NOR_S;
      }
} // END - void fixskt ( void )

// This routine prints a Sanskrit syllable stack, prepared in con[]
void prn_syl ( void )
{
    int i; // x;
    int dwn;  // Amount to step down for compound letters
    int drp;  // Total drop of character
    int bs;
    int h;
    int l, r, cl, cr; /** Values for spacing **/
    int btm;        /** Bottom of compound character **/
    int obsl, obsr; /** Left and right extremes of characters **/
    int cntrshft;   // Centre shift for character.
    int boxwdth;    // Bounding box width.
    int chrwdth;    // Effective character width.
    int chrcntr;    // Position of character centre, from left edge.
    int maxrgtpnt;   // Max point on right of characters.
    int maxlftpnt; // Max point on left of characters.

    for ( i = 0; i < 9; ++i )
      conshft[i] = 0; // NEW - 24/9/1997

    obsl = logicx; // These four are for checking if a compound
    obsr = logicx; // character intrudes on the next line
    btm = logicy;
    drp = 0;  // MOVED FROM PRN_SKT ();

/** CHECK HOW MANY STACKED CHARACTERS, SO THAT WE CAN SQUASH THEM **/

  if ( d_bug == 6 && firstpass )
    {
      if ( dbg_fil == NULL )
        dbg_fil = fopen ( "WORDS.DAT", "a+" );
      if ( dbg_fil == NULL )
        {
          Msg ( "Error opening DEBUG file!" );
        }
      if ( dbg_fil && ( numcon == 1 || numcon == 2 ) )
        {
          sprintf ( d_bugstr, "%s", wrd_tst );
          wrt_debug ();
        }
    }
    i = 0;
    if ( vwlltr )
      ++i;
    if ( vwlltr2 )
      ++i;
    i = i + numcon; /* Number of stacked characters to print */
    if ( i > 1 )
      {
        vwlltr = squsktid ( vwlltr );
        for ( i = 0; i < numcon; ++i )   // WHAT??
          if ( i == 0 && con[1] == ID_SUB_R )
            break;
          else
            con[i] = squsktid ( con[i] );
      }
    fixskt ();
/** FIX UP THE SPACING **/
    l = chrwd[smlfnt];      // General character width for particular font.
    r = chrwd[smlfnt]; /** These are artificial, and need to be changed **/
    boxwdth = chrwd[smlfnt]; // Bounding box width
// FROM HERE, NEED TO USE CENTRE OF CHARACTERS
// GET SPACING FOR STACKED CONSONANTS
    maxrgtpnt = 0; // Max point on right of characters.
    maxlftpnt = boxwdth; // Max point on left of characters.
    if ( numcon )
      for ( i = 0; i < numcon; ++i )
        {
        // this gives two numbers, the minimum white space dots either
        // side of character in bounding box. Therefore Bound - these
        // two = the extreme width of the character in dots
          getchrspc ( (int) con[i], &cl, &cr );

          if ( d_bug == 3 )
            {
              sprintf ( d_bugstr, "CHR = %s, left = %d, right = %d\n",
                      chrs[con[i]], cl, cr );
              wrt_debug ();
            }

       // cl, cr is min left, right white space
          chrwdth = boxwdth - cl - cr; // Effective character width.
          chrcntr = chrwdth / 2 + cl; // Position of character centre,
                                      // from left edge.
// This gives the shift to the right (+ve) of the character to centre it.
          cntrshft = boxwdth / 2 - chrcntr;
// CHECK SPECIAL CASES -
          cntrshft = chk_sktshift ( i, cntrshft );
// Hence, effective white space on left is cl+cntrshft, etc.
          cl = cl + cntrshft;
          if ( cl < maxlftpnt )
            maxlftpnt = cl;
          cr = cl + chrwdth;
          if ( cr > maxrgtpnt )
            maxrgtpnt = cr;
          conshft[i] = cntrshft;  // Array for character centre shifts.
        }
// GET SPACING FOR VOWEL LETTERS
    if ( vwlltr )
      {
        getchrspc ( (int) vwlltr, &cl, &cr ); // Left and right distances
                                              // from sides of bound box
        chrwdth = boxwdth - cl - cr; // Effective character width.
        chrcntr = chrwdth / 2 + cl; // Position of character centre,
                                      // from left edge.
// This gives the shift to the right (+ve) of the character to centre it.
        cntrshft = boxwdth / 2 - chrcntr;
// Hence, effective white space on left is cl+cntrshft, etc.

// SHIFT SPECIAL CASES LEFT OR RIGHT
        if ( vwlltr == ID_SQU_A )
          cntrshft = cntrshft + vwlshft ();
        else if ( vwlltr == ID_SSB_L )
          {
// ki
            if ( con[0] == ID_SQU_K && numcon == 1 )
              cntrshft = cntrshft + smlconv ( 3 );
// gi
            else if ( con[0] == ID_SQU_G && numcon == 1 )
              cntrshft = cntrshft + smlconv ( 3 );
// ghi
//          else if ( con[0] == ID_SQU_GH && numcon == 1 )
//            cntrshft = cntrshft + smlconv ( 5 );
// i
            else if ( con[0] == ID_SQU_SH && numcon == 1 )
              cntrshft = cntrshft + smlconv ( 5 );
          }
        vwl1shft = cntrshft;
        cl = cl + cntrshft;
        if ( cl < maxlftpnt )
          maxlftpnt = cl;
        cr = cl + chrwdth;
        if ( cr > maxrgtpnt )
          maxrgtpnt = cr;
      }
    if ( vwlltr2 )
      {
        getchrspc ( (int) vwlltr2, &cl, &cr );
        chrwdth = boxwdth - cl - cr; // Effective character width.
        chrcntr = chrwdth / 2 + cl; // Position of character centre,
                                      // from left edge.
        cntrshft = boxwdth / 2 - chrcntr;
        if ( vwlltr2 == ID_SQU_A && vwlltr == ID_NOR_SA )
          cntrshft = cntrshft + smlconv ( 2 );

        if ( vwlltr2 == ID_SQU_A )
          cntrshft = cntrshft + vwlshft ();
        vwl2shft = cntrshft;

        cl = cl + cntrshft;
        if ( cl < maxlftpnt )
          maxlftpnt = cl;
        cr = cl + chrwdth;
        if ( cr > maxrgtpnt )
          maxrgtpnt = cr;
      }
    if ( vwlsgn )
      {
        getchrspc ( (int) vwlsgn, &cl, &cr );
        chrwdth = boxwdth - cl - cr; // Effective character width.
        chrcntr = chrwdth / 2 + cl; // Position of character centre,
                                      // from left edge.
// This gives the shift to the right (+ve) of the character to centre it.
        vwlsgnshft = boxwdth / 2 - chrcntr;
        if ( vwlsgn == ID_NOR_I || vwlsgn == ID_VOW_E )
          if ( con[0] == ID_NOR_TS || con[0] == ID_NOR_TSH ||
               con[0] == ID_SQU_TS || con[0] == ID_SQU_TSH ||
               con[0] == ID_NOR_DZ || con[0] == ID_SKT_JR )
            vwlsgnshft = vwlsgnshft - smlconv ( 9 );
      }
// NOW PRINT THE SYLLABLE
    bs = maxlftpnt - jus();
    h = maxrgtpnt + jus(); // THIS IS WRONG ????????
    if ( chk_nxto ( h - bs, vwlsgn, &obsl ) ) // Check for next obscure.
      {
        bs = maxlftpnt - jus();
        h = maxrgtpnt + jus(); // THIS IS WRONG ????????
        if ( visarg && shd )
          h = h + shadspc[smlfnt] + aftshad[smlfnt];
      }
    backspc ( bs );
    dwn = 0;
    /** PRINT ANUSVARA **/
    if ( anusvar )
      {
        if ( anusvar == ID_ANUSVZ ) /** IF WITH CRESCENT **/
          dwn = -4;
        if ( vwlsgn == ID_NOR_I || vwlsgn == ID_VOW_E || vwlsgn == ID_VOW_AI )
          {
            backspc ( -9 );
            prn_chr ( anusvar, 0, dwn - 5 );
            backspc ( 9 );
          }
        else if ( vwlsgn == ID_VOW_O || vwlsgn == ID_VOW_AU )
          prn_chr ( anusvar, 0, dwn - 5 );
        else if ( vwlsgn == ID_VOW_IR )
          {
            backspc ( 9 );
            prn_chr ( anusvar, 0, dwn - 5 );
            backspc ( -9 );
          }
        else
          prn_chr ( anusvar, 0, dwn );
        dwn = 0;
      }
// PRINT STACKED CONSONANTS:
    for ( i = 0; i < numcon; ++i )
      {
// FIX SPECIAL CASE DROPS:
        if ( i > 0 && numcon > i )
          {
            if ( con[i] == ID_SKT_FO ) // f
              dwn = dwn + smlconv ( 6 );
            else if ( con[i] == ID_SKT_QS ) // q
              dwn = dwn - smlconv ( 9 );
            else if ( con[i-1] == ID_NOR_TS && con[i] == ID_SUB_Y )
              dwn = dwn - smlconv ( 6 );  // cya
            else if ( con[i-1] == ID_SLO_TS && con[i] == ID_SUB_Y )
              dwn = dwn - smlconv ( 6 );  // cya
            else if ( con[i-1] == ID_SKT_FO && con[i] == ID_SQU_SZH )
              dwn = dwn - smlconv ( 3 );  // f
            else if ( con[i-1] == ID_SQU_SH && con[i] == ID_SQU_SZH )
              dwn = dwn - smlconv ( 6 );  // 
            else if ( con[i-1] == ID_NOR_N && con[i] == ID_SUB_R )
              dwn = dwn - smlconv ( 12 ); // nr
            else if ( con[i-1] == ID_NOR_D && con[i] == ID_SUB_R )
              dwn = dwn - smlconv ( 6 );  // dr
            else if ( con[i-1] == ID_RSP_D && con[i] == ID_SUB_Y )
              dwn = dwn - smlconv ( 3 );  // dy
            else if ( con[i-1] == ID_SQU_K && con[i] == ID_SQU_SZH )
              dwn = dwn - smlconv ( 9 );  // kSa
          }
// PROBLEM HERE IS THAT IT SHOULD BE DONE EARLIER IN ORDER TO ESTABLISH
// OVERALL CHARACTER WIDTH ACCURATELY
        backspc ( -conshft[i] );
        if ( d_bug == 5 )
          {
            sprintf ( d_bugstr, "Backspacing: %d", -conshft[i] );
            wrt_debug ( );
          }
        if ( ( i == numcon - 1 ) && !vwlltr && !vwlsgn && !vwlltr2 )
          prn_chr ( con[i], h-conshft[i], dwn ); // h = distance to
                                                 // advance.
        else
          prn_chr ( con[i], -conshft[i], dwn );
        dwn = dwn + chrdrp ( con[i] );

      } // END - for ( i = 0; i < numcon; ++i )
// PRINT VOWEL LETTERS
    if ( vwlltr )
      {
// First, check special conditions for changing drop for some letters:
        if ( con[0] == ID_RSP_DR && numcon == 1 && vwlltr == ID_SQU_A )
// 
          dwn = dwn - smlconv ( 12 );
        else if ( con[0] == ID_NOR_D && numcon == 1 && vwlltr == ID_SQU_A )
// d
          dwn = dwn - smlconv ( 12 );
        else if ( con[0] == ID_SQU_DZ && con[1] == ID_SQU_NY &&
                  numcon == 2 && vwlltr == ID_SQU_A )
// j
          dwn = dwn - smlconv ( 3 );
        else if ( con[0] == ID_SQU_NR && con[1] == ID_RSP_DR &&
                  numcon == 2 && vwlltr == ID_SQU_A )
// 
          dwn = dwn - smlconv ( 9 );
        else if ( con[0] == ID_SKT_SZH && numcon == 1 && vwlltr == ID_SQU_A )
// 
          dwn = dwn - smlconv ( 18 );
        if ( con[numcon-1] == ID_SQU_NR && vwlltr == ID_SQU_A )
          dwn = dwn - smlconv ( 8 ); // Move the vowel up a little - 4/8/2012

// NOW PRINT:
        if ( d_bug == 3 )
          {
            sprintf ( d_bugstr, "Printing vowel, DWN = %d", dwn );
            wrt_debug ( );
          }
        backspc ( -vwl1shft );
        if ( !vwlsgn && !vwlltr2 )
          prn_chr ( vwlltr, h-vwl1shft, dwn );
        else
          prn_chr ( vwlltr, -vwl1shft, dwn );
        dwn = dwn + chrdrp ( vwlltr ); // dwn is accumulative, from baseline.
      }
    if ( vwlltr2 )
      {
// FIRST, CHECK SPECIAL CONDITIONS FOR CHANGING DROP FOR SOME LETTERS:
        if ( con[0] == ID_SQU_NR && con[1] == ID_RSP_DR &&
             vwlltr2 == ID_SQU_A )
// , etc
          dwn = dwn - smlconv ( 12 );
        else if ( con[0] == ID_RSP_DR && numcon == 1 && vwlltr2 == ID_SQU_A )
// 
          dwn = dwn - smlconv ( 12 );
        else if ( con[0] == ID_RSP_D && numcon == 1 && vwlltr2 == ID_SQU_A )
// d
          dwn = dwn - smlconv ( 9 );
        if ( con[0] == ID_SQU_SZH && con[1] == ID_SQU_NR &&
             numcon == 2 && vwlltr2 == ID_SQU_A )
// 
          dwn = dwn - smlconv ( 6 );
// NOW PRINT:
        backspc ( -vwl2shft );
        if ( !vwlsgn )
          prn_chr ( vwlltr2, h-vwl2shft, dwn );
        else
          prn_chr ( vwlltr2, -vwl2shft, dwn );
        dwn = dwn + chrdrp ( vwlltr2 );
      }
    drp = drp + dwn;
    if ( d_bug == 3 )
      {
        sprintf ( d_bugstr, "vwlltr: %s, vwlltr2: %s, dwn: %d drp: %d.", 
                  chrs[vwlltr], chrs[vwlltr2], dwn, drp );
        wrt_debug ();
      }
/*** PRINT THE VOWEL SIGN ***/
    if ( vwlsgn )
      {
        if ( d_bug == 3 )
          {
            sprintf ( d_bugstr, "DWN = %d\n", dwn );
            wrt_debug ();
          }
        backspc ( -vwlsgnshft );
        if ( vwlsgn != ID_NOR_U )
          {
            dwn = 0;
            if ( con[0] == ID_SKT_FO &&
               ( vwlsgn == ID_NOR_I || vwlsgn == ID_VOW_IR ))
            dwn = dwn - smlconv ( 10 ); // New 1/2/2004
          }
        else
          drp = drp + chrdrp ( vwlsgn );
        prn_chr ( vwlsgn, h-vwlsgnshft, dwn );
      }
/** END OF THIS SYLLABLE, SO DEAL WITH OBSCURING CHARACTERS **/
    if ( drp > linspc - bas2top && obblck ) /** Check this **/
      {
        btm = drp + btm;
        obsr = logicx;
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Add2obsq1: drp: %d, linspc: %d", drp, linspc );
            wrt_debug ();
          }
        add2obsq ( obsl, obsr, btm );
      }
} // End of PRN_SYL

void dissktstr ( void ) // Prints out sktstr array
  { // This old debug routine disabled for now.
    int j, hldpstn;
    hldpstn = 0;
    do
      {
//      printf ( "%d: ", sktstr[hldpstn] );
        for ( j = hldpstn + 1; j < 1 + hldpstn + sktstr[hldpstn]; ++j )
          ;
//      printf ( "%s, ", chrs[sktstr[j]] );
        hldpstn = j;
//        if ( j > 14 )
//          getch ();
      } while ( sktstr[j] != 0 );
//  printf ( "\n" );
  }

/**** MAIN ROUTINE TO PRINT SANSKRIT WORD IN TEST BUFFER ****/

void prn_skt ( void ) /** SKT **/
  {
    int i;
    int h;
    int c;
    int subjoin;
    int syln;   // For debug only.
    int bs;
    int obsl; //, obsr; /** Left and right extremes of characters **/
    for ( i = 0; i < 10; ++i )
      con[i] = 0;
    if ( jd_bug )
      syln = 0;
    i = 0;
    if ( d_bug == 3 )
      {
        sprintf ( d_bugstr, "prn_skt, word: %s", wrd_tst );
        wrt_debug ();
//      dissktstr ();      // PRINTS OUT sktstr ARRAY
      }
/** DEAL WITH SOME SPECIAL CASES **/
    if ( strncmp ( wrd_tst, "vajr", 4 ) == 0 )
      sktstr[1] = ID_NOR_B;
// GO THROUGH STRING, PRINTING EACH SYLLABLE 
    skthld = 0;
    while ( sktstr[skthld] )
      {
        visarg = 0;
        vwlsgn = 0; vwlltr = 0; vwlltr2 = 0; numcon = 0; anusvar = 0;
        sktptr = skthld + 1;
        if ( jd_bug )
          {
            sprintf ( d_bugstr, "Syllable no: %d, Xcur = %d\n", syln, logicx );
            wrt_debug ();
            ++syln;
          }
        subjoin = 0;
// GO THROUGH EACH CHARACTER, PLACING THEM IN con[] STACK 
        for ( i = 0; i < sktstr[skthld]; ++i )
          {
            c = sktstr[sktptr + i];

            if ( c == ID_NOR_I || c == ID_NOR_U || c == ID_NOR_E || c ==
                ID_NOR_O || c == ID_SKT_AU || c == ID_SKT_AI )
              {
                vwlsgn = c;
                if ( i == 0 )  /* eg. for, om, ah, etc */
                  vwlltr = ID_NOR_SA; /** Short A **/
              }
            else if ( c == ID_NOR_SA )
              {
                if ( i == 0 )
                  vwlltr = ID_NOR_SA;
                else
                  vwlltr = 0;
              }
            else if ( c == ID_SKT_A )  /** Long A **/
              {
                // What if this is the first letter, as in ah ??
                vwlltr = ID_NOR_A;
                if ( i == 0 )  /* eg. for, om, ah, etc */
                  {
                    vwlltr = ID_NOR_SA; /** Short A **/
                    vwlltr2 = ID_SQU_A; /** Long A **/
                  }
              }
            else if ( c == ID_SKT_I ) /** Long I **/
              {
                if ( i == 0 )
                  vwlltr = ID_NOR_SA;
                vwlltr2 = ID_SQU_A; /** Long A **/
                vwlsgn = ID_NOR_I;
              }
            else if ( c == ID_SKT_U )  /** Long U **/
              {
                if ( i == 0 )
                  vwlltr = ID_NOR_SA;
                vwlltr2 = ID_SQU_A; /** Long A **/
                vwlsgn = ID_VOW_U;
              }
            else if ( c == ID_SKT_R || c == ID_SKT_LR )
              {
                vwlsgn = ID_VOW_IR;
//              vwlltr = ID_NOR_R;   // 7/9/97
                vwlltr = ID_SSB_R;
                con[numcon++] = ID_SKT_R; /** Mark it as a vowel letter **/
                if ( c == ID_SKT_LR )
                  vwlltr2 = ID_SQU_A; /** Long A **/
              }
            else if ( c == ID_SKT_LI || c == ID_SKT_LLI )
              {
                vwlsgn = ID_VOW_IR;  // Reversed i mark.
//              vwlltr = ID_NOR_L;    // 7/9/97
                vwlltr = ID_SSB_L;
                con[numcon++] = ID_SKT_LI; /** Mark it as a vowel letter **/
                if ( c == ID_SKT_LLI )
                  vwlltr2 = ID_SQU_A; /** Long A **/
              }
            else if ( c == ID_SKT_M ) /* M dot, Anusvara */
              anusvar = chk_anus ();
            else if ( c == ID_STOP ) /* Full stop */
              subjoin = 1;  // Sets to check if next character is "Y".
            else
              {
                if ( subjoin && c == ID_NOR_Y )
                  con[numcon++] = ID_SUB_Y;
                else
                  con[numcon++] = c;
              }
          }
/** SORT OUT ANY SPECIAL CASES **/
        if ( d_bug == 3 )
          {
            if ( numcon == 1 )
              sprintf ( d_bugstr, "Syllable = %s", chrs[(int)con[0]] );
            else if ( numcon == 2 )
              sprintf ( d_bugstr, "Syllables = %s, %s", chrs[(int)con[0]],
                                                       chrs[(int)con[1]] );
            else if ( numcon == 3 )
              sprintf ( d_bugstr, "Syllables = %s, %s, %s", chrs[con[0]],
                               chrs[(int)con[1]], chrs[(int)con[2]] );
            else if ( numcon == 4 )
              sprintf ( d_bugstr, "Syllables = %s, %s, %s, %s", chrs[(int)con[0]],
                               chrs[(int)con[1]], chrs[(int)con[2]], chrs[(int)con[3]] );
            if ( firstpass )
              wrt_debug ( );
          }
        if ( numcon >= 1 )  /** Watch this! **/ // WHY???
          chk_spcskt ();
        if ( con[0] == ID_SKT_H ) // Visarga, it is treated as a consonant
          visarg = 1;
        if ( con[numcon-1] == ID_SKT_R )  /** Get rid of marker RA vowels **/
          --numcon;
        else if ( con[numcon-1] == ID_SKT_LI ) //Get rid of marker LA vowels
          --numcon;
        prn_syl ();
        skthld = skthld + sktstr[skthld] + 1;
      } /** GO BACK FOR NEXT SYLLABLE **/
/** PRINT ANY 'SHAD, VISARGA' OR DOTS REQUIRED AFTER SANSKRIT WORD **/
    if ( shd && !visarg )
      {
        if ( shd == 64 )   // NEW, 30/5/94 - "Continuation" shad
          {
            h = chrwd[smlfnt] - consr[smlfnt] + shadspc[smlfnt] * 2;
            chk_nxto ( h - aftshad[smlfnt], 0, &obsl );
            prn_chr ( ID_CONTSH, h, 0 ); /** Shad **/
          }
        else
          {
// shdsl, shdsr - spacing left and right for shad.
            backspc ( shdsl[smlfnt] - jus () );
            h = chrwd[smlfnt] + jus() - shdsr[smlfnt] + shadspc[smlfnt];
            h += aftshad[smlfnt];
            if ( shd < 8 )
              prn_chr ( ID_SHAD, h, 0 ); /** Shad **/
            else
              prn_chr ( ID_NYISH, h, 0 ); /** Shad **/
// This whole logical block added 15/11/2003:
            if ( shd == 2 ) // this is from prn_wrd
              {
                h = chrwd[smlfnt] + jus() - shdsr[smlfnt];
                h += aftshad[smlfnt];
                backspc ( shdsl[smlfnt] - intcspc );
                prn_chr ( ID_SHAD, h, 0 ); /** Shad **/
              }
          }
      }
    else if ( !visarg )  // JUST PRINT DOT ONLY.
      {
        if ( !ignore_tsheg )
          {
            obsl = logicx;
            h = chrwd[smlfnt] + jus() - dotsr[smlfnt] + 2 * intwspc;
            bs = dotsl[smlfnt] - intcspc;
            if ( chk_nxto ( h - bs, 0, &obsl ) ) // NEW - 30/5/94
              {
                h = chrwd[smlfnt] + jus() - dotsr[smlfnt];
                bs = dotsl[smlfnt] - intwspc;   // XZXZXZ
              }
            backspc ( bs );
            prn_chr ( ID_DOT, h, 0 ); /** Dot **/
          }
        else
          {
            ignore_tsheg = 0;
            nxt_wrd_font_chg = 0;
          }
      }
    else if ( shd && visarg ) /** IGNORE FIRST SHAD **/
      {
        backspc ( -shadspc[smlfnt] - aftshad[smlfnt] - jus () ); /** INCREASE A
                                                                  BIT ??  **/
        if ( shd > 1 )
          {
            h = chrwd[smlfnt] + jus() - shdsr[smlfnt]; /*+ shadspc[smlfnt];*/
            h += aftshad[smlfnt];
            backspc ( shdsl[smlfnt] - intcspc );
            prn_chr ( ID_SHAD, h, 0 ); /** Shad **/
          }
      }
    else if ( visarg && !shd ) // Just move forward a little.
      {
        backspc ( -2 * bfrshad[smlfnt] );
      }
    if ( d_bug == 3 )
      {
        sprintf ( d_bugstr, "End of PRN_SKT" );
        wrt_debug ();
      }
  } // End of PRN_SKT

int chk_anus ( void ) /** SKT **/
  {
    if ( strcmp ( wrd_tst, "hkmlwrya" ) == 0 
         || strcmp ( wrd_tst, "hkmlvrya" ) == 0 
         || strcmp ( wrd_tst, "hkSmlwryaM" ) == 0 )
      return ( ID_ANUSVZ );
    if ( strcmp ( wrd_tst, "h" ) == 0 || strcmp ( wrd_tst, "hUM" ) == 0 )
      return ( ID_ANUSVZ );
    if ( strcmp ( wrd_tst, "bhr" ) == 0 || strcmp ( wrd_tst, "bhrUM" ) == 0 )
      return ( ID_ANUSVZ );
    if ( ( strcmp ( wrd_tst, "ba" ) == 0 || strcmp ( wrd_tst, "baM" ) == 0)
           && ( strcmp ( lst_wrd, "h" ) == 0 || strcmp ( lst_wrd, "hUM" ) == 0 ) )
      return ( ID_ANUSVZ );
    else
      return ( ID_ANUSV );
  }

void chk_spctib ( void )
  {
    if ( !sup && !sub ) // 5/10/1997
      {
        if ( rot == ID_NOR_TS &&
           ( vw1 == ID_VOW_I || vw1 == ID_VOW_E || vw1 == ID_VOW_O ) )
          rot = ID_SLO_TS;
        else if ( rot == ID_NOR_TSH &&
           ( vw1 == ID_VOW_I || vw1 == ID_VOW_E || vw1 == ID_VOW_O ) )
          rot = ID_SLO_TSH;
        else if ( rot == ID_NOR_DZ &&
           ( vw1 == ID_VOW_I || vw1 == ID_VOW_E || vw1 == ID_VOW_O ) )
          rot = ID_SLO_DZ;
      }
    else if ( sup && !sub ) // 3/10/1997
      {
        switch ( sup )
          {
            case ID_SUP_R:
            // RK
              if ( rot == ID_SQU_K )
                rot = ID_RSB_K;
            // RG
              else if ( rot == ID_SQU_G )
                rot = ID_RSB_G;
            // RNG
              else if ( rot == ID_SQU_NG )
                rot = ID_NOR_NG;
            // RNY
              else if ( rot == ID_RSP_NY )
                sup = ID_SUP_SR;
            // RJ
              else if ( rot == ID_RSP_J )
                rot = ID_NOR_J;
            // RN - eg. rnam, rnum
              else if ( rot == ID_SQU_N  )
                {
                  if ( vw1 == ID_VOW_U )
                    rot = ID_RSB_N;
                  else
                    rot = ID_NOR_N;
                }
            // RB - eg: rbad, rab rlabs, etc
              else if ( rot == ID_SQU_B )
                rot = ID_NOR_B;
            // RM
              else if ( rot == ID_SQU_M )
                rot = ID_NOR_M;
           // rts
              else if ( rot == ID_NOR_TS )
                rot = ID_SLO_TS;
            // rdz - eg. rdzogs
              else if ( rot == ID_SQU_DZ )
                rot = ID_SLO_DZ;
              break;
            case ID_SUP_L:
            // lk
              if ( rot == ID_SQU_K )
                rot = ID_RSB_K;
            // lg
              else if ( rot == ID_SQU_G )
                rot = ID_RSB_G;
            // lng
              else if ( rot == ID_SQU_NG )
                sup = ID_SQU_L;
            // lj
              else if ( rot == ID_RSP_J && vw1 != ID_VOW_U )
                {
                  sup = ID_SQU_L;
                  rot = ID_NOR_J;
                }
           // lt
              else if ( rot == ID_SQU_T && vw1 != ID_VOW_U )
                sup = ID_SQU_L;
            // lp
              else if ( rot == ID_SQU_P )
                sup = ID_SQU_L;
              break;
            case ID_SUP_S:
           // sk
//            if ( rot == ID_SQU_K && vw1 != ID_VOW_U )
              if ( rot == ID_SQU_K )
                {
                  sup = ID_SQU_S;
                  rot = ID_RSB_K;
                }
           // SG
              else if ( rot == ID_SQU_G && vw1 != ID_VOW_U )
                {
                  sup = ID_SQU_S;
                  rot = ID_RSB_G;  // Can probably use this for "gyu" as well?
                }
           // SNY
              else if ( rot == ID_RSP_NY ) // Only, J, NY, D
                sup = ID_SQU_S;
           // ST
              else if ( rot == ID_SQU_T )
                sup = ID_SQU_S;
           // SP
              else if ( rot == ID_SQU_P )
                sup = ID_SQU_S;
           // STS  
              else if ( rot == ID_NOR_TS )
                {
//                sup = ID_SQU_S;
                  rot = ID_SLO_TS;
                }
              break;
            default: ; break;
          }
      }
    if ( sub )
      {
        switch ( sub )
          {
            case ID_SUB_W:
            // TW
              if ( rot == ID_NOR_T || rot == ID_SQU_T )
                {
                  rot = ID_SKT_TW;
                  sub = 0;
                }
              break;
            case ID_SUB_Y:
            // HY
              if ( rot == ID_NOR_H || rot == ID_SQU_H )
                {
                  rot = ID_SKT_HY;
                  sub = 0;
                }
              break;
            case ID_SUB_R:
            // TR
              if ( rot == ID_NOR_T || rot == ID_SQU_T )
                {
                  rot = ID_SKT_TR;
                  sub = 0;
                }
            // HR
              else if ( rot == ID_NOR_H || rot == ID_SQU_H )
                {
                  rot = ID_SKT_HR;
                  sub = 0;
                }
              break;
            case ID_SUB_L:
            // SLU
              if ( rot == ID_NOR_S && vw1 == ID_VOW_U )
                rot = ID_SQU_S;
              break;
            default: ; break;
          }
      }
  } // END - chk_spctib

void chk_subdrop ()
  {
    if ( rot == ID_NOR_R && sub == ID_SUB_L )
      dwn = dwn - smlconv ( 10 );
  }

int chk_rotshift ( void )
  {
// This is called AFTER: chk_spctib ();
    int shft = 0;  // Amount to backspace root character.

    if ( sup == ID_SUP_R && ( rot == ID_NOR_N || rot == ID_RSB_N ) )
      shft = 6;
    else if ( ( sup == ID_SUP_S || sup == ID_SQU_S ) && rot == ID_SLO_TS )
      shft = -3;  // Shift root to right.
    shft = smlconv ( shft );
    return ( shft );
  } // END - chk_rotshift ( )

int chk_spcvowelshift ( void )
  {
// This is AFTER chk_spctib ();
    int shft = 0;  // Amount to backspace vowel sign.

    if ( sup == 0 )
      {
        if ( ( rot == ID_SLO_TS || rot == ID_SLO_TSH ||
               rot == ID_SLO_DZ || rot == ID_NOR_DZH ) &&
             ( vw1 == ID_NOR_O || vw1 == ID_NOR_E || vw1 == ID_NOR_I ) )
          shft = smlconv ( 3 ); // This moves vowel to the left
// ngo, etc
        if ( vw1 == ID_NOR_O )
          {
            if ( rot == ID_NOR_NG || rot == ID_NOR_TH ||
                 rot == ID_NOR_N || rot == ID_NOR_P ||
                 rot == ID_NOR_PH || rot == ID_NOR_B ||
                 rot == ID_NOR_M || rot == ID_NOR_S )
              shft = smlconv ( -6 ); // This moves vowel to the right
          }
      }
    else
      {
        if ( vw1 == ID_NOR_O )
          {
            if ( sup == ID_SUP_S || rot == ID_NOR_TS )
              shft = smlconv ( -6 ); // This moves vowel to the right
          }
      }
    return ( shft );
  } // END - chk_spcvowelshift ( void )

/**** MAIN ROUTINE TO PRINT TIBETAN WORD IN TEST BUFFER ****/
void prn_wrd ( void )
  {
    int h;
    int frst_ltr;   /** Flag to indicate first letter printed **/
    int btm;   /** Bottom of compound character **/
    int drp;   /** Total drop of character **/
    int obsl, obsr; /** Left and right extremes of characters **/
    int no1shad; /** Flag to indicate skipping first of 2 shad **/
    int fwdstp, bs;
    int spcshft = 0;  // SPECIAL SHIFT VALUES;

    if ( d_bug > 1 )
      {
        sprintf ( d_bugstr, "PRN_WRD: %s, Cursor = %d", wrd_tst, pcur_x );
          wrt_debug ( );
      }
    if ( nxt_wrd_font_chg )
      {
        if ( d_bug > 1 && firstpass )
          {
            sprintf ( d_bugstr, "FONT CHANGE" );
            wrt_debug ( );
          }

        if ( ( nxt_wrd1[0] == 39 && nxt_wrd1[1] == 'i' ) ||
               strcmp ( nxt_wrd1, "sa" ) == 0 ||
               ( nxt_wrd1[0] == 's' && nxt_wrd1[1] == '\0' ) )
          {
            ignore_tsheg = 1;
          }
      }
    if ( lng == 2 )     /** If Language == Sanskrit **/
      {
        prn_skt ();
        strcpy ( lst_wrd, wrd_tst );
        return;
      }
    if ( d_bug == 7 && firstpass )
      {
        if ( dbg_fil == NULL )
          dbg_fil = fopen ( "WORDS.DAT", "a+" );
        if ( dbg_fil == NULL )
          Msg ( "Error opening DEBUG file!" );
        if ( rot == ID_NOR_TS || rot == ID_SQU_TS ||
             rot == ID_NOR_TSH || rot == ID_SQU_TSH ||
             rot == ID_NOR_DZ || rot == ID_SQU_DZ ||
             rot == ID_SLO_TS || rot == ID_SLO_TSH ||
             rot == ID_SLO_DZ ) 
          {
            sprintf ( d_bugstr, "%s", wrd_tst );
            wrt_debug ();
          }
      }
    dwn = 0; frst_ltr = 1; no1shad = 0; drp = 0;
    if ( sup && !sub )
      rot = gsup_chr ( rot ); /** ID for root with superjoined **/
    else if ( sup && sub )
      rot = gssp_chr ( rot ); /** ID for root with super and sub-joined **/
    else if ( !sup && sub )
      rot = sub_chr ( rot ); /** ID for root with subjoined **/
    if ( sup )  /** Get superjoin ID **/
      {
        if ( sup == ID_NOR_R ) /** rago **/
          sup = ID_SUP_R;
        else if ( sup == ID_NOR_L ) /** lago **/
          sup = ID_SUP_L;
        else if ( sup == ID_NOR_S ) /** sago **/
          sup = ID_SUP_S;
      }
    if ( sub )  /** Get subjoin ID **/
      {
        if ( sub == ID_NOR_Y ) /** yata **/
          sub = ID_SUB_Y;
        else if ( sub == ID_NOR_R ) /** rata **/
          sub = ID_SUB_R;
        else if ( sub == ID_NOR_L ) /** lata **/
          sub = ID_SUB_L;
        else if ( sub == ID_NOR_H ) /** hata **/
          sub = ID_SUB_H;
        else if ( sub == ID_NOR_A ) /** 'ata **/
          sub = ID_SUB_A;
        else if ( sub == ID_NOR_W ) /** wazur **/
          sub = ID_SUB_W;
      }
    if ( sub2 ) /** Get 2nd subjoin ID **/
      {
        if ( sub2 == ID_NOR_W ) /** wazur **/
          sub2 = ID_SUB_W;
      }
    chk_spctib ();   // SPECIAL INSTANCES
    getbasspc ();  /** Spacing for prefix and root **/
// PREFIX 
    if ( prf )
      {
        bs = prfsl - jus();
        h = chrwd[smlfnt] + jus() - prfsr;
        if ( chk_nxto ( h - bs, 0, &obsl ) )  //  POSSIBLE SERIOUS PROBLEM - OBSL
                                              //  NOT INSTANTIATED
// Maybe doesn't matter - cursor is set in CHK_NXTO - is OBSL ever used/needed?
          {
            bs = prfsl - jus();
            h = chrwd[smlfnt] + jus() - prfsr;
          }
        if ( prf == ID_NOR_A ) // Backspace a little more.
          bs = bs + smlconv ( 6 );
        backspc ( bs );
        prn_chr ( prf, h, 0 );    /** PRINT prefix **/
      }
// GET IN ROOT 
    if ( vw1 == ID_NOR_A ) /*** Nothing to print ***/
      vw1 = 0;
    if ( sup || sub || vw1 || sub2 )
      h = 0;
    bs = rotsl - jus();
    fwdstp = chrwd[smlfnt] + jus() - rotsr;

    if ( chk_nxto ( fwdstp - bs, vw1, &obsl ) )
      {
        bs = rotsl - jus();
        if ( jd_bug )
          {
            sprintf ( d_bugstr, "BS = %d, obsl = %d\n", bs, obsl );
            wrt_debug ();
          }
        fwdstp = chrwd[smlfnt] + jus() - rotsr;
      }
    if ( rot == ID_NOR_B && sub == ID_SUB_L && !sup ) // Backspace a little
                                                      // more.
      bs = bs + smlconv ( 6 );

    if ( rot == ID_SWIDSH ) // There must be a more elegant way of doing this???
      {
        fwdstp = 72;
        bs = -24;
      }
    backspc ( bs );
    if ( sup )
      {
        prn_chr ( sup, h, 0 ); /** PRINT super join **/
        spcshft = 0;
      }
    btm = logicy; /** Before we start to print the root **/
    obsl = logicx;
    if ( sub )
      {
        if ( sup )
          dwn = chrdrp ( sup ) + chrdrp ( rot );
        else
          dwn = chrdrp ( rot );
        chk_subdrop ();
        if ( rot == ID_SQU_H && sub == ID_SUB_W )
          spcshft = smlconv ( 12 );
        else if ( rot == ID_NOR_NY && sub == ID_SUB_W )
          spcshft = smlconv ( 12 );
        if ( spcshft )
          backspc ( spcshft );
        prn_chr ( sub, h+spcshft, dwn ); /** PRINT subjoin **/
        drp = chrdrp ( sub ) + dwn;
        spcshft = 0;
      }
    if ( sub2 )
      {
        dwn = chrdrp ( sub ) + dwn;
        if ( sub == ID_SUB_R && sub2 == ID_SUB_W ) // grwa, etc
          dwn = dwn - smlconv ( 6 );
        prn_chr ( sub2, h, dwn );  /** PRINT 2nd subjoin **/
        drp = chrdrp ( sub2 ) + dwn;
      }
    if ( sup )
      dwn = chrdrp ( sup );
    else
      dwn = 0;
    if ( !sub )
      {
        if ( sup )
          drp = chrdrp ( sup ) + chrdrp ( rot );
        else
          drp = chrdrp ( rot );
      }
    if ( vw1 )
      {
        if ( vw1 == ID_NOR_U ) /** U **/
          {
            if ( d_bug == 5 )
              {
                sprintf ( d_bugstr, "Print 1st vowel, H = %d\n", h );
                wrt_debug ();
              }
            prn_chr ( vw1, h, drp + 1 ); /** Drop the U down a bit **/
            drp = chrdrp ( vw1 ) + drp;
          }
        else
          {
            if ( d_bug == 5 )
              {
                sprintf ( d_bugstr, "Print 1st vowel, H = %d\n", h );
                wrt_debug ();
              }
            spcshft = chk_spcvowelshift ();   // 14/7/1997
            if ( spcshft )
              backspc ( spcshft );
            prn_chr ( vw1, h+spcshft, 0 ); /** PRINT first vowel **/
          }
      }
    h = fwdstp;
    if ( d_bug == 5 )
      {
        sprintf ( d_bugstr, "Print Root, H = %d\n", h );
        wrt_debug ();
      }

    spcshft = chk_rotshift ();
    if ( spcshft )
      backspc ( spcshft );
    prn_chr ( rot, h, dwn ); /** PRINT root **/
    obsr = logicx;
// End of root syllable, so deal with obscuring characters 
    if ( drp > linspc - bas2top && obblck ) /** Check this **/
      {
        btm = drp + btm;
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Add2obsq2: drp: %d, linspc: %d", drp, linspc );
            wrt_debug ();
          }
        add2obsq ( obsl, obsr, btm );
      }
    if ( vw2 )
      {
        bs = vwlsl[smlfnt] - jus();
        h = chrwd[smlfnt] + jus() - vwlsr[smlfnt];
        if ( vw2 != ID_NOR_A )
          {
            if ( chk_nxto ( h - bs, vw2, &obsl ) )
              {
                bs = vwlsl[smlfnt] - jus();
                h = chrwd[smlfnt] + jus() - vwlsr[smlfnt];
              }
            backspc ( bs );
            prn_chr ( ID_NOR_A, 0, 0 );
            if ( vw2 == ID_NOR_U ) /** U **/
              {
                drp = chrdrp ( ID_NOR_A );
                prn_chr ( vw2, h, drp + 1 ); /** Drop the U down a bit **/
              }
            else
              prn_chr ( vw2, h, 0 );
          }
        else
          {
            if ( chk_nxto ( h - bs, vw2, &obsl ) )
              {
                bs = vwlsl[smlfnt] - jus();
                h = chrwd[smlfnt] + jus() - vwlsr[smlfnt];
              }
            backspc ( bs ); /** NOT SURE ??? **/
            prn_chr ( ID_NOR_A, h, 0 );
          }
        if ( vw3 )
          {
            h = chrwd[smlfnt] + jus() - vwlsr[smlfnt];
            if ( chk_nxto ( h - bs, vw3, &obsl ) )
              {
                bs = vwlsl[smlfnt] - jus();
                h = chrwd[smlfnt] + jus() - vwlsr[smlfnt];
              }
            backspc ( bs ); /** NOT SURE ??? **/
            if ( vw3 != ID_NOR_A )
              {
                prn_chr ( ID_NOR_A, 0, 0 );
                prn_chr ( vw3, h, 0 );
              }
            else
              prn_chr ( ID_NOR_A, h, 0 );
          }
      }
    /** Get suffix **/
    if ( suf1 )
      {
        bs = suf1sl - jus();
        h = chrwd[smlfnt] + jus() - suf1sr;
        if ( suf1 == ID_NOR_G && suf2 == 0 && shd == 2 )
          {
            no1shad = 1;
            h = h + shadspc[smlfnt] + aftshad[smlfnt];
          }
        if ( chk_nxto ( h - bs, 0, &obsl ) )
          {
            bs = suf1sl - jus();
            h = chrwd[smlfnt] + jus() - suf1sr;
            if ( suf1 == ID_NOR_G && suf2 == 0 && shd == 2 )
              {
                no1shad = 1;
                h = h + shadspc[smlfnt] + aftshad[smlfnt];
              }
          }
        backspc ( bs );
        prn_chr ( suf1, h, 0 );       /** PRINT first suffix **/
      }
    if ( suf2 )
      {
        bs = suf2sl - jus();
        h = chrwd[smlfnt] + jus() - suf2sr;
        if ( chk_nxto ( h - bs, 0, &obsl ) )
          {
            bs = suf2sl - jus();
            h = chrwd[smlfnt] + jus() - suf2sr;
          }
        backspc ( bs );
        prn_chr ( suf2, h, 0 );       /** PRINT second suffix **/
      }
    if ( shd )
      {
        if ( suf1 == ID_NOR_NG && !suf2 ||
             ( rot == ID_NOR_NG && !suf1 && !suf2 ) ) // NEW - 15/11/2003
          {
            h = chrwd[smlfnt] + jus() - dotsr[smlfnt];
            bs = dotsl[smlfnt] - intcspc;
            if ( chk_nxto ( h - bs, 0, &obsl ) )
              {
                h = chrwd[smlfnt] + jus() - dotsr[smlfnt];
                bs = dotsl[smlfnt] - intcspc;
              }
            backspc ( bs );
            prn_chr ( ID_DOT, h, 0 ); /** Dot **/
          }
        h = chrwd[smlfnt] + jus() - dotsr[smlfnt] + shadspc[smlfnt];
        h += aftshad[smlfnt];
        backspc ( shdsl[smlfnt] - intcspc - bfrshad[smlfnt] );
        if ( !no1shad) /* If we are not skipping a shad **/
          {
            if ( shd == 64 )   // NEW, 30/5/94 - "Continuation" shad
              {
                h = chrwd[smlfnt] - consr[smlfnt] + shadspc[smlfnt] * 2;
                chk_nxto ( h - aftshad[smlfnt], 0, &obsl );
                prn_chr ( ID_CONTSH, h, 0 ); /** Shad **/
              }
            else
              {
                chk_nxto ( h - aftshad[smlfnt], 0, &obsl );
                if ( shd < 8 )
                  prn_chr ( ID_SHAD, h, 0 ); /** Shad **/
                else
                  prn_chr ( ID_NYISH, h, 0 ); /** Shad **/
              }
          }
        if ( shd == 2 )
          {
            h = chrwd[smlfnt] + jus() - shdsr[smlfnt];
            h += aftshad[smlfnt];
            backspc ( shdsl[smlfnt] - intcspc );
            prn_chr ( ID_SHAD, h, 0 ); /** Shad **/
          }
      }
    else
      {
        h = chrwd[smlfnt] + jus() - dotsr[smlfnt] + 2 * intwspc; 
        backspc ( dotsl[smlfnt] - intwspc );
        prn_chr ( ID_DOT, h, 0 ); /** Dot **/
      }
    strcpy ( lst_wrd, wrd_tst );
    nxt_wrd_font_chg = 0;
  } /** END OF prn_wrd **/

// ROUTINE TO SORT OUT SPECIAL CASES WITH SANSKRIT
void chk_spcskt ( void ) /** SKT **/
// After this call, all NOR characters are SQUashed.
  {
    int i, x, wrd_flg;
    if ( d_bug == 3 )
      {
        sprintf ( d_bugstr, "chk_spcskt, word: %s", wrd_tst );
        wrt_debug ();
      }
/** HUM **/
    if ( strncmp ( wrd_tst, "h", 3 ) == 0 || strncmp ( wrd_tst, "hUM", 3 ) == 0 )
      {
        con[0] = ID_SKT_HUM;
        vwlltr = 0; /** Kill vowels **/
        vwlltr2 = 0;
        vwlsgn = 0;
        numcon = 1;
        return;
      }
    if ( strncmp ( wrd_tst, "y", 4 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_TR;
        con[1] = ID_SQU_Y;
        numcon = 2;
        return;
      }
    if ( strncmp ( wrd_tst, "i", 4 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SKT_SHR;
        con[1] = 0;
        vwlltr = 0; /** Kill vowels **/
        vwlltr2 = 0;
        numcon = 1;
        return;
      }
    if ( strncmp ( wrd_tst, "hya", 4 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_H;
        con[1] = ID_SQU_Y;
        numcon = 2;
        return;
      }
    if ( strncmp ( wrd_tst, "hy", 4 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_H;
        con[1] = ID_SQU_Y;
        numcon = 2;
        return;
      }

    if ( strncmp ( wrd_tst, "hra", 3 ) == 0 &&
         strncmp ( nxt_wrd1, "hr", 3 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_H;
        con[1] = ID_SQU_R;
        numcon = 2;
        return;
      }
    if ( strncmp ( lst_wrd, "hra", 3 ) == 0 &&
         strncmp ( wrd_tst, "hr", 3 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_H;
        con[1] = ID_SQU_R;
        numcon = 2;
        return;
      }

    if ( strncmp ( wrd_tst, "hwa", 3 ) == 0 &&
         strncmp ( nxt_wrd1, "hw", 3 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_H;
        con[1] = ID_SQU_W;
        numcon = 2;
        return;
      }
    if ( strncmp ( lst_wrd, "hwa", 3 ) == 0 &&
         strncmp ( wrd_tst, "hw", 3 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_H;
        con[1] = ID_SQU_W;
        numcon = 2;
        return;
      }
    if ( strncmp ( wrd_tst, "i", 3 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_TR;
        con[1] = ID_SQU_R;
        vwlltr = 0; /** Kill vowels **/
        vwlltr2 = 0;
        numcon = 2;
        return;
      }
    if ( strncmp ( wrd_tst, "", 3 ) == 0 ) // Kalacakra
      {
        con[0] = ID_SQU_TR;
        numcon = 1;
        return;
      }

    for ( i = 0; i < numcon - 1; ++i )
      {
        switch ( con[i] )
          {
/** HR **/
            case ID_NOR_H:
              if ( con[i+1] == ID_NOR_R || con[i+1] == ID_SKT_R )
                {
                  if ( con[i+1] == ID_SKT_R )
                    {
                      if ( strcmp ( nxt_wrd1, "da" ) == 0 )
                        wrd_flg = 1;
                      else
                        wrd_flg = 0;
                      if ( wrd_flg == 0 )
                        break;
                    }

                  con[i] = ID_SKT_HR;
                  if ( con[i+1] == ID_SKT_R )
                    vwlltr = 0; /** Kill vowel if ours is it **/
                  sldsktsyl ( i+1 ); /** Slide Sanskrit syllable **/

                  // NEXT WORD
                  if ( d_bug == 3 )
                    {
                      sprintf ( d_bugstr, "Next Word1 = %s, Word2 = %s\n", 
                                nxt_wrd1, nxt_wrd2 );
                      wrt_debug ();
                    }
                }
              break;
// KY
            case ID_NOR_K:
              if ( con[i+1] == ID_NOR_Y )
                {
                  con[i] = ID_SQU_K;
                  con[i+1] = ID_SUB_Y;
                }
/** KR **/
              else if ( con[i+1] == ID_NOR_R || con[i+1] == ID_SKT_R )
                {
                  con[i] = ID_SQU_K;
                  if ( con[i+1] == ID_SKT_R )
                    vwlltr = 0; /** Kill vowel if ours is it **/
                  con[i+1] = ID_SUB_R;
                }
              break;
/** GR **/
            case ID_NOR_G:  /*** WORKING ON THIS ONE 14/3/91 ****/
              if ( con[i+1] == ID_NOR_R || con[i+1] == ID_SKT_R )
                {
                  con[i] = ID_SQU_G;
                  if ( con[i+1] == ID_SKT_R )
                    vwlltr = 0; /** Kill vowel if ours is it **/
                  con[i+1] = ID_SUB_R;
                }
              break;
/** GHR **/
            case ID_NOR_GH:
              if ( con[i+1] == ID_NOR_R /* || con[i+1] == ID_SKT_R */ )
                {
                  con[i] = ID_NOR_G;
                  con[i+1] = ID_SKT_HR;
                }
              break;
/** TRRA **/
            case ID_NOR_TR:
              if ( con[i+1] == ID_NOR_R || con[i+1] == ID_SKT_R )
                {
                  con[i] = ID_SKT_TRR;
                  if ( con[i+1] == ID_SKT_R )
                    vwlltr = 0; /** Kill vowel if ours is it **/
                  sldsktsyl ( i+1 ); /** Slide Sanskrit syllable **/
                }
              break;
/** DYA **/ /* eg. vidya */
            case ID_NOR_D:
              if ( con[i+1] == ID_NOR_Y )
                {
                  con[i] = ID_SQU_D;  // CHECK - CHANGE MADE HERE
                  con[i+1] = ID_SUB_Y;
                }
              break;
            default:  break;
          }

/** RATA **/
        if ( ( con[i+1] == ID_NOR_R || con[i+1] == ID_SKT_R ) &&
                  ( con[i] == ID_NOR_D || con[i] == ID_NOR_N ||
                    con[i] == ID_NOR_P || con[i] == ID_NOR_PH ||
                    con[i] == ID_NOR_B || con[i] == ID_NOR_M ) )
         /** Check also for number of characters following ?? **/
          {
            if ( con[i+1] == ID_SKT_R )
              vwlltr = 0;    /** Kill vowel if ours is it **/
            con[i+1] = ID_SUB_R; /** Rata **/
          }

      } /**** END OF LOOP ON con[i] ****/
// FROM HERE WE CHECK TOP OF THE SYLLABLE STACK:
    switch ( con[0] )
      {
        case ID_NOR_K:
// KYAI
          if ( con[1] == ID_NOR_Y && vwlsgn == ID_VOW_AI )
            con[1] = ID_SUB_Y;
// KRI -- eg. krita hum phat
          else if ( numcon <= 2 && ( vwlltr == ID_NOR_R || con[1] == ID_SKT_R
                                                                          ) )
            {
              con[0] = ID_RSB_K; // KA ** changed, 2/7/96
              con[1] = ID_SUB_R; /** Rata **/
              numcon = 2;
              vwlltr = 0;
            }
// kc -- eg. wkcitta
          else if ( con[1] == ID_NOR_TS )
            con[1] = ID_SLO_TS;
          break;
        case ID_NOR_KH:
          if ( con[1] == ID_NOR_R )
            con[1] = ID_SUB_R;
          break;
        case ID_NOR_TS:
// cc
          if ( con[1] == ID_NOR_TS && numcon == 2 )
            con[1] = ID_SLO_TS;
// cch
          else if ( con[1] == ID_NOR_TSH && numcon == 2 )
            con[1] = ID_SLO_TSH;
// cyai
          else if ( con[1] == ID_NOR_Y && numcon == 2 )
            con[1] = ID_SUB_Y;
          break;
        case ID_NOR_TSH:
// chch
          if ( con[1] == ID_NOR_TSH && numcon == 2 )
            con[1] = ID_SLO_TSH;
          break;
        case ID_NOR_NY:
// c -- eg. paca
          if ( con[1] == ID_NOR_TS )
            con[1] = ID_SLO_TS;
          break;
        case ID_NOR_NR:
          if ( con[1] == ID_NOR_Y )
            {
              con[0] = ID_SKT_NRY;
              if ( numcon > 2 )
                {
                  for ( x = 1; x < numcon - 1; ++x )
                    con[x] = con[x+1];
                }
              --numcon;
            }
          else if ( con[1] == ID_NOR_R )
            {
              con[0] = ID_SKT_NRR;
              if ( numcon > 2 )
                {
                  for ( x = 1; x < numcon - 1; ++x )
                    con[x] = con[x+1];
                }
              --numcon;
            }
          break;
        case ID_NOR_DZ:
// JRA -- eg vajra
          if ( con[1] == ID_NOR_R )
            {
              con[0] = ID_SKT_JR;
              if ( numcon > 2 )
                {
                  for ( x = 1; x < numcon - 1; ++x )
                    con[x] = con[x+1];
                }
              --numcon;
            }
// JVA
          else if ( con[1] == ID_SKT_V )
            con[1] = ID_SUB_W; // Wazur
          break;
        case ID_NOR_DR:
// DrRI
          if ( vwlltr == ID_NOR_R && numcon <= 2 )
            {
              con[0] = ID_DRRI; /** Compound **/
              vwlltr = 0;
            }
          else if ( vwlltr == ID_SSB_R && numcon <= 2 ) // 11/9/97
            {
              con[0] = ID_DRRI; /** Compound **/
              vwlltr = 0;
            }
          else if ( con[1] == ID_NOR_Y && numcon == 2 ) // 14/9/97
            {
              con[0] = ID_SKT_DRY; /** Compound **/
              --numcon;
            }
          break;
        case ID_NOR_D:    // This is to fix "padma"
// DMA
          if ( con[1] == ID_NOR_M && numcon == 2 )
            {
              con[0] = ID_SQU_D;
              con[1] = ID_SQU_M;
            }
          break;
        case ID_NOR_DH:
// DHRA
          if ( con[1] == ID_NOR_R || con[1] == ID_SKT_R )
            {
              con[0] = ID_SQU_D;
              if ( con[1] == ID_SKT_R )
                vwlltr = 0; /** Kill vowel if ours is it **/
              con[1] = ID_SQU_HR;        // 86
            }
// DHYA
          else if ( con[1] == ID_NOR_Y )
            {
              con[0] = ID_SQU_D;
              con[1] = ID_SQU_HY;
            }
          break;
        case ID_NOR_PH: // NEW - 11/5/2001
// PHYWA
          if ( con[1] == ID_NOR_Y && con[2] == ID_NOR_W )
            {
              con[1] = ID_SUB_Y;
              con[2] = ID_SUB_W;
            }
          break;
        case ID_NOR_B:
// BYA
          if ( con[1] == ID_NOR_Y )
            {
              con[0] = ID_SQU_B;
              con[1] = ID_SUB_Y;
            }
          break;
        case ID_NOR_BH:
// BHYA
          if ( con[1] == ID_NOR_Y )
            {
              con[0] = ID_SQU_B;
              con[1] = ID_SQU_HY;
            }
          break;
        case ID_NOR_M:
// MVA - eg: samvara
          if ( con[1] == ID_NOR_V )
            con[1] = ID_NOR_B;
// MYA
          else if ( con[1] == ID_NOR_Y )
            con[1] = ID_SUB_Y;
          break;
        case ID_NOR_W:
// WW --  needed for "dus 'khor".
          if ( strncmp ( wrd_tst, "wwa", 3 ) == 0 )
            {
              con[0] = ID_SQU_W;
              con[1] = ID_SQU_W;
            }
// WV --  needed for "rdo rje shugs".
          else if ( con[1] == ID_SKT_V )
            con[1] = ID_SUB_W; // Wazur
          break;
        case ID_NOR_Y:
// YV  --  needed for "rdo rje shugs".
          if ( con[1] == ID_SKT_V )
            con[1] = ID_SUB_W;        // Wazur
          break;
        case ID_NOR_R:
// RDH
          if ( con[1] == ID_NOR_DH )
            {
              con[0] = ID_SUP_R; /** Rago **/
            }
// RVE
          // This one for a particular case in "rdo rje shugs":
          else if ( con[1] == ID_SKT_V && numcon == 2 && vwlsgn == ID_VOW_E )
            con[1] = ID_SUB_W;
// RVA
          else if ( con[1] == ID_SKT_V )
            con[1] = ID_NOR_B; /** BA - "sarva" **/
// RYA
          else if ( con[1] == ID_NOR_Y )
            con[1] = ID_SUB_Y;

          if ( con[1] == ID_NOR_K ||
               con[1] == ID_NOR_G || con[1] == ID_NOR_GH ||
               con[1] == ID_NOR_C || con[1] == ID_NOR_CH ||
               con[1] == ID_NOR_T ||
               con[1] == ID_NOR_TH || con[1] == ID_NOR_D ||
               con[1] == ID_NOR_NR || con[1] == ID_NOR_N ||
               con[1] == ID_NOR_DZ || con[1] == ID_NOR_M ||
               con[1] == ID_NOR_TS || con[1] == ID_NOR_TSH ||
               con[1] == ID_NOR_B || con[1] == ID_NOR_BH )
            con[0] = ID_SUP_R; /** Rago **/
          break;
// I
        case ID_SKT_R:
          if ( vwlltr == ID_SSB_R && numcon == 1 ) // XXXX 11/9/97
            {
              con[0] = ID_NOR_R;
              vwlltr = 0;
            }
          else if ( vwlltr2 == ID_SQU_A && numcon == 1 ) // XXXX 11/9/97
            {
              con[0] = ID_NOR_R;
              vwlltr = 0;
            }
          break;
        case ID_NOR_L:
// LV  --  needed for "rdo rje shugs".
          if ( con[1] == ID_SKT_V )
            con[1] = ID_SUB_W; // Wazur
          break;
        case ID_SKT_LI:
          if ( vwlltr == ID_SSB_L && numcon == 1 ) // XXXX 11/9/97
            {
              con[0] = ID_NOR_L;
              vwlltr = 0;
            }
          else if ( vwlltr2 == ID_SQU_A && numcon == 1 ) // XXXX 11/9/97
            {
              con[0] = ID_NOR_L;
              vwlltr = 0;
            }
          break;
        case ID_NOR_SH:
// c
          if ( con[1] == ID_NOR_TS )
            con[1] = ID_SLO_TS;
          break;
        case ID_SKT_SZH:
// SZHYA
          if ( con[1] == ID_NOR_Y )
            {
              con[0] = ID_SZHYA; /** Compound **/
              con[1] = 0;
              --numcon; // Perhaps should slide instead??
            }
          break;
        case ID_NOR_H:
// HV --  needed for "rdo rje shugs".
          if ( con[1] == ID_SKT_V )
            con[1] = ID_SUB_W;      // Wazur
          break;
        case ID_SKT_AVA:  // NOT FINISHED YET: - look at doubles
// 'O -- eg. hoh'o
          if ( vwlsgn == ID_VOW_O || vwlsgn == ID_VOW_I || vwlsgn == ID_VOW_U )
            {
              con[0] = ID_NOR_A;
              vwlltr = 0;
            }
          break;
        default: break;
      } /**** END OF SWITCH ON con[0] ****/

// CHECK BOTTOM OF STACK:

    if ( numcon >= 2 )
      {
      switch ( con[numcon-2] )
      {
        case ID_NOR_TR:
// TrV
          if ( con[numcon-1] == ID_NOR_V ||
               con[numcon-1] == ID_SKT_V || con[numcon-1] == ID_NOR_W )
            {
              con[numcon-2] = ID_SKT_TRW;
              con[numcon-1] = 0;
              --numcon;
            }
          break;
        case ID_NOR_DR:
          if ( con[numcon-1] == ID_NOR_Y )
            {
              con[numcon-2] = ID_SKT_DRY; /** Compound **/
              con[numcon-1] = 0;
              --numcon;
            }
          break;
        case ID_NOR_T:
// TV
          if ( con[numcon-1] == ID_NOR_V || con[numcon-1] == ID_SKT_V )
            {
              con[numcon-2] = ID_SKT_TW; /** T + Wazur **/
              con[numcon-1] = 0;
              --numcon;
            }
// TR
          else if ( con[numcon-1] == ID_NOR_R || con[numcon-1] == ID_SKT_R )
            {
              con[numcon-2] = ID_SKT_TR; /** T + Rata **/
              if ( con[numcon-1] == ID_SKT_R )
                vwlltr = 0; /** Kill vowel if ours is it **/
              con[numcon-1] = 0;
              --numcon;
            }
// TY
          else if ( con[numcon-1] == ID_NOR_Y )
            {
              con[numcon-2] = ID_SKT_TY; /** T + YATA **/
              con[numcon-1] = 0;
              --numcon;
            }
          break;
        case ID_NOR_TH:
// THR
          if ( con[numcon-1] == ID_NOR_R || con[numcon-1] == ID_SKT_R )
            {
              if ( con[numcon-1] == ID_SKT_R )
                vwlltr = 0; /** Kill vowel if ours is it **/
              con[numcon-1] = ID_SUB_R; /** Rata **/
            }
          break;
        case ID_NOR_D:
// DV
          if ( con[numcon-1] == ID_NOR_V || con[numcon-1] == ID_SKT_V )
            {
              con[numcon-2] = ID_SQU_D; /** T + Wazur **/
              con[numcon-1] = ID_SUB_W; /* Wazur */
            }
          break;
        case ID_NOR_DH:
// DHWA
          if ( con[numcon-1] == ID_NOR_W || con[numcon-1] == ID_NOR_V )
            con[numcon-1] = ID_SUB_W; // Wazur
// DHRA
          else if ( con[numcon-1] == ID_NOR_R || con[numcon-1] == ID_SKT_R )
                {
                  con[numcon-2] = ID_SQU_D;
                  if ( con[numcon-1] == ID_SKT_R )
                    vwlltr = 0; /** Kill vowel if ours is it **/
                  con[numcon-1] = ID_SQU_HR; // 86
                }
          break;
        case ID_NOR_BH:
/** BHRA **/
          if ( con[numcon-1] == ID_NOR_R || con[numcon-1] == ID_SKT_R )
            {
              con[numcon-2] = ID_SKT_BHR;
              if ( con[numcon-1] == ID_SKT_R )
                vwlltr = 0; /** Kill vowel if ours is it **/
              sldsktsyl ( numcon-1 ); /** Slide Sanskrit syllable **/
            }
          break;
        case ID_NOR_M:
// eg: lakmyai
          if ( con[numcon-1] == ID_NOR_Y )
            con[numcon-1] = ID_SUB_Y;
          break;
        case ID_NOR_SH:
// SHR
          if ( con[numcon-1] == ID_NOR_R )
            {
              con[numcon-2] = ID_SKT_SHR;
              sldsktsyl ( numcon-1 ); /** Slide Sanskrit syllable **/
            }
// SHVA -- eg. ishvara
          else if ( con[numcon-1] == ID_SKT_V )
            con[numcon-1] = ID_SUB_W; /** Wazur **/
// SHYA
          else if ( con[numcon-1] == ID_NOR_Y && numcon < 3 )
            {
              con[numcon-2] = ID_RSB_SH;
              con[numcon-1] = ID_SUB_Y;
            }
          break;
        case ID_SKT_KZH:
// akye  // Problem here - conflict with many Kalacakra syllables:
          if ( con[numcon-1] == ID_NOR_Y && vwlsgn != 0 )
            {
              con[numcon-2] = ID_NOR_K;
              con[numcon-1] = ID_SZHYA;
            }
          break;
        case ID_NOR_S:
// SVA  --- eg. svabhava
          if ( con[numcon-1] == ID_SKT_V )
            con[numcon-1] = ID_SUB_W; /** Wazur **/
// SRA
          else if ( con[numcon-1] == ID_NOR_R )
            con[numcon-1] = ID_SUB_R;
// SYA
          else if ( con[numcon-1] == ID_NOR_Y && numcon < 3 )
            con[numcon-1] = ID_SUB_Y; /** Compound **/
          break;
        case ID_NOR_H:
// HYA
          if ( con[numcon-1] == ID_NOR_Y )
            {
              con[numcon-2] = ID_SKT_HY; /** Compound **/
              con[numcon-1] = 0;
              --numcon;
            }
          break;
        default: ; break;
      }
      } // End of conditional switch on "numcon"
/** TRV **/
    if ( numcon >= 3 &&
         con[numcon-3] == ID_NOR_T && con[numcon-2] == ID_NOR_R &&
            ( con[numcon-1] == ID_NOR_V ||
              con[numcon-1] == ID_SKT_V || con[numcon-1] == ID_NOR_W ) )
      {
        con[numcon-3] = ID_SKT_TR;
        con[numcon-2] = ID_SUB_W;
        --numcon;
      }
  } // END - chk_spcskt ()
  