/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
#include <io.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <dos.h>
#include "tform.h"
#include "tibform.ext"
#include "tibform.ids"

// Locals:

int strtlin;
int fntsav;

struct jus_qu
  {
    int xl;      /** Start of justification group **/
    int xr;      /** End of justification group **/
    int juscall; /** Number of calls made to jus along the way **/
    int jusfil;  /** Number of dots to be made up **/
  } js[24];

struct obsc_dat ob_detT[45]; // FOR TESTING ONLY.
struct jus_qu jsT[24];

// Leave these for now. Moving them upset justification:

int jussiz = 23;
int jussav;
int justol = 5;
int njfils;

struct  c_map c_mp[155] = {  0,   0,   0,
/** Normal characters **/  37,  33,  53,    /**   1 ka **/
                           37,  34,  28,    /**   2 kha **/
                           37,  35,  53,    /**   3 ga **/
                           37,  36,  53,    /**   4 gha **/
                           37,  37,  31,    /**   5 nga **/
                           37,  38,  31,    /**   6 ca **/
                           37,  39,  32,    /**   7 cha **/
                           37,  40,  34,    /**   8 ja **/
                           37,  41,  53,    /**   9 nya **/
                           37,  42,  53,    /**  10 tra **/
                           37,  43,  40,    /**  11 thra **/
                           37,  44,  53,    /**  12 dra **/
                           37,  45,  53,    /**  13 drha **/
                           37,  46,  53,    /**  14 nra **/
                           37,  47,  53,    /**  15 ta **/
                           37,  48,  40,    /**  16 tha **/
                           37,  49,  53,    /**  17 da **/
                           37,  50,  53,    /**  18 dha **/
                           37,  51,  53,    /**  19 na **/
                           37,  52,  33,    /**  20 pa **/
                           37,  53,  32,    /**  21 pha **/
                           37,  54,  35,    /**  22 ba **/
                           37,  55,  52,    /**  23 bha **/
                           37,  56,  32,    /**  24 ma **/
                           37,  57,  31,    /**  25 tsa **/
                           37,  58,  32,    /**  26 tsha **/
                           37,  59,  34,    /**  27 dza **/
                           37,  60,  59,    /**  28 dzha **/
                           37,  61,  43,    /**  29 wa **/
                           37,  62,  52,    /**  30 zha **/
                           37,  63,  34,    /**  31 za **/
                           37,  64,  34,    /**  32 'a **/
                           37,  65,  34,    /**  33 ya **/
                           37,  66,  37,    /**  34 ra **/
                           37,  67,  34,    /**  35 la **/
                           37,  68,  53,    /**  36 sha **/
                           37,  69,  53,    /**  37 szha **/
                           37,  70,  32,    /**  38 sa **/
                           37,  71,  49,    /**  39 ha  **/
                           37,  72,  32,    /**  40 a   **/
                           37,  73,  60,    /**  41 ksa **/ // CHECK??
                           37,  74,   0,    /**  42 **/
                           37,  75,  63,    /**  43 nrya **/
                           37,  76,  50,    /**  44 avagraha **/
                           37,  77,  31,    /**  45 K **/
/*** Sanskrit IDS for **/  37,  78,  31,    /**  46 KH **/
                           37,  79,  31,    /**  47 G **/
/* squashed characters*/   37,  80,  50,    /**  48 GH **/
/* Quick fix, do again */  37,  81,  24,    /**  49 NG **/
                           37,  82,  24,    /**  50 C **/
                           37,  83,  25,    /**  51 CH **/
                           37,  84,  26,    /**  52 J **/
                           37,  85,  38,    // 42 /** 53 NY **/
                           37,  86,  36,    /**  54 TR **/
                           37,  87,  31,    /**  55 THR **/
                           37,  88,  27,    /**  56 DR **/
                           37,  89,  50,    /**  57 DRH **/
                           37,  90,  37,    /**  58 NR **/
                           37,  91,  36,    /**  59 T **/
                           37,  92,  31,    /**  60 TH **/
                           37,  93,  27,    /**  61 D **/
                           37,  94,  47,    /**  62 DH **/
                           37,  95,  37,    /**  63 N **/
                           37,  96,  25,    /**  64 P **/
                           37,  97,  24,    /**  65 PH **/
                           37,  98,  26,    /**  66 B **/
                           37,  99,  45,    /**  67 BH **/
                           37, 100,  23,    /**  68 M **/
                           37, 101,  24,    /**  69 TS **/
                           37, 102,  25,    /**  70 TSH **/
                           37, 103,  26,    /**  71 DZ **/
                           37, 104,  48,    /**  72 DZH **/
                           37, 105,  33,    /**  73 W **/
                           37, 106,  24,    /**  74 ZH **/
                           37, 107,  24,    /**  75 Z **/
                           37, 108,  22,    /**  76 'A **/  // 24
                           37, 109,  24,    /**  77  Y **/
                           37, 110,  24,    /**  78  R **/ // 26
                           37, 111,  25,    /**  79  L **/
                           37, 112,  34,    /**  80  SH **/
                           37, 113,  34,    /**  81 SZH **/
                           37, 114,  23,    /**  82  S **/
                           37, 115,  35,    /**  83  H **/
                           37, 116,  23,    /**  84 A **/
                           37, 117,  48,    /**  85 KZA **/
                           37, 118,  43,    /**  86 hra **/ // 43
                           37, 119,  45,    /**  87 shra **/
                           37, 120,  63,    /**  88 szhya **/
                           37, 121,  48,    /**  89 drri  **/
                           37, 122,  60,    /**  90 tw **/
                           37, 123,  65,    /**  91 bhy **/
                           37, 124,  59,    /**  92 bhr **/
                           37, 125,  25,    /**  93 po **/
                           37, 126,  41,    /**  94 TRR **/
/** Root, super only **/   38,  33,  25,    /**  95 JA  **/
                           38,  34,  39,    /**  96 NYA **/  
                           38,  35,  39,    /**  97 DA  **/
                           38,  36,  39,    /**  98 DRA **/
                           38,  37,  56,    /**  99 NRR **/
/** Roots, sub only **/    38,  38,  39,    /** 100 KA  **/
                           38,  39,  39,    /** 101 GA  **/
                           38,  40,  44,    /** 102 DA  **/
                           38,  41,  43,    /** 103 NA  **/
                           38,  42,  40,    /** 104 ZHA **/
                           38,  43,  40,    /** 105 SHA **/
                           38,  44,  31,    /** 106 SLO_TS **/
                           38,  45,  32,    /** 107 SLO_TSH **/
                           38,  46,  34,    /** 108 SLO_DZ **/
                           38,  47,  30,    /** 109 DA  **/
                           38,  48,  23,    /** 110 PA  **/
                           38,  49,  24,    /** 111 RA  **/ 
                           38,  50,  20,    /** 112 LA  **/
/** Actual superjoins **/  38,  51,  19,    /** 113 RAGO, medium **/
                           38,  52,  13,    /** 114 RAGO **/
                           38,  53,  22,    /** 115 LAGO **/
                           38,  54,  20,    /** 116 SAGO **/
/** Actual subjoins **/    38,  55,  16,    /** 117 YATA **/
                           38,  56,  19,    /** 118 RATA **/
                           38,  57,  29,    /** 119 LATA **/
                           38,  58,  27,    /** 120 'A  **/
                           38,  59,  21,    /** 121 WAZUR **/
                           38,  60,  50,    /** 122 HRA **/
                           38,  61,  53,    /** 123 SHRA **/
                           38,  62,  40,    /** 124 HUM **/
                           38,  63,  44,    /** 125 DZR **/
                           38,  64,  47,    /** 126 TY **/
                           38,  65,  52,    /** 127 TR **/
                           38,  66,  42,    /** 128 HY **/
                           38,  67,  53,    /** 129 HY - larger **/
                           38,  68,   0,    /** 130 i   **/
                           38,  69,   0,    /** 131 rev i sign **/
                           38,  70,   0,    /** 132 u   **/
                           38,  71,   0,    /** 133 e   **/
                           38,  72,   0,    /** 134 ai  **/
                           38,  73,   0,    /** 135 o   **/
                           38,  74,   0,    /** 136 au  **/
                           38,  75,  40,    /** 137 h dot **/
                           38,  76,   0,    /** 138 nyishad */
                           38,  77,   0,    /** 139     **/
                           38,  78,   0,    /** 140     **/
                           38,  79,   0,    /** 141 Start Page 1 **/
                           38,  80,   0,    /** 142 Start Page 2 **/
                           38,  81,   0,    /** 143 Start Page 3 **/
                           38,  82,   0,    /** 144 .   **/
                           38,  83,   0,    /** 145 Shad **/
                           38,  84,   0,    /** 146 Anusvara **/
                           38,  85,  -4,    /** 147 Anusvara + zla **/
                           38,  86,   0,    /** 148 sbrul shad **/
                           38,  87,   0,    /** 149 cont, shad **/
                           38,  88,   0,    /** 150 **/
                           38,  89,  54,    // 151 trw
                           38,  90,  63,    /** 152 dry **/
                           38,  91,  60,    /** 153 fh **/
                           38,  92,  49 };  /** 154 fhs **/

/*** ROUTINE TO DETERMINE SPACING FOR A CHARACTER ***/

void getchrspc ( int chr, int *lft, int *rgt )
  {
    if ( !smlfnt )
      {
        if ( c_mp[chr].fnt == 37 )
          {
            *lft = f37_pa[c_mp[chr].chr - 33].i[0];
            *rgt = f37_pa[c_mp[chr].chr - 33].i[1];
          }
        else if ( c_mp[chr].fnt == 38 )
          {
            *lft = f38_pa[c_mp[chr].chr - 33].i[0];
            *rgt = f38_pa[c_mp[chr].chr - 33].i[1];
          }
        else
          {
            sprintf ( d_bugstr, "1: SPACING NOT KNOWN, chr = %d", chr );
            wrt_debug ();
            sprintf ( d_bugstr, "Queue = %d, word = %s", lstqfch, wrd_tst );
            wrt_debug ();
          }
      }
    else
      {
        if ( c_mp[chr].fnt == 37 )
          {
            *lft = f37_pb[c_mp[chr].chr - 33].i[0];
            *rgt = f37_pb[c_mp[chr].chr - 33].i[1];
          }
        else if ( c_mp[chr].fnt == 38 )
          {
            *lft = f38_pb[c_mp[chr].chr - 33].i[0];
            *rgt = f38_pb[c_mp[chr].chr - 33].i[1];
          }
        else
          {
            sprintf ( d_bugstr, "2: SPACING NOT KNOWN, chr = %d, ", chr );
            wrt_debug ();
            sprintf ( d_bugstr, "Queue = %d, word = %s", lstqfch, wrd_tst );
            wrt_debug ();
          }
      }
  }

/** ROUTINE TO GET THE DROP FOR A CHARACTER **/
int chrdrp ( int i )
  {
    int x;
    x = smlconv ( c_mp[i].drp );
    return ( x );
  }

int anlystitp ( int * i )
  {
    int more = 1;
    if ( titpfnd == 0 && titpageon == 0 )
      {
        juschk = 0;
        jusdo = 0;
        titpfnd = 1;
      }
    else
      {
        titpend = 1;
        if ( portprn )
          titpfnd = 0;   // IMPORTANT CHANGE - 29/5/94 --XXXXX--
        more = 0;  /** End of Title **/
        (*i)++; /** Put in queue, but nothing to print **/
      }
    return ( more );
  } // END - anlystitp ( )

void anlyshead ( void )
  {
    fntsav = smlfnt;
    smlfnt = 0;
    prn_chr ( ID_SRTPG_1, chrwd[smlfnt], 0 );
    prn_chr ( ID_SRTPG_2, 40, 0 );
    prn_chr ( ID_SRTPG_3, 40, 0 );
    prn_chr ( ID_SHAD, 200, 0 );
    prn_chr ( ID_SHAD, 20, 0 );
    strtlin = 0;
    smlfnt = fntsav;
  }

void add2obsq ( int l, int r, int btm )
  {
    int j;
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "add2obsq: Left: %d, Right: %d, btm = %d\n", l, r, btm );
        wrt_debug ();
      }
    ob_det[obptr].left = l;
    ob_det[obptr].right = r;
    ob_det[obptr++].depth = btm;
    if ( d_bug == 4 )
      {
        j = obptr - 1;
        sprintf ( d_bugstr,
              "New Element %d, Depth = %d, Left = %d, Right = %d\n",
            j, ob_det[j].depth, ob_det[j].left, ob_det[j].right );
        wrt_debug ( );
      }
    if ( obptr >= MAX_OB_DET )
      {
        sprintf ( d_bugstr, "QUEUE BLOWN FOR OBSCURING CHARACTERS! obptr = %d", obptr );
        wrt_debug ( );
        exit (0);
      }
  } // END - add2obsq ()

void add2jus ( int r, int jc, int fil )
  {
    int j;
    js[jusptr].xr = r;
    js[jusptr].juscall = jc;
    js[jusptr].jusfil = fil;
    ++jusptr;
    if ( fil < 0 )
      {
        if ( jd_bug )
         {
           sprintf ( d_bugstr, "ALERT - NEGATIVE JUSTIFICATION FILL\n" );
           wrt_debug ();
           for ( j = 0; j < jusptr; ++j )
             {
               sprintf ( d_bugstr, "\nJUS %d, Left = %d, Right = %d, Fills = %d, Calls = %d.\n",
                         j, js[j].xl, js[j].xr, js[j].jusfil, js[j].juscall );
               wrt_debug ();
             }
         }
      }
    if ( jusptr > jussiz )
      {
        sprintf ( d_bugstr, "BLOWN JUSTIFICATION QUEUE, jusptr: %d.", jusptr );
        wrt_debug ();
      }
  } // END - add2jus ()

int jus ( void )
  {
    int c, n;
    if ( !juson || titpfnd )
      return ( intcspc );
    else if ( jusdo )
      {
        n = 0;
        c = intcspc;
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Call to JUS." );
            wrt_debug ( );
          }

        if ( !juscalf && jd_bug )
          {
            sprintf ( d_bugstr, "Word: %s, Fillf = %d, Callf = %d.", wrd_tst,
                    js[jusptr].jusfil, js[jusptr].juscall );
            wrt_debug ();
          }

        if ( js[jusptr].jusfil >= 0 )
          {
           juscalf += js[jusptr].jusfil;
           ++juscall;
           if ( jd_bug )
             {
               sprintf ( d_bugstr, "JUSFILF = %d, JUSCALF = %d, dif = %d\n",
                         jusfilf, juscalf, juscalf - jusfilf );
               wrt_debug ();
             }
          jusx:
          if ( jusfilf < juscalf )
            {
              ++njfils;
              if ( jd_bug || d_bug == 4 )
                {
                  sprintf ( d_bugstr, "Word: %s CALL: %d, ADDING A SPACE - %d\n", 
                            wrd_tst, juscall, njfils );
                  wrt_debug ();                             
                }
              ++n;
              ++c;
              jusfilf += js[jusptr].juscall;
              if ( n < 5 )
                goto jusx;
            }
          }
        else // IF NEGATIVE.
          {
            juscalf -= js[jusptr].jusfil;
            ++juscall;
            if ( jd_bug )
              {
                printf ( "JUSFILF = %d, JUSCALF = %d, dif = %d\n",
                         jusfilf, juscalf, juscalf - jusfilf );
                wrt_debug ();                          
              }
           if ( jusfilf < juscalf )
             {
               ++njfils;
               if ( jd_bug || d_bug == 4 )
                 {
                   sprintf ( d_bugstr,
                         "Word: %s CALL: %d, SUBTRACTING SPACE - %d\n",
                                wrd_tst, juscall, njfils );
                   wrt_debug ( );
                 }
              ++n;
              --c;
              jusfilf += js[jusptr].juscall;
            }
          }
        return ( c );
      }
    else
      {
        ++juscall;
        if ( jd_bug )
          {
            sprintf ( d_bugstr, "Word: %s, Call = %d.", wrd_tst, juscall );
            wrt_debug ();
          }
        return ( intcspc );
      }
  }

int compard ( struct obsc_dat * o1, struct obsc_dat * o2 )
  {
    return ( o2->depth - o1->depth );
  }

int comparl ( struct obsc_dat * o1, struct obsc_dat * o2 )
  {
    return ( o1->left - o2->left );
  }

/*** SORT OUT THE OBSCURE QUEUE ***/
void sort_obs ( int y )
  {
    int ytgt, j;
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "Before removing unecessary elements:" );
        wrt_debug ();//       
        for ( j = 0; j < obptr; ++j )
          {
            sprintf ( d_bugstr, "Element %d, Depth = %d, Left = %d, Right = %d.", j,
                      ob_det[j].depth, ob_det[j].left, ob_det[j].right );
            wrt_debug ();
          }
      }
    ytgt = y - bas2top;
    if ( obptr > 1 )
      {
//tf2a.c(508): Note! I2003: source conversion type is 'int (*)(struct obsc_dat *__p1,struct obsc_dat *__p2)'
//tf2a.c(508): Note! I2004: target conversion type is 'int (*)(void const *__p1,void const *__p2)'
        qsort ( (void*) ob_det, (size_t) obptr, (size_t) sizeof (ob_det[0]),
                compard );
        j = 0;
        do
          {
            if ( ob_det[j].depth < ytgt )
              obptr = j;
            ++j;
          } while ( j < obptr );
      }
    else
      {
        if ( ob_det[0].depth < ytgt )
          obptr = 0;
      }
    if ( obptr > 1 )
      qsort ( (void*) ob_det, (size_t) obptr, (size_t) sizeof (ob_det[0]),
            comparl );
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "After sorting along the line:" );
        wrt_debug ();
        for ( j = 0; j < obptr; ++j )
          {
            sprintf ( d_bugstr, "Element %d, Depth = %d, Left = %d, Right = %d", 
                      j, ob_det[j].depth, ob_det[j].left, ob_det[j].right );
            wrt_debug ();
          }
      }
    prvobptr = obptr;
  }

int chk_nxto ( int h, int vwl, int *ol )
{
  int y, res, jfill;
  int gaptoleave;
  int vwlspc, oblftmg, obrgtmg;
  if ( nxtobj >= prvobptr )  // Should never be greater, but just in case!
    return ( 0 );
  oblftmg = 3;    // tolerances left and right - 3
  obrgtmg = 3;    // 3
  res = 0;

  if ( d_bug == 4 && logicx > 0 && 0 )
    {
      sprintf ( d_bugstr, "CHK_NXTO - H = %d, Logicx = %d, ", h, logicx );
      wrt_debug ( );
      sprintf ( d_bugstr, "Object Left = %d, Right = %d.\n",
                ob_det[nxtobj].left, ob_det[nxtobj].right );
      wrt_debug ( );
    }
// prvobptr == number of objects in list.
// FIRST CASE - we have not yet reached next object:
obtryagain:
  if ( ob_det[nxtobj].left >= logicx + h - obrgtmg )
    return ( res );
// SECOND CASE - we have gone beyond next object:
  else if ( ob_det[nxtobj].right <= logicx + oblftmg )
    {
      ++nxtobj;
      if ( nxtobj == prvobptr )
        return ( res );
      else
        goto obtryagain;
    }
// THIRD CASE - we are right in the next object:
//if ( ( nxtobj < prvobptr ) &&
//     ( ob_det[nxtobj].left < logicx + h - obrgtmg ) &&
//     ( ob_det[nxtobj].right > logicx + oblftmg ) )
//  goto xntomov;
xntomov:
  vwlspc = smlconv ( 14 ); // CHANGED FROM 12 - 30/5/94
  y = logicy;

  if ( vwl == ID_NOR_O || vwl == ID_NOR_I || vwl == ID_NOR_E ||
       vwl == ID_VOW_AU || vwl == ID_VOW_AI || vwl == ID_SKT_R )
    y = y - bas2top + vwlspc;
  if ( ob_det[nxtobj].depth > y )
    {
/** THIS IS WHERE WE MOVE FORWARD PAST AN OBSCURING CHARACTER **/
      if ( juson )
        {
          if ( juschk )  // First passes
            { /** Here, we add to the jus queue **/

              jfill = ob_det[nxtobj].left - logicx - justol;
              add2jus ( logicx, juscall, jfill ); // INCREMENTS jusptr.
              if ( jfill < 0 )
                {
                  sprintf ( d_bugstr, "Negative JFILL:" ); // XXXXXX
                  wrt_debug ( );
                  sprintf ( d_bugstr, "OLFT = %d, Logicx = %d, Justol = %d.",
                    ob_det[nxtobj].left, logicx, justol );
                  wrt_debug ( );
                }
              juscall = 0;
              js[jusptr].xl = ob_det[nxtobj].right;
            }
          else if ( jusdo )  // Second passes
            {
              if ( jd_bug )
                {
                  sprintf ( d_bugstr, "OPTR = %d, JPTR = %d.", nxtobj, jusptr );
                  wrt_debug ();
                  sprintf ( d_bugstr, "H = %d, Logicx = %d, ", h, logicx );
                  wrt_debug ();
                  sprintf ( d_bugstr, "Object Left = %d, Right = %d.",
                            ob_det[nxtobj].left, ob_det[nxtobj].right );
                  wrt_debug ();
                }
              ++jusptr;
              if ( jusptr > jussiz )
                {
                  sprintf ( d_bugstr, "BLOWN JUSTIFICATION QUEUE, jusptr: %d", jusptr );
                  wrt_debug ();
                }
/** Add here setting of information for jus, for how to add points **/
              juscall = 0;
              juscalf = 0;
              jusfilf = 0;
              njfils = 0;
            }
          if ( d_bug == 4 )
            {
               sprintf ( d_bugstr, "Moving past object:" );
               wrt_debug ( );
               sprintf ( d_bugstr,
            "OPTR = %d, JPTR = %d, Logicx = %d, Object Left = %d, Right = %d.",
               nxtobj, jusptr, logicx, ob_det[nxtobj].left,
                                              ob_det[nxtobj].right );
               wrt_debug ( );
            }
        } // END OF - if ( juson )
      gaptoleave = smlconv ( 6 ); // This is new - 15/9/1997
      if ( d_bug == 4 )
        {
          sprintf ( d_bugstr, "Call 3 to setcurpos: X: %d, Y: %d.", ob_det[nxtobj].right + gaptoleave, logicy );
          wrt_debug ();
        }    
      setcurpos ( ob_det[nxtobj].right + gaptoleave, logicy ); // RESET
      *ol = ob_det[nxtobj].right;
      res = 23;
    } // END OF - if ( ob_det[nxtobj].depth > y )
  ++nxtobj;
// NEW 28/5/94:
  if ( nxtobj >= prvobptr )  // Should never be greater, but just in case!
    return ( res );

  if ( ( ob_det[nxtobj].left < logicx + h - obrgtmg ) &&
       ( ob_det[nxtobj].right > logicx + oblftmg ) )
    goto xntomov;
  return ( res );
} // END OF - chk_nxto

// Returns number of words to print
int anlslin ( int sq )
  {
    int cq, more, j, obbsav, jfill;
    int lastx, lstcall, newlin, topx;

    wd_qu[sq].topx = 0;

    topx = txtrct.right;

    noprint = 1;  /** Turn off actual printing **/
    if ( juson )
      {
        juschk = 1; // Turn on, show queue is being built
        obbsav = obblck;
        obblck = 0;  // Prevents adding to obscure queue
        jusptr = 0; /** CAREFUL **/
        js[jusptr].xl = txtrct.left;
      }
    else
      juschk = 0; /** Turn off justification checking **/
//adoagain:
    jusdo = 0;
    juscall = 0;
    jusptr = 0;  /** CAREFUL **/
    newlin = 0;
    cq = sq;
    njfils = 0;
    strtlin = 1;
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "Call 4 to setcurpos: X: %d, Y: %d.", txtrct.left, pcurly );
        wrt_debug ();
      }        
    setcurpos ( txtrct.left, pcurly );
    if ( d_bug > 1 )
      {
        sprintf ( d_bugstr, "anlslin: First Pass." );
        wrt_debug ( );
      }
    more = 1;
    lastx = 0;
    nxtobj = 0;
    if ( titpageon )
      {
        ++titnumlines;
        if ( d_bug > 1 )
          {
            sprintf ( d_bugstr, "TITLE PAGE LINE COUNT = %d", titnumlines );
            wrt_debug ( );
          }
      }
    while ( more )
      {
        get_que ( cq );
        smlfnt = fnt;
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "anlslin: CQ: %d, WORD = %s, flgs: %d", 
                      cq, wrd_tst, flgs );
            wrt_debug ( );
          }
        if ( flgs == 1 )  /** New line required **/
          {
            more = 0;
            ++cq;
            newlin = 1;
          }
        else if ( flgs == 4 ) // TITLE PAGE TEXT FLAG ** XXXXXXX
          {
            more = anlystitp ( &cq ); // Returns 1 if found title start.
            if ( more & !portprn ) // Not needed for portrait mode
              {
                titpageon = 1;
                titnumlines = 1;
              }
          }
        else if ( flgs == 16 ) // NEW PAGE NEEDED
          {
            more = 0;
            ++cq;
            newpage = 1;
            more = 0;
          }
        else
          {
            if ( strtlin && strtpg )  // START OF NEW PAGE
              anlyshead ();
            firstpass = 1;
            prn_wrd ();
            firstpass = 0;
          }
        ++cq;
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "TOPX = %d, LOGICX = %d", topx, logicx );
            wrt_debug ( );
          }
        if ( logicx >= topx )         // NEW - 4/3/96
          more = 0;
        else
          {
            lastx = logicx;
            lstcall = juscall;
          }
        if ( cq >= qu_ptr )
          {
            if ( more )
              {
                ++cq; /** End reached before margin, error **/
                sprintf ( d_bugstr, "END OF QUEUE REACHED PREMATURELY!" );
                wrt_debug ();
              }
            more = 0;
            endfilfnd = 1;
          }
      } // End - while ( more )
      jfill = topx - lastx - 3;
      if ( jfill < 0 || newlin || endfilfnd || newpage ) // NEW - 29/5/94
        add2jus ( topx, lstcall, 0 );
      else
        add2jus ( topx, lstcall, jfill );
      if ( d_bug == 4 )
        {
          sprintf ( d_bugstr, "After first pass:" );
          wrt_debug ();        
          for ( j = 0; j < jusptr; ++j )
            {
              sprintf ( d_bugstr,
                    "JUS %d, Left = %d, Right = %d, Fills = %d, Calls = %d.",
                     j, js[j].xl, js[j].xr, js[j].jusfil, js[j].juscall );
              wrt_debug ( );
            }
          sprintf ( d_bugstr, "obptr = %d", obptr );
          wrt_debug ();            
          for ( j = 0; j < obptr; ++j )
            {
              sprintf ( d_bugstr,
                    "Element %d, Depth = %d, Left = %d, Right = %d", j,
                    ob_det[j].depth, ob_det[j].left, ob_det[j].right );
              wrt_debug ( );
            }
          sprintf ( d_bugstr, "CQ = %d, topx = %d, lastx = %d, logicx = %d.",
                            cq, topx, lastx, logicx );
          wrt_debug ( );
        }
    if ( topx < txtrct.right )
      {
        for ( j = obptr; j >= 0; --j )
          {
            if ( ob_det[j].left > js[jusptr-1].xr )
              js[jusptr-1].xr = ob_det[j].left - 4;
          }
        js[jusptr-1].jusfil = js[jusptr-1].xr - logicx - 3;
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Setting right to: %d", js[jusptr-1].xr );
            wrt_debug ( );
            sprintf ( d_bugstr, "Setting jfill to: %d", js[jusptr-1].jusfil );
            wrt_debug ( );
          }
      }
    juschk = 0;
    obblck = obbsav; // Allow adding to object queue?? But it doesn't happen?
    jusdo = 1;
// END OF FIRST PASS - ANLSLIN
    if ( titpfnd && titnumlines == 1 )  // If beginning of title page
      {
        titpfnd = 0; // Added to make title page print properly
        titpageon = 0; // REALLY NOT SURE ABOUT THIS!
        titnumlines = 0;

        if ( logicx > topx )
          {
            titrgtx = topx;
            if ( d_bug == 4 )
              {
                sprintf ( d_bugstr, "ANLYSLIN1: TITRGTX set to: %d", titrgtx );
                wrt_debug ( );
              }
          }
      }
    else if ( titpend && titnumlines > 1 )  // End of title page
      titpageon = 1;

    newlin = 0;
    njfils = 0;
    juscall = 0;
    jusptr = 0;  //  CAREFUL 13/3/96
    strtlin = 1;
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "Call 5 to setcurpos: X: %d, Y: %d.", txtrct.left, pcurly );
        wrt_debug ();
      }            
    setcurpos ( txtrct.left, pcurly );
    cq = sq;
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "\nANALYSE LINE: Second Pass.\n" );
        wrt_debug ( );
      }
    more = 1;
    lastx = 0;
    jusfilf = 0;
    juscalf = 0;
    nxtobj = 0;
    while ( more )
      {
        get_que (cq);
        smlfnt = fnt;
        if ( flgs == 1 )  /** New line required **/
          {
            more = 0;
            ++cq;
            newlin = 1;
          }
        else if ( flgs == 4 ) // TITLE PAGE TEXT FLAG
          {
            more = anlystitp ( &cq );
            if ( more )
              {
                titpageon = 1;
                titnumlines = 1;
              }
            else
              {
                titpageon = 0;   // NOT SURE - WHAT ABOUT FIRST PASS?
                tittopline = titnumlines;
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "TITLE PAGE LAST LINE = %d",
                                        tittopline );
                    wrt_debug ( );
                    sprintf ( d_bugstr, "ANLYSLIN, TITPAGEON TURNED OFF " );
                    wrt_debug ( );
                  }
                titlastx = logicx;
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "TITLE LAST X set to: %d",
                                        titlastx );
                    wrt_debug ( );
                  }
                if ( logicx > titrgtx )
                  {
                    titrgtx = logicx;
                    if ( d_bug == 4 )
                      {
                        sprintf ( d_bugstr, "ANLYSLIN2: TITRGTX set to: %d",
                                            titrgtx );
                        wrt_debug ( );
                      }
                  }
              }
          }
        else if ( flgs == 16 ) // NEW PAGE NEEDED
          {
            newpage = 1;
            ++cq;
            more = 0;
          }
        else
          {
            if ( strtlin && strtpg )  // START OF NEW PAGE
              anlyshead ();
            prn_wrd ();
          }
        ++cq;
        if ( logicx >= topx )
          more = 0;
        else
          {
            lastx = logicx;
            lstcall = juscall;
          }
        if ( cq >= qu_ptr )
          {
            if ( more )
              {
                ++cq; /** End reached before margin, error **/
                sprintf ( d_bugstr, "END OF QUEUE REACHED PREMATURELY!" );
                wrt_debug ();
              }
            more = 0;
            endfilfnd = 1;
          }
      }
// END OF SECOND PASS - ANLSLIN
// DON'T NEED THIS AT ALL!
    ++jusptr;     // ONLY FOR TEST PURPOSES
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "After second pass:n" );
        wrt_debug ();
        for ( j = 0; j < jusptr; ++j )
          {
            sprintf ( d_bugstr,
                  "JUS %d, Left = %d, Right = %d, Fills = %d, Calls = %d.",
                   j, js[j].xl, js[j].xr, js[j].jusfil, js[j].juscall );
            wrt_debug ( );
          }
        sprintf ( d_bugstr, "CQ = %d, lastx = %d, logicx = %d.",
                            cq, lastx, logicx );
        wrt_debug ( );
        sprintf ( d_bugstr, "obptr = %d", obptr );
        wrt_debug ();
        for ( j = 0; j < obptr; ++j )
          {
            sprintf ( d_bugstr,
                    "Element %d, Depth = %d, Left = %d, Right = %d", j,
                    ob_det[j].depth, ob_det[j].left, ob_det[j].right );
            wrt_debug ( );
          }
      }
    jusdo = 0;   // WHY???
    if ( topx < txtrct.right )
      cq -= 1;
    else
      cq -= 2; // Because we have counted one word more than needed for line,
             // and incremented cq as well

    noprint = 0;  /** Turn printing back on **/
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr,
                 "ANALYSE LINE: Cursor = %d, Margin = %d\n", lastx,
                  txtrct.right );
        wrt_debug ( );
      }

    for ( j = 0; j < jusptr; ++j )
      jsT[j] = js[j];
    for ( j = 0; j < obptr; ++j )
      ob_detT[j] = ob_det[j];

    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "ANLSLIN, returning: %d", cq );
        wrt_debug ( );
      }
    return (cq);
  } // END - anlslin

/**** Routine to print 'n' words from current cursor position ****/
void prn_nlin ( int fq, int lq )
// fq, lq - first and last members of queue to print
  {
    int  i, j, jfill, cq, x;
    int  titleftadjust = 0;
    int obbsav, lstcall, newlin, topx, lastx;
    int jerror = 0;

    if ( wd_qu[fq].topx ) // NEW - 5/3/96
      topx = wd_qu[fq].topx;
    else
      topx = txtrct.right;
    cur_fnt = 0;
    if ( juson )
      {
        noprint = 1; /** Turn printing off **/
        juschk = 1;  /** Turn on justification checking **/
        obbsav = obblck;
        obblck = 0;  // Block adding to obscure queue
        jusptr = 0; /** CAREFUL **/
        js[jusptr].xl = txtrct.left;
      }
    else
      juschk = 0; /** Turn off justification checking **/

    noprint = 1; // Also, if no justification at all !?!

    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "\nPRN_NLIN: First Pass\n" );
        wrt_debug ( );
        sprintf ( d_bugstr, "TITPON %d TITPEND %d FSTISTIT %d TITPFND %d",
                  titpon, titpend, fstistit, titpfnd );
        wrt_debug ( );
      }
    jusdo = 0;
    juscall = 0;
    jusptr = 0;  /** CAREFUL **/
    newlin = 0;
    cq = fq;
    njfils = 0;
    strtlin = 1;
    nxtobj = 0;
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "Call 6 to setcurpos: X: %d, Y: %d.", txtrct.left, pcurly );
        wrt_debug ();
      }            
    setcurpos ( txtrct.left, pcurly );  /** ADDED **/
    if ( titpageon )
      {
        ++titnumlines;
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "PRN_NLIN: TITLE LINES INCREMENTED: %d",
                                 titnumlines );
            wrt_debug ( );
          }
        if ( titnumlines == tittopline )
          {
            x = titrgtx - titlastx;
            if ( d_bug == 4 )
              {
                sprintf ( d_bugstr, "PRN_NLIN: Adjusting left margin, X = %d",
                                    x );
                wrt_debug ( );
              }
            if ( x > 10 )
              {
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "Call 7 to setcurpos: X: %d, Y: %d.", txtrct.left + x/2, pcurly );
                    wrt_debug ();
                  }                          
                setcurpos ( txtrct.left + x/2, pcurly );
                titleftadjust = txtrct.left + x/2;
              }
          }
      }
    while ( cq <= lq )
      {
        lastx = logicx;     // ADDED - 5/3/96
        lstcall = juscall;
        get_que (cq);
        smlfnt = fnt;
        if ( flgs == 1 )  /** New line required **/
          newlin = 1;
        else if ( flgs == 4 ) /** TITLE PAGE TEXT FLAG **/
          {
            if ( titpfnd == 0 && titpageon == 0 )
              {
                titpfnd = 1;
                titpageon = 1;
                titnumlines = 1;
              }
            else
              {
                titpfnd = 0;
                titpend = 1;
              }
          }
        else if ( flgs == 16 ) // NEW PAGE NEEDED
          newpage = 1;
        else if ( flgs == 0 || flgs & 8 ) /** No new line required **/
          {
            if ( strtlin && strtpg )
              anlyshead ( );
            prn_wrd ();
          }
        ++cq;
      }
    jfill = topx - logicx - 3;
    if ( logicx < topx - 275 ) // Need to make this a variable ???
// NEED ANOTHER WAY TO DETECT LAST LINE!!!
      {
        jfill = 0; // NEW - 29/5/94 - FIXED LAST LINE PROBLEM.
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "JFILL SET TO ZERO" );
            wrt_debug ( );
            sprintf ( d_bugstr, "LOGICX = %d, TOPX = %d", logicx, topx );
            wrt_debug ( );
          }
      }

    if ( jfill < 0 || newlin || newpage )  // XXXX
      add2jus ( topx, juscall, 0 );
    else
      add2jus ( topx, juscall, jfill );
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "\nPRN_NLIN, after first pass:" );
        wrt_debug ( );
        for ( i = 0; i < jusptr; ++i )
          {
            sprintf ( d_bugstr,
               "JUS %d, Left = %d, Right = %d, Fills = %d, Calls = %d.",
                   i, js[i].xl, js[i].xr, js[i].jusfil, js[i].juscall );
            wrt_debug ( );
          }
        sprintf ( d_bugstr, "CQ = %d, logicx = %d.", cq, logicx );
        wrt_debug ( );
        sprintf ( d_bugstr, "obptr = %d", obptr );
        wrt_debug ( );        
        for ( i = 0; i < obptr; ++i )
          {
            sprintf ( d_bugstr,
                    "Element %d, Depth = %d, Left = %d, Right = %d", i,
                    ob_det[i].depth, ob_det[i].left, ob_det[i].right );
            wrt_debug ( );
          }
      }
    if ( topx < txtrct.right )
      {
        for ( j = obptr; j >= 0; --j )
          {
            if ( ob_det[j].left > js[jusptr-1].xr )
              js[jusptr-1].xr = ob_det[j].left - 4;
          }
        js[jusptr-1].jusfil = js[jusptr-1].xr - logicx - 3;
      }
    juschk = 0;   // SETS FOR SECOND PASS
    obblck = obbsav;
    jusdo = 1;    // SETS FOR SECOND PASS
// END OF FIRST PASS - PRN_NLIN
    if ( titpfnd && titnumlines == 1 )  // If beginning of title page
      {
        titpfnd = 0; // Added to make title page print properly
        titpageon = 0; 
        titnumlines = 0;
      }
    else if ( titpend && titnumlines > 1 )  // End of title page
      titpageon = 1;

    newlin = 0;
    cq = fq;
    njfils = 0;
    strtlin = 1;
    juscall = 0;
    jusptr = 0;  /** CAREFUL **/
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "PRN_NLIN: Second Pass" );
        wrt_debug ( );
        sprintf ( d_bugstr, "JUSCHK = %d, JUSDO = %d", juschk, jusdo );
        wrt_debug ( );
      }
    jusfilf = 0;
    juscalf = 0;
    noprint = 0; /** Turn printing back on **/
    nxtobj = 0;
    if ( titleftadjust )
      {
      if ( d_bug == 4 )
        {
        sprintf ( d_bugstr, "Call 8 to setcurpos: X: %d, Y: %d.", titleftadjust, pcurly );
        wrt_debug ();
        } 
        setcurpos ( titleftadjust, pcurly );
      }
    else
      {
      if ( d_bug == 4 )
        {
        sprintf ( d_bugstr, "Call 9 to setcurpos: X: %d, Y: %d.", txtrct.left, pcurly );
        wrt_debug ();
        } 
        setcurpos ( txtrct.left, pcurly );
      }
    while ( cq <= lq )
      {
        get_que (cq);
        smlfnt = fnt;
        if ( flgs == 1 )  /** New line required **/
          newlin = 1;
        else if ( flgs == 4 ) /** TITLE PAGE TEXT FLAG **/
          {
            if ( titpfnd == 0 && titpageon == 0 )
              {
                titpfnd = 1;
                titpageon = 1;
                titnumlines = 1;
              }
            else
              {
                titpfnd = 0;
                titpend = 1;
                titpageon = 0;   // NOT SURE - WHAT ABOUT FIRST PASS?
                tittopline = titnumlines;
                titnumlines = 0;   // CAREFUL
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "PRN_NLINE: TITLE PAGE LAST LINE = %d",
                                        tittopline );
                    wrt_debug ( );
                    sprintf ( d_bugstr, "TITPAGEON TURNED OFF" );
                    wrt_debug ( );
                  }
              }
          }
        else if ( flgs == 16 ) // NEW PAGE NEEDED
          newpage = 1;
        else if ( flgs == 0 || flgs & 8 ) /** No new line required **/
          {
            if ( strtlin && strtpg )
              anlyshead ( );
            prn_wrd ();
          }
        ++cq;
      }
// END OF BOTH PASSES - PRN_NLIN
    if ( juson )  // Only valid on second pass
      {
        ++jusptr;
        if ( jusptr > jussiz )
          {
            sprintf ( d_bugstr, "BLOWN JUSTIFICATION QUEUE" );
            wrt_debug ();
          }
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "\nPRN_NLIN, after second pass:" );
            wrt_debug ( );
            for ( i = 0; i < jusptr; ++i )
              {
                sprintf ( d_bugstr,
                "JUS %d, Left = %d, Right = %d, Fills = %d, Calls = %d.",
                       i, js[i].xl, js[i].xr, js[i].jusfil, js[i].juscall );
                wrt_debug ( );
              }
            sprintf ( d_bugstr, "CQ = %d, logicx = %d.", cq, logicx );
            wrt_debug ( );
            for ( i = 0; i < obptr; ++i )
              {
                sprintf ( d_bugstr,
                    "Element %d, Depth = %d, Left = %d, Right = %d", i,
                    ob_det[i].depth, ob_det[i].left, ob_det[i].right );
                wrt_debug ( );
              }
          }
        jusdo = 0;
      }
// TESING ONLY: - XXXX
    if ( jusptr > 1 )
      {
      for ( i = 0; i < jusptr; ++i )
        {
          if ( jsT[i].xl != js[i].xl || jsT[i].xr != js[i].xr ||
             jsT[i].jusfil != js[i].jusfil || jsT[i].juscall != js[i].juscall )
            jerror = 1;
        }
      }
    if ( jerror )
      {
        for ( i = 0; i < jusptr; ++i )
          {
            sprintf ( d_bugstr,
            "JUST %d, Left = %d, Right = %d, Fills = %d, Calls = %d.",
                   i, jsT[i].xl, jsT[i].xr, jsT[i].jusfil, jsT[i].juscall );
            wrt_debug ( );
          }
        for ( i = 0; i < jusptr; ++i )
          {
            sprintf ( d_bugstr,
            "JUS %d, Left = %d, Right = %d, Fills = %d, Calls = %d.",
                   i, js[i].xl, js[i].xr, js[i].jusfil, js[i].juscall );
            wrt_debug ( );
          }
        sprintf ( d_bugstr, "JUSTIFICATION ERROR" );
        wrt_debug ( );
      }
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "PRN_NLIN with CQ = %d", cq );
        wrt_debug ( );
      }
  } // END - prn_nlin
  