/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
/**** TF3.C ***/

/**** Main Parsing routines ****/
/** Queue handling, file reading **/

#include <windows.h>
#include <io.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <dos.h>
#include "tform.h"
#include "tibform.ext" 
#include "tibform.ids"

/**** File variables ****/

extern HANDLE fl_hndl;
extern int csx_on;

/**** ROUTINE TO CHECK BACKWARDS FOR DOUBLE TRANSCRIBED LETTER ****/
int bck_dc ( char *str )
  {
    int  vm1;
    vm1 = 0;
    vm1stp = 0;
    if ( *str == 'h' && *(str - 1) == 'k' )
      {
        vm1 = ID_NOR_KH;
        vm1stp = 2;
      }
    else if ( *str == 'g' && *(str - 1) == 'n' )
      {
        vm1 = ID_NOR_NG;
        vm1stp = 2;
      }
    else if ( *str == 'h' && *(str - 1) == 'c' )
      {
        vm1 = ID_NOR_CH;
        vm1stp = 2;
      }
    else if ( *str == 'y' && *(str - 1) == 'n' )
      {
        vm1 = ID_NOR_NY;
        vm1stp = 2;
      }
    else if ( *str == 'h' && *(str - 1) == 't' )
      {
        vm1 = ID_NOR_TH;
        vm1stp = 2;
      }
    else if ( *str == 'h' && *(str - 1) == 'p' )
      {
        vm1 = ID_NOR_PH;
        vm1stp = 2;
      }
    else if ( *str == 's' && *(str - 1) == 't' )
      {
        vm1 = ID_NOR_TS;
        vm1stp = 2;
      }
    else if ( *str == 'h' && *(str - 1) == 's' && *(str - 2 ) == 't' )
      {
        vm1 = ID_NOR_TSH;
        vm1stp = 3;
      }
    else if ( *str == 'z' && *(str - 1) == 'd' )
      {
        vm1 = ID_NOR_DZ;
        vm1stp = 2;
      }
    else if ( *str == 'h' && *(str - 1) == 'z' )
      {
        vm1 = ID_NOR_ZH;
        vm1stp = 2;
      }
    else if ( *str == 'h' && *(str - 1) == 's' )
      {
        vm1 = ID_NOR_SH;
        vm1stp = 2;
      }
    return ( vm1 );
  }

/**** ROUTINE TO DETERMINE A SINGLE TRANSCRIBED CONSONANT ****/
int bck_sc ( char chr )
  {
    int    j, hit;
    j = -1;
    hit = 0;
    switch ( chr )
      {
        case 'k': j = ID_NOR_K; break;
        case 'g': j = ID_NOR_G; break;
        case 'c': j = ID_NOR_C; break;
        case 'j': j = ID_NOR_J; break;
        case 't': j = ID_NOR_T; break;
        case 'd': j = ID_NOR_D; break;
        case 'n': j = ID_NOR_N; break;
        case 'p': j = ID_NOR_P; break;
        case 'b': j = ID_NOR_B; break;
        case 'm': j = ID_NOR_M; break;
        case 'w': j = ID_NOR_W; break;
        case 'z': j = ID_NOR_Z; break;
        case  39: j = ID_NOR_A; break;
        case 'y': j = ID_NOR_Y; break;
        case 'r': j = ID_NOR_R; break;
        case 'l': j = ID_NOR_L; break;
        case 's': j = ID_NOR_S; break;
        case 'h': j = ID_NOR_H; break;
        case 'a': j = ID_NOR_SA; break;
        default: j = 0; break;
      }
    return ( j );
  }

/**** ROUTINE TO ANALYSE A VOWEL ****/
int chk_vw ( char chrct )
  {
    if ( chrct == 'a' )
      return ( ID_NOR_A );
    else if ( chrct == 'i' )
      return ( ID_NOR_I );
    else if ( chrct == 'u' )
      return ( ID_NOR_U );
    else if ( chrct == 'e' )
      return ( ID_NOR_E );
    else if ( chrct == 'o' )
      return ( ID_NOR_O );
    else if ( chrct == '\0' )
      return ( ID_NOR_A );
    return ( ID_NOR_A );
  }

/**** ROUTINE TO ANALYSE FLAG WORD IN TEST BUFFER ****/
int spec_an ( void )
  {
    if ( wrd_tst[1] == 'N' || wrd_tst[1] == 'n' )
      {
        flgs = 1;     /** New line **/
        return ( 1 );
      }
    else if ( wrd_tst[1] == 'S' || wrd_tst[1] == 's' )
      {
        flgs = 2;  /** Toggle between font sizes - NEEDED ?? **/
        if ( sml_fnt == 0 )
          sml_fnt = 1;
        else
          sml_fnt = 0;
        return ( 0 );
      }
    else if ( wrd_tst[1] == 'P' || wrd_tst[1] == 'p' )
      {
        flgs = 16;
        return ( 1 );
      }
    else if ( wrd_tst[1] == 'T' && wrd_tst[2] == 'S' )
      {
        return ( 2 );   /** Side bar title text **/
      }
    else if ( wrd_tst[1] == 'T' && wrd_tst[2] == 'P' )
      {
        flgs = 4;  /** Title page **/
        return ( 1 );
      }
    return ( 0 );  // NEW - TO FIX COMPILER WARNING
  }

/**** MAIN ROUTINE TO ANALYSE WORD IN TEST BUFFER ****/
int word_an ( void )
  {
    int   i, vwlpstn;
    int   vm1, errhit;
    lng = 0; flgs = 0;
    prf = 0; sup = 0; rot = 0; sub = 0; sub2 = 0;
    vw1 = 0; vw2 = 0; vw3 = 0; suf1 = 0; suf2 = 0; shd = 0;
    errhit = 0;
    if ( wrd_tst[0] == '$' )
      {
        if ( wrd_tst[1] == 'X' || wrd_tst[1] == 'x' ) // NEW - 15/11/2003
          {
            rot = ID_SWIDSH; //  148 sbrul shad/"twiddly" shad
            goto an_end;
          }
        return ( spec_an () );
      }
    if ( chk4skt () ) /** Do rough check for Sanskrit, to save time **/
      {
        an_skt ();
        return ( 1 );
      }
// First, deal with any special difficult cases:
    if ( strcmp ( wrd_tst, "'ang'o" ) == 0 ) 
      {
        vw1 = ID_NOR_A;
        suf1 = ID_NOR_NG;
        vw2 = ID_NOR_O;
        lng = 1;          // Indicates Tibetan word
        return ( 1 );
      }
/**** FIND FIRST VOWEL IN SYLLABLE ****/
    i = -1;
    vwlpstn = -1;
    do
      {
        ++i;
        if ( wrd_tst[i] == 'a' || wrd_tst[i] == 'i' || wrd_tst[i] == 'u'
          || wrd_tst[i] == 'e' || wrd_tst[i] == 'o' )
        vwlpstn = i; /** Mark vowel position in string **/
      } while ( wrd_tst[i + 1] != '\0' && vwlpstn == -1 );
    if ( vwlpstn == -1 )
      {
        sprintf ( d_bugstr, "ERROR - Vowel not found, word: %s.", wrd_tst );
        wrt_debug ( );
      }
    else
      vw1 = chk_vw ( wrd_tst[i] );  /** Determine ID for vowel **/
    if ( vwlpstn == 0 )
      {
        rot = ID_NOR_SA;
        goto get_suf;    /**** Jump to determine any suffix ****/
      }
    /**** DETERMINE PREVIOUS CONSONANT ****/
    /**** CHECK FOR DOUBLE TRANSCRIBED ****/
    i = vwlpstn - 1;
    vm1 = bck_dc ( wrd_tst + i ); /** Check for double **/
/**** NONE OF THESE CAN BE SUBJOINED, SO:- ****/
    if ( vm1stp ) /** Set to 2 or 3 depending on double, eg. kh, tsh **/
      {
        rot = vm1;
      }
/**** CHECK FOR POSSIBLE SUBJOINS ****/
    else  /** i points to character immediately before vowel **/
      {
        if ( ( wrd_tst[i] == 'y' || wrd_tst[i] == 'r' || wrd_tst[i] == 'l'
          || wrd_tst[i] == 'w' || wrd_tst[i] == 'h' ) && i > 0 )
          {
            if ( wrd_tst[i] == 'y' )
              {
                if ( wrd_tst[i-1] != 46 )
                  sub = ID_NOR_Y;
                else if ( wrd_tst[i-1] == 't' )
                  sub = 0;
              }
            else if ( wrd_tst[i] == 'r' )
              sub = ID_NOR_R;
            else if ( wrd_tst[i] == 'l' )
              sub = ID_NOR_L;
            else if ( wrd_tst[i] == 'h' )
              sub = ID_NOR_H;
            else if ( wrd_tst[i] == 'w' )
              sub = ID_NOR_W;
            rot = 0;
          }
        if ( !sub )
/**** DETERMINE ROOT ****/
          rot = bck_sc ( wrd_tst[i] ); /** Determines a single consonant **/
        vm1stp = 1;
        if ( sub == ID_NOR_W && wrd_tst[i-1] == 'r' )
          {
            if ( i == 1 )
              {
                rot = ID_NOR_R;
                vm1stp = 1;
              }
            else
              {
                sub = ID_NOR_R;
                sub2 = ID_NOR_W;
                vm1stp = 2;
              }
          }
        if ( sub == ID_NOR_H && wrd_tst[i-1] == 'l' )
          {    /*** This might be a bit flaky ***/
            rot = ID_NOR_H;
            sup = ID_NOR_L;
            sub = 0;
            vm1stp = 1;
          }
      }
/**** We arrive here either with the root or subjoin determined, and, only ONE of these ****/
/**** POINT TO START OF DETERMINED CHARACTER ****/
    i = vwlpstn - vm1stp;
    if ( i == 0 )
      goto get_suf;      /**** Jump to determine any suffix ****/
/**** IF SUBJOIN, DETERMINE ROOT ****/
    if ( sub )
      {
        --i;
        rot = bck_dc ( wrd_tst + i );
        if ( rot == 0 )
          rot = bck_sc ( wrd_tst[i] );
        else
          i = i + 1 - vm1stp;

        if ( i == 0 )    /**** No prefix or superjoin ****/
          goto get_suf;
      }
/**** IF ROOT, DETERMINE SUPERJOIN ****/
    --i;
    if ( wrd_tst[i] == 'r' )
      sup = ID_NOR_R;
    else if ( wrd_tst[i] == 's' )
      sup = ID_NOR_S;
    else if ( wrd_tst[i] == 'l' )
      sup = ID_NOR_L;
    if ( sup && i == 0 )
      goto get_suf;
    if ( sup )
      --i;
/****  DETERMINE PREFIX ****/
    if ( wrd_tst[i] == 'g' )
      prf = ID_NOR_G;
    else if ( wrd_tst[i] == 'd' )
      prf = ID_NOR_D;
    else if ( wrd_tst[i] == 'b' )
      prf = ID_NOR_B;
    else if ( wrd_tst[i] == 'm' )
      prf = ID_NOR_M;
    else if ( wrd_tst[i] == 39 )
      prf = ID_NOR_A;
    else if ( wrd_tst[i-1] == 'g' && wrd_tst[i] == 46 ) /** 'g.' **/
      {
        prf = ID_NOR_G;
        rot = ID_NOR_Y;
        sub = 0;
      }
    if ( !prf )
      {
        sprintf ( d_bugstr, "ERROR - PREFIX NOT DETERMINED, %s.", wrd_tst );  
        wrt_debug ();
        errhit = 1;
      }
/**** DETERMINE SUFFIX, IF ANY ****/
    get_suf: i = vwlpstn + 1;
    if ( wrd_tst[i] == '\0' )
      goto an_end;
    if ( wrd_tst[i] == 39 )
    /** We have a second vowel **/
      {
        ++i;
        vw2 = chk_vw ( wrd_tst[i] );
        ++i;
        if ( wrd_tst[i] == '\0' )
          goto an_end;
        if ( wrd_tst[i] == 39 )
        /** We have a third vowel **/
          {
            ++i;
            vw3 = chk_vw ( wrd_tst[i] );
            goto an_end;
          }
      }
    if ( wrd_tst[i] == 'n' && wrd_tst[i + 1] == 'g' )
      {
        suf1 = ID_NOR_NG;
        i += 2;
      }
    else
      {
        suf1 = bck_sc ( wrd_tst[i] );
        ++i;
      }
    if ( wrd_tst[i] == '\0' )
      goto an_end;
    if ( wrd_tst[i] == 's' )
      suf2 = ID_NOR_S;
    else
      {
        sprintf ( d_bugstr, "ERROR IN FINAL SUFFIX, %s.", wrd_tst );
        wrt_debug ();
        errhit = 1;
      }
    ++i;
    if ( wrd_tst[i] != '\0' )
      {
        sprintf ( d_bugstr, "ERROR IN FINAL SUFFIX, %s.", wrd_tst ); 
        wrt_debug ();
        errhit = 1;
      }
    an_end:
    ++i;
    if ( errhit )
      an_skt ();
    else
      lng = 1;  /** Indicates Tibetan word **/
    return ( 1 );
  }   /** END word_an **/

/** Routine to get one character from file **/
char get_fch ( void )
  {
    char  c;
    unsigned fil_len; 
    int Bytes;
    if ( flbftptr == flbfcptr && !file_read ) // Either buffer is empty or finished
      {
        fil_len = GetFileSize ( fl_hndl, NULL );
        sprintf ( d_bugstr, "Input file length = %ld", fil_len );
        wrt_debug ( );   
        if ( FALSE == ReadFile( fl_hndl, fil_buf, fil_len, (LPDWORD) &Bytes, NULL) )
          {
            Msg ( "Read Main File error!" );
            CloseHandle( fl_hndl );
            sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
            wrt_debug (); 
            return (0);
          }        
        flbftptr = fil_len;
        if ( fil_buf[flbftptr-1] == 26 )
          fil_buf[flbftptr-1] = 0;              
        fil_buf[flbftptr] = '\0';
        fil_buf[flbftptr+1] = '\0';    
        flbfcptr = 0;        
        file_read = 255;
      }
    ++flbfcptr;
    c = *(fil_buf + flbfcptr - 1 );
    if ( c == 124 )   /** '|' **/
      c = 47;         /** '/' **/
    return ( c );
  } // END - get_fch ( void )

void fil_gnbuf ( void )
  {
    int  more, x; /** x is for testing **/
    char c;
    int ctrlchr = 0;
/** Fill with a word at a time **/
    sprintf ( d_bugstr, "fil_gnbuf end, gnbufptr: %d", gnbufptr );
    wrt_debug ();    
    more = 1;
    x = 0;
    while ( more )
    do
      {
        skpwgt:
        do /** Skip white space **/
          {
            c = get_fch ();
          } while ( c == 10 || c == 13 || c == 32 );
        if ( c == 123 )   /** { **/
          {
            do
              {
                c = get_fch ();
              } while ( c != 125 ); /** } **/
            goto skpwgt;
          }
        /** Arrive here with non-white space character **/
        if ( c == 26 || c == 0 ) /*** EOF marker ***/
          {
            more = 0;
            *( gen_buf + gnbufptr++ ) = 0;  // Was c
          }
        else
          {
// We definately have a character 
// Problem with $S, $N, etc., so, first:
            if ( c == '$' ) // It should only ever be the first character
              ctrlchr = 1;
            do
              {
                if ( !ctrlchr ) // Convert old prel1, CSX and 
                  {             // Harvard-Kyoto to prel2 encodings
                    // This is also where UTF-8 conversion should go, before the others
                    if ( csx_on )
                      {
                        switch ( c )
                          { 
                            case 224: c = 163; break; // Long a, CSX
                            case 227: c = 131; break; // Long i, CSX
                            case 229: c = 133; break; // Long u, CSX
                            case 231: c = 135; break; // vowel ri, CSX
                            case 235: c = 139; break; // vowel li, CSX
                            case 239: c = 168; break; // n, dot over, CSX
                            case 241: c = 145; break; // t dot, CSX                 
                            case 243: c = 147; break; // d dot, CSX
                            case 245: c = 149; break; // n, dot under, CSX
                            case 252: c = 151; break; // m, dot under, CSX
                            case 247: c = 153; break; // s, acute, CSX
                            case 249: c = 170; break; // s, dot under, CSX
                            case 254: c = 160; break; // visarga, CSX
                            default: ; break;
                          }                      
                      }
                    else // Not CSX
                      {  
                        switch ( c )
                          { 
                            case 129: c = 163; break; // Long a, prel1                 
                            case 'A': c = 163; break; // Long a, Harvard-Kyoto
                            case 'I': c = 131; break; // Long i, Harvard-Kyoto
                            case 'U': c = 133; break; // Long u, Harvard-Kyoto
                            case 'R': c = 135; break; // vowel ri, Harvard-Kyoto
                            case 'L': c = 139; break; // vowel li, Harvard-Kyoto
                            case 141: c = 166; break; // long vowel li, prel1
                            case 143: c = 168; break; // n, dot over, prel1
                            case 'G': c = 168; break; // n, dot over, Harvard-Kyoto
                            case 'T': c = 145; break; // t dot, Harvard-Kyoto
                            case 'D': c = 147; break; // d dot, Harvard-Kyoto
                            case 'N': c = 149; break; // n, dot under, Harvard-Kyoto
                            case 'M': c = 151; break; // m, dot under, Harvard-Kyoto
                            case 'Z': c = 153; break; // s, acute, Harvard-Kyoto
                            case 157: c = 170; break; // s, dot under, prel1
                            case 'S': c = 170; break; // s, dot under, Harvard-Kyoto
                            case 'H': c = 160; break; // visarga, Harvard-Kyoto
                            default: ; break;
                          }
                      } 
                   }
                *( gen_buf + gnbufptr++ ) = c;
                c = get_fch ();
              } while ( c > 32 );
            ctrlchr = 0;
            /** End of word, so bung in a space **/
            *( gen_buf + gnbufptr++ ) = 32;
          }
      } while ( more );
    *( gen_buf + gnbufptr ) = '\0';
    sprintf ( d_bugstr, "fil_gnbuf end, gnbufptr: %d", gnbufptr );
    wrt_debug ();    
  } // END - fil_gnbuf

/** GETS WORD INTO WRD_TST **/
int gtnxwrd ( void )
  {
    int i;
    unsigned start_ptr;
    for ( i = 0; i < 12; ++i )
      wrd_tst[i] = '\0';
    i = 0;
    start_ptr = gcbufptr;
    do
      {
        wrd_tst[i] = gen_buf[gcbufptr];
        ++i;
        ++gcbufptr;
      } while ( gen_buf[gcbufptr] != 32 && gen_buf[gcbufptr] != 0 &&
                gen_buf[gcbufptr] != 26 && gen_buf[gcbufptr-1] != 47 && // 26 = EOF
                gen_buf[gcbufptr] != 47 );
    wrd_tst[i] = '\0';
    while ( gen_buf[gcbufptr] == 32 ) 
      ++gcbufptr;
    if ( gen_buf[gcbufptr] == 26 || gen_buf[gcbufptr] == 0 ) 
      {
        if ( d_bug == 2 )
          {
             sprintf ( d_bugstr, "Gtnxwrd, found %d, passing END: %s, start ptr: %d, current ptr: %d", 
                   (int) gen_buf[gcbufptr], wrd_tst, start_ptr, gcbufptr );
             wrt_debug ();
          }
         return (-1); 
      }                   
    return (gcbufptr);
  }

void tst_wrd ( int n ) // Very old test code - keep for now
  {
//     printf ("\nWord number %d: ", n );
     get_que (n);
//     printf ("Word = %s\n", wrd_tst );
/**  printf ("Prefix = %d\n", wd_qu[n].prf );
     printf ("Super  = %d\n", wd_qu[n].sup );
     printf ("Root   = %d\n", wd_qu[n].rot );
     printf ("Sub    = %d\n", wd_qu[n].sub );
     printf ("Sub2   = %d\n", wd_qu[n].sub2 );
     printf ("Vowel1 = %d\n", wd_qu[n].vw1 );
     printf ("Vowel2 = %d\n", wd_qu[n].vw2 );
     printf ("Vowel3 = %d\n", wd_qu[n].vw3 );
     printf ("Suf1   = %d\n", wd_qu[n].suf1 );
     printf ("Suf2   = %d\n", wd_qu[n].suf2 );
     printf ("Shad   = %d\n", wd_qu[n].shd );
     printf ("Word   = %s\n", wd_qu[n].wrd );
     printf ("Length = %d\n", strlen (wd_qu[n].wrd) );
     printf ("\n");
     printf ("Press any key for next word:\n");
     getch (); **/
/**  printf ("\n"); ******/
  }

/**** Routine to put word information into end of queue  ****/
/**** 'qu_ptr' always points to first available position ****/
void put_que ( void )
  {
     unsigned i;
     if ( qu_ptr + 1 >= WDQUMX )
       {
         sprintf ( d_bugstr, "ERROR - QUEUE BLOWN! %s", wrd_tst );
         wrt_debug ();
       }
     wd_qu[qu_ptr].lng = lng;     /** Language - 1 = Tib, 2 = Skt **/
     wd_qu[qu_ptr].flgs = flgs;   /** Flags **/
     wd_qu[qu_ptr].prf = prf;     /** Prefix **/
     wd_qu[qu_ptr].sup = sup;     /** Superjoined letter **/
     wd_qu[qu_ptr].rot = rot;     /** Root letter **/
     wd_qu[qu_ptr].sub = sub;     /** Subjoined letter **/
     wd_qu[qu_ptr].sub2 = sub2;   /** Subjoined letter **/
     wd_qu[qu_ptr].vw1 = vw1;     /** First vowel **/
     wd_qu[qu_ptr].vw2 = vw2;     /** Second vowel **/
     wd_qu[qu_ptr].vw3 = vw3;     /** Third vowel **/
     wd_qu[qu_ptr].suf1 = suf1;   /** First suffix **/
     wd_qu[qu_ptr].suf2 = suf2;   /** Second suffix **/
     wd_qu[qu_ptr].shd = shd;     /** Shad required **/
     wd_qu[qu_ptr].hm = hm;       /** Total horizontal length in dots **/
     wd_qu[qu_ptr].fnt = fnt;     /** Font **/
     i = 0;
     do
       {
         wd_qu[qu_ptr].wrd[i] = wrd_tst[i];
       } while ( wrd_tst[i++] != 0 );
     wd_qu[qu_ptr].wrd[i] = '\0';
     if ( lng == 2 )  /** If Sanskrit **/
       {
         for ( i = 0; i < 16; ++i )
           wd_qu[qu_ptr].sktwrd[i] = sktstr[i];
       }
     ++qu_ptr;  /*** Update top of queue pointer ***/
     if ( qu_ptr > maxqptr )
       maxqptr = qu_ptr;
  }

/**** ROUTINE TO SHUFFLE DOWN THE QUEUE ****/
/**** 'QU_PTR' IS RESET AT THE END      ****/
void shuf_que ( int n )
  {
    int   i, j;
    unsigned k;
    i = 0;
    j = n;
    while ( j < qu_ptr )
      {
        wd_qu[i].lng = wd_qu[j].lng;
        wd_qu[i].flgs = wd_qu[j].flgs;
        wd_qu[i].prf = wd_qu[j].prf;
        wd_qu[i].sup = wd_qu[j].sup;
        wd_qu[i].rot = wd_qu[j].rot;
        wd_qu[i].sub = wd_qu[j].sub;
        wd_qu[i].sub2 = wd_qu[j].sub2;
        wd_qu[i].vw1 = wd_qu[j].vw1;
        wd_qu[i].vw2 = wd_qu[j].vw2;
        wd_qu[i].vw3 = wd_qu[j].vw3;
        wd_qu[i].suf1 = wd_qu[j].suf1;
        wd_qu[i].suf2 = wd_qu[j].suf2;
        wd_qu[i].shd = wd_qu[j].shd;
        wd_qu[i].hm = wd_qu[j].hm;
        wd_qu[i].fnt = wd_qu[j].fnt;
        k = 0;
        do
          {
            wd_qu[i].wrd[k] = wd_qu[j].wrd[k];
          } while ( wd_qu[j].wrd[k++] != 0 );
         for ( k = 0; k < 16; ++k )
           wd_qu[i].sktwrd[k] = wd_qu[j].sktwrd[k];
        ++i;
        ++j;
      }
    qu_ptr = i;     /*** Update top of queue pointer ***/
  }

/**** 'qu_ptr' always points to first available position ****/
void get_que ( int n )
  {
    unsigned i;
    lstqfch = n;   /** For debugging: the last value fetched **/
    lng = wd_qu[n].lng;
    flgs = wd_qu[n].flgs;
    prf = wd_qu[n].prf;
    sup = wd_qu[n].sup;
    rot = wd_qu[n].rot;
    sub = wd_qu[n].sub;
    sub2 = wd_qu[n].sub2;
    vw1 = wd_qu[n].vw1;
    vw2 = wd_qu[n].vw2;
    vw3 = wd_qu[n].vw3;
    suf1 = wd_qu[n].suf1;
    suf2 = wd_qu[n].suf2;
    shd = wd_qu[n].shd;
    hm = wd_qu[n].hm;
    fnt = wd_qu[n].fnt;
/**  strcpy ( wrd_tst, wd_qu[n].wrd );  **/
    i = 0;
    do
      {
        wrd_tst[i] = wd_qu[n].wrd[i];
      } while ( wd_qu[n].wrd[i++] != 0 );
    i = 0;
     do
       {
         nxt_wrd1[i] = wd_qu[n+1].wrd[i];
       } while ( wd_qu[n+1].wrd[i++] != 0 );
    if ( d_bug == 4 )
      {    
        sprintf ( d_bugstr, "From queue: %s, %s, %s\n", lst_wrd, wrd_tst, nxt_wrd1 );
        wrt_debug ();
      }
    if ( wd_qu[n].fnt != wd_qu[n+1].fnt )
       nxt_wrd_font_chg = 1;
    i = 0;
    do
      {
        nxt_wrd2[i] = wd_qu[n+2].wrd[i];
      } while ( wd_qu[n+2].wrd[i++] != 0 );

    if ( lng == 2 )  /** If Sanskrit **/
      {
        for ( i = 0; i < 16; ++i )
          sktstr[i] = wd_qu[n].sktwrd[i];
      }
    if ( (d_bug == 4 ) && (flgs & 8) )  
      {
        sprintf ( d_bugstr, "GET_QUE: LAST WORD: %s", wrd_tst );
        wrt_debug ( );
      }
  } // END - get_que ()

/**** ROUTINE FOR STRING ANALYSIS ****/
void str_an ( void )
  {
    int  more, j, x;

    if ( qu_ptr >= WDQUMX ) /** If queue full **/
      return;
    if ( pasteof )          /** If end of file already reached **/
      return;               /** Everything is in the queue **/
    more = 1;
    x = 0;
    do  /** One word at a time **/
      {
        j = gtnxwrd (); /** gets word pointed by 'j' into wrd_tst **/
        if ( d_bug == 5 ) 
          {
            sprintf ( d_bugstr, "Parsing word: %s, j = %d", wrd_tst, j );
            wrt_debug ();
          }
        if ( j == -1 )
          {
            more = 0; /** End of file reached **/
            pasteof = 1;
          }
        if ( wrd_tst[0] == 47 )  /** We have a 'shad' **/
          wd_qu[qu_ptr-1].shd += 1;  /* What if word is $ control ?? */
        else if ( wrd_tst[0] == 37 ||  /** percent or colon **/
                  wrd_tst[0] == 58 ) /** We have a Terma 'shad' **/
          wd_qu[qu_ptr-1].shd += 8;  /* What if word is $ control ?? */
        else if ( wrd_tst[0] == 92 ) // Continuation shad
          wd_qu[qu_ptr-1].shd = 64;
/****** ID_SWIDSH   148    sbrul shad **/
/****** ID_CONTSH   149 "Continuation" shad **/
/****** exclamation mark == 33          **/
        else
          {
            j = word_an (); /** Font switch is determined here **/
            if ( j == 1 )
              {
                fnt = sml_fnt;  /** Set font change **/
                put_que (); /** Put into queue, increments queue pointers **/
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "PUT into queue: %s, %d, ", wrd_tst, qu_ptr );
                    wrt_debug ();
                    sprintf ( d_bugstr, "SML_FNT = %d\n", sml_fnt );
                    wrt_debug ();
                  }
              }
            else if ( j == 2 ) /** Found a side title **/
              an_titb ();
          }
        if ( pasteof )
          {
            if ( d_bug == 4 )
              {
                sprintf ( d_bugstr, "SETTING END OF FILE\n" );
                wrt_debug ( );
              }
            wd_qu[qu_ptr-1].flgs += 8; /* Indicate last word */
          }
        if ( qu_ptr >= WDQUMX - 2 )
          more = 0;
        ++x;
        if ( x > 15 )
          {
            x = 0;
          }
      } while ( more );
  } // END - str_an ()
  