/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
/**** TF4.C ****/

/**** Sanskrit Parsing routines ****/

#include <io.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <dos.h>
#include "tform.h"
#include "tibform.ext"
#include "tibform.ids"

extern struct c_map c_mp[140];

// Locals:

int  chradv;  /** For specifying jump over a character **/

struct skt_map
  {
    int   chr;
    int   id;
  };

// Could lose some of these now:
struct skt_map s_mp[26] = { 129,  ID_SKT_A,          /** Long A **/
                            131,  ID_SKT_I,          /** Long I **/
                            133,  ID_SKT_U,          /** Long U **/
                            135,  ID_SKT_R,          /** RI     **/
                            137,  ID_SKT_LR,         /** Long RI **/
                            139,  ID_SKT_LI,         /** LI     **/
                            141,  ID_SKT_LLI,        /** Long LI **/
                            143,  ID_NOR_NG,         /** N dot over **/
                            145,  ID_NOR_TR,         /** T dot under **/
                            147,  ID_NOR_DR,         /** D dot under **/
                            149,  ID_NOR_NR,         /** N dot under **/
                            151,  ID_SKT_M,          /** M dot under **/
                            153,  ID_NOR_SH,         /** S slash **/
                            157,  ID_SKT_SZH,        /** S dot under **/
                            160,  ID_SKT_H,          /** H dot under **/
                            163,  ID_SKT_A,          /** NEW == Long A **/
                            164,  ID_NOR_NY,         /** N tilda **/
                            166,  ID_SKT_LLI,        /** NEW == Long LI **/
                            168,  ID_NOR_NG,         /** NEW == N dot over **/
                            170,  ID_SKT_SZH,        /** NEW == S dot under **/
                            'v',  ID_SKT_V,          /** V **/
                            'f',  ID_SKT_FO,   
                            'q',  ID_SKT_Q,    
                              0,  0 };

extern int jussav;

FILE *fpsrc = '\0';

struct wrd_struct nwd_qu[12];
int     nqu_ptr;      /*** Pointer into word queue ***/
struct wrd_struct twd_qu[12];
int     tqu_ptr;      /*** Pointer into word queue ***/

char *unitStr[9] = { "gcig", "gnyis", "gsum", "bzhi", "lnga", "drug",
                    "bdun", "brgyad", "dgu" };
char *tencStr[9] = { "bcu", "nyer", "so", "zhe", "nga", "re", "don",
       "gya", "go" };
char *ten1Str[9] = { "bcu", "nyi", "sum", "bzhi", "lnga", "drug", "bdun",
                   "brgyad", "dgu" };

char *numStr[10];

/** ROUTINE TO GET SANSKRIT ID OF SINGLE CHARACTER **/
/** RETURNS 0 IF NOT KNOWN **/
/** This is used for specific recognition only **/
int getsktid ( char c ) /** SKT **/
  {
    int i, j;
    i = 0;
    j = 0;
    while ( s_mp[i].chr != 0 )
      {
         if ( s_mp[i].chr == c )
           j = s_mp[i].id;
         ++i;
      }
    return ( j );
  }

/*** ROUTINE TO ROUGH CHECK TEST WORD FOR SANSKRIT ***/
/*** RETURNS ZERO IF NOT FOUND ***/
int chk4skt ( void ) /** SKT **/
  {
    int i, j;
    i = 0;
    if ( strncmp ( wrd_tst, "hra", 3 ) == 0 )
      return ( 1 );
    if ( strncmp ( wrd_tst, "hwa", 3 ) == 0 )
      return ( 1 );
    while ( wrd_tst[i] != '\0' )
      {
        if ( j = getsktid ( wrd_tst[i] ) )
          return ( j );
        else
          ++i;
      }
    switch ( wrd_tst[0] )
      {
        case 'g':
          if ( wrd_tst[1] == 'h' )
            return ( 1 );
          else if ( wrd_tst[1] == 'g' && wrd_tst[3] == '\0' )
            return ( 1 );
          else if ( wrd_tst[1] == 'h' && wrd_tst[2] == 'r' )
            return ( 1 ); break;
        case 'c':
          if ( wrd_tst[1] == 'y' )
            return ( 1 ); break;
        case 'j':
          if ( wrd_tst[1] == 'h' )
            return ( 1 ); break;
        case 't':
          if ( wrd_tst[1] == 'y' )
            return ( 1 );
          else if ( wrd_tst[1] == 's' && wrd_tst[2] == 'y' )  // cya, tsy
            return ( 1 );
          else if ( wrd_tst[1] == 'v' )
            return ( 1 );
          else if ( wrd_tst[1] == 'w' )
            return ( 1 );
          else if ( wrd_tst[1] == 'r' && wrd_tst[2] == 'w' )
            return ( 1 ); break;
        case 'd':
          if ( wrd_tst[1] == 'h' )
            return ( 1 );
          else if ( wrd_tst[1] == 'd' && wrd_tst[3] == '\0' )
            return ( 1 ); break;
        case 'n':
          if ( wrd_tst[1] == 46 ) /* n. */
            return ( 1 ); break;
        case 'p':
          if ( wrd_tst[1] == 'h' && wrd_tst[2] == 'y' // phywa - NEW 11/5/2001
                                 && wrd_tst[3] == 'w' ) // Needs to be treated as
                                                        // Sanskrit
            return ( 1 ); break;
        case 'b':
          if ( wrd_tst[1] == 'h' )
            return ( 1 );
          else if ( wrd_tst[1] == 'b' && wrd_tst[3] == '\0' )
            return ( 1 ); break;
        case 'm':
          if ( wrd_tst[1] == 'm' )
            return ( 1 ); break;
        case 'w':
          if ( wrd_tst[1] == 'w' )
            return ( 1 ); break;
        case 'y':
          if ( wrd_tst[1] == 'y' )
            return ( 1 ); break;
        case 'r':
          if ( wrd_tst[1] == 'r' )
            return ( 1 );
          else if ( wrd_tst[1] == 'd' && wrd_tst[2] == 'h' )
            return ( 1 ); break;
        case 's':
          if ( wrd_tst[1] == 's' )
            return ( 1 );
          else if ( wrd_tst[1] == 'y' && wrd_tst[2] == 'e' )
            return ( 1 ); break;
        case 'h':
          if ( wrd_tst[1] == 'y' )
            return ( 1 ); break;
        default: ; break;
      }
    i = 0;
    while ( wrd_tst[i+1] != '\0' )
/**** THIS NEEDS LOOKING AT ****/
      {
        if ( wrd_tst[i] == 'a' && wrd_tst[i+1] == 'i' )
          return ( 1 );
        else if ( wrd_tst[i] == 'a' && wrd_tst[i+1] == 'u' )
          return ( 1 );
        else
          ++i;
      }
    return ( 0 );
  } // END - chk4skt ()

/** Routine to check an ID for being a vowel **/
/** Returns 1 if it is, 0 if not **/
int issktvwl ( int id ) /** SKT **/
  {
    if ( id == ID_NOR_I || id == ID_NOR_U || id == ID_NOR_E || id == ID_NOR_O
         || id == ID_NOR_SA || id == ID_SKT_AU || id == ID_SKT_AI || id ==
         ID_SKT_A || id == ID_SKT_I || id == ID_SKT_U || id == ID_SKT_R
         || id == ID_SKT_LR || id == ID_SKT_LI || id == ID_SKT_LLI )
      return ( 1 );
    else
      return ( 0 );
  }

/** Routine to determine general character ID **/
/** Translates internal ids **/
int detsktid ( char *s ) /** SKT **/
  {
    int j;
    chradv = 1;
    switch ( *s )
      {
//      case 39: j = ID_NOR_A; break; // CHANGED 14-5-94
        case 39: j = ID_SKT_AVA; break; // Avagraha, or, stop before 'o
        case 46: j = ID_STOP; break;
/*** First, the diacritics ***/
        case 163: j = ID_SKT_A; break;           /** Long A **/
        case 131: j = ID_SKT_I; break;           /** Long I **/
        case 133: j = ID_SKT_U; break;           /** Long U **/
        case 135:                                /** RI   **/
          if ( *(s+1) == 'i' )
            {
              j = ID_SKT_R;
              chradv = 2;
            }
          else if ( *(s+1) == 131 )   // LONG I
            {
              j = ID_SKT_LR;
              chradv = 2;
            }
          else
            j = ID_SKT_R;
          break;
        case 137:                     // Long RI
          if ( *(s+1) == 'i' || *(s+1) == 131 )  // LONG I
            chradv = 2;
          j = ID_SKT_LR;
          break;
        case 139:                    // LI
          if ( *(s+1) == 'i' )
            {
              j = ID_SKT_LI;
              chradv = 2;
            }
          else if ( *(s+1) == 131 )  // LONG I
            {
              j = ID_SKT_LLI;
              chradv = 2;
            }
          else
            j = ID_SKT_LI;
          break;
        case 166: 
          if ( *(s+1) == 'i' || *(s+1) == 131 )  // LONG I
            chradv = 2;
          j = ID_SKT_LLI;
          break;
        case 168: j = ID_NOR_NG; break;          // N dot over
        case 145:
          if ( *(s+1) == 'h' )
            {
              j = ID_NOR_TRH;                    // TH dot under
              chradv = 2;
            }
          else
            j = ID_NOR_TR;                       // T dot under
          break;
        case 147:
          if ( *(s+1) == 'h' )
            {
              j = ID_NOR_DRH;                    // DH dot under
              chradv = 2;
            }
          else
            j = ID_NOR_DR;                       // D dot under 
          break;
        case 149: j = ID_NOR_NR; break;          // N dot under 
        case 151: j = ID_SKT_M; break;           // M dot under 
        case 153: j = ID_NOR_SH; break;          // S slash 
        case 170: j = ID_SKT_SZH; break;         // S dot under 
        case 160: 
          if ( *(s+1) == 'p' ) // Hp
            {
              j = ID_SKT_FO; 
              chradv = 2;
            }
          else if ( *(s+1) == 'k' ) // Hk
            {
              j = ID_SKT_Q; 
              chradv = 2;
            }            
          else
            j = ID_SKT_H; 
          break;           /** H dot under **/
// ADD HERE OTHER ASPIRATES.
        case 241: // NEW 
        case 164: j = ID_NOR_NY; break;          /** N tilda **/
/*** Now, the 'normal' characters ***/
        case 'k':
          if ( *(s+1) == 'h' )
            {
              j = ID_NOR_KH;
              chradv = 2;
            }
          else if ( *(s+1) == 157 ) /** KZHA **/
            {
              j = ID_SKT_KZH;
              chradv = 2;
            }
          else
            j = ID_NOR_K;
          break;
        case 'g':
          if ( *(s+1) == 'h' )
            {
              j = ID_SKT_GH;
              chradv = 2;
            }
          else
            j = ID_NOR_G;
          break;
        case 'c':
          if ( *(s+1) == 'h' )                  /** CH **/
            {
              j = ID_NOR_TSH;
              chradv = 2;
            }
          else                                  /** C **/
            j = ID_NOR_TS;
          break;
        case 'j':
          if ( *(s+1) == 'h' )
            {
              j = ID_NOR_DZH;
              chradv = 2;
            }
          else
            j = ID_NOR_DZ;
          break;
        case 'd':
          if ( *(s+1) == 'z' && *(s+2) == 'h' )
            {
              j = ID_NOR_DZH;
              chradv = 3;
            }
          else if ( *(s+1) == 'z' )
            {
              j = ID_NOR_DZ;
              chradv = 2;
            }
          else if ( *(s+1) == 'h' )
            {
              j = ID_SKT_DH;
              chradv = 2;
            }
          else
            j = ID_NOR_D;
          break;
        case 't':
          if ( *(s+1) == 's' && *(s+2) == 'h' ) /** CH **/
            {
              j = ID_NOR_TSH;
              chradv = 3;
            }
          else if ( *(s+1) == 's' )             /** C **/
            {
              j = ID_NOR_TS;
              chradv = 2;
            }
          else if ( *(s+1) == 'h' )             /** TH **/
            {
              j = ID_SKT_TH;
              chradv = 2;
            }
          else
            j = ID_NOR_T;
          break;
        case 'n': j = ID_NOR_N; break;
        case 'p':
          if ( *(s+1) == 'h' )
            {
              j = ID_NOR_PH;
              chradv = 2;
            }
          else
            j = ID_NOR_P;
          break;
        case 'b':
          if ( *(s+1) == 'h' )
            {
              j = ID_SKT_BH;
              chradv = 2;
            }
          else
            j = ID_NOR_B;
          break;
        case 's':
          if ( *(s+1) == 'h' )
            {
              j = ID_NOR_SH;  /** S slash - this is in addition to the
                                  diacritic **/
              chradv = 2;
            }
          else
            j = ID_NOR_S;
          break;
        case 'm': j = ID_NOR_M; break;
        case 'y': j = ID_NOR_Y; break;
        case 'r': j = ID_NOR_R; break;
        case 'l': j = ID_NOR_L; break;
        case 'v': j = ID_SKT_V; break;
        case 'w': j = ID_SKT_V; break;  /** Is this a good idea? **/
        case 'f': j = ID_SKT_FO; break;  // Hp
        case 'q': j = ID_SKT_Q; break;   // Hk
        case 'h':
          if ( *(s+1) == 'r' )
            {
              j = ID_SKT_HR;
              chradv = 2;
            }
          else
            j = ID_NOR_H;
          break;
/*** Now, the other vowels ***/
        case 'a':
          if ( *(s+1) == 'i' )
            {
              j = ID_SKT_AI;
              chradv = 2;
            }
          else if ( *(s+1) == 'u' )
            {
              j = ID_SKT_AU;
              chradv = 2;
            }
          else
            j = ID_NOR_SA;
          break;
        case 'i': j = ID_NOR_I; break;
        case 'u': j = ID_NOR_U; break;
        case 'e': j = ID_NOR_E; break;
        case 'o': j = ID_NOR_O; break;
        default: j = 0; break;
      }
    if ( d_bug == 3 )
      {
        sprintf ( d_bugstr, "DETSKTID: SKT ID = %s.", chrs[j] );
        wrt_debug ();
      }
    return ( j );
  } // END - detsktid ()

/** MAIN ROUTINE TO ANALYSE SANSKRIT WORD **/
void an_skt ( void ) /** SKT **/
  {
    int i, c, more, vwltst;
    int src_ptr, tgt_ptr;  // Pointers into test word and target array
    int hld_ptr;  // Pointer to start of current syllable in target
    lng = 2;  // Indicates Sanskrit
    for ( i = 0; i < 16; ++i ) // Clear the Sanskrit buffer
      sktstr[i] = 0;
    src_ptr = 0;
    tgt_ptr = 0;
    while ( wrd_tst[src_ptr] != '\0' ) // ONE SYLLABLE AT A TIME
      {
        hld_ptr = tgt_ptr;
        ++tgt_ptr;
        more = 1;
        do     // ONE CHARACTER AT A TIME
          {
            if ( d_bug == 3 )
              {
                sprintf ( d_bugstr, "an_skt, NEW CHARACTER:" );
                wrt_debug ();
              }
            c = detsktid ( wrd_tst + src_ptr );
            src_ptr += chradv;   /** How far to skip in test word **/
            ++sktstr[hld_ptr];   /** Indicates another character added **/
            sktstr[tgt_ptr++] = c;
            vwltst = issktvwl ( c );
            if ( c == ID_SKT_AVA )
              {
                if ( *( wrd_tst + src_ptr ) == 'h' )
                  vwltst = ID_NOR_SA;
              }
/*** TEST HERE FOR VISARGA AFTER A CONSONANT ***/
            if ( c == ID_SKT_H && sktstr[hld_ptr] == 1 ) // IT'S THE FIRST
              more = 0;
            else if ( wrd_tst[src_ptr] == 160 /* SKT_H */ && !vwltst )
              {  /* ONLY KNOWN OCCURENCE IS IN PHATH */
                ++sktstr[hld_ptr]; /** Indicates another character added **/
                sktstr[tgt_ptr++] = ID_NOR_SA;
                more = 0;  // END THIS SYLLABLE
              }
            else if ( wrd_tst[src_ptr] == '\0' || vwltst )
              {
              // WE HAVE TO TEST FOR THE POSSIBILITY THAT THE NEXT
              // CHARACTER IS AN ANUSVARA 
                c = detsktid ( wrd_tst + src_ptr );
                if ( vwltst && c == ID_SKT_M ) /* M dot */
                  {
                    ++sktstr[hld_ptr]; /* Indicates character added */
                    sktstr[tgt_ptr++] = c;
                    ++src_ptr;
                  }
                more = 0;  // END THIS SYLLABLE
              }
          } while ( more );
      }
    i = 0;
    if ( d_bug )
      {
        if ( tgt_ptr > max_sktstr )
          max_sktstr = tgt_ptr;
      }
  } // END - an_skt ()

/*** Routine to print a Tibetan character from current position ***/
void prn_chr ( int ch, int adv, int dwn )
  {
    int h;
    int curx, cury;
    curx = logicx;
    cury = logicy;
    sel_fnt ( c_mp[ch].fnt );
    if ( d_bug == 5 )
      {
        sprintf ( d_bugstr, "prn_chr, character, %d, font = %d.", ch, c_mp[ch].fnt );
        wrt_debug ();
      }
    h = adv;
    if ( dwn )
      {
        if ( cury + dwn < mincurpos && !sidprn && pageinprogress )
          {
            sprintf ( d_bugstr, "Cursor at %d Chr = %s, Word = %s",
                      cury + dwn, chrs[ch], wrd_tst );
            wrt_debug ();
          }
        if ( sidprn )
          setcurpos ( curx, cury - dwn );
        else
          setcurpos ( curx, cury + dwn );
      }
    pn_chr ( c_mp[ch].chr );
    if ( d_bug == 5 )
      {
        sprintf ( d_bugstr,
            "Printing character: %s, Advance = %d, Down = %d\n",
             chrs[ch], adv, dwn );
        if ( firstpass )
          wrt_debug ( );
      }

    curx += adv;
    if ( d_bug == 4 )
      {
        sprintf ( d_bugstr, "Call 21 to setcurpos: X: %d, Y: %d.", curx, cury );
        wrt_debug ();
      }    
    setcurpos ( curx, cury );
  } // END - prn_chr ()

/*** Set printer cursor position ***/
void setcurpos ( int x, int y )
  {
    if ( d_bug > 1 )
      {
        sprintf ( d_bugstr, "setcurpos, Cursor, X: %d, Y: %d", x, y );
        wrt_debug ( );
      }
    if ( sidprn )
      {
// x is value down from top of page, so is device x 
// y is value from left hand edge of page, so equals logical x
        if ( scaling_needed && !prev_on )
          {
            dev_cur_x = (int) ( (float) x * pr_scalex );       
            dev_cur_y = pr_vertres - (int) ( (float) y * pr_scaley );    
          }
        else
          {
            dev_cur_x = x;    
            dev_cur_y = pr_vertres - y;       
          }      
        if ( d_bug > 1)
          {
            sprintf ( d_bugstr, "setcurpos, side; Dev Cursor, X: %d, Y: %d", dev_cur_x, dev_cur_y );
            wrt_debug ( );
          }     
        logicx = y;
        logicy = x;                    
        pcur_x = y;
        pcur_y = x;
      }
    else if ( portprn )
      {  
// pr_scalex, pr_scaley - scaling factors from 300dpi to current device
        if ( scaling_needed )
          {
            dev_cur_x = (int) ( (float) x * pr_scalex );    
            dev_cur_y = (int) ( (float) y * pr_scaley );       
          }
        else
          {
            dev_cur_x = x;    
            dev_cur_y = y;       
          }      
        if ( d_bug > 1)
          {
            sprintf ( d_bugstr, "setcurpos, Dev Cursor, X: %d, Y: %d", dev_cur_x, dev_cur_y );
            wrt_debug ( );
          }     
      }
    else // if landscape
      {  
// pr_scalex, pr_scaley - scaling factors from 300 dpi to current device
        if ( scaling_needed )
          {
            dev_cur_x = (int) ( (float) y * pr_scalex );       
            dev_cur_y = pr_vertres - (int) ( (float) x * pr_scaley );    
          }
        else
          {
            dev_cur_x = y;    
            dev_cur_y = pr_vertres - x;       
          }      
        if ( d_bug > 1)
          {
            sprintf ( d_bugstr, "setcurpos, Dev Cursor, X: %d, Y: %d", dev_cur_x, dev_cur_y );
            wrt_debug ( );
          }     
      }
    logicx = x;
    logicy = y;
    pag_x = dev_cur_x;
    pag_y = dev_cur_y;              
  } // END - setcurpos ()

/*** Backspace by 'n' 300dpi dots ***/
void backspc ( int n )
  {
    if ( n )
      {
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Call 22 to setcurpos: X: %d, Y: %d.", logicx - n, logicy );
            wrt_debug ();
          }    
        setcurpos ( logicx - n, logicy );
      }
  }

/**** ROUTINE TO SELECT CHARACTER FONT ****/
void sel_fnt ( int x )
  {
    int y;
    if ( d_bug == 5 )
      {
        sprintf ( d_bugstr, "Selecting font = %d.", x );    
        wrt_debug ();
      }
    if ( smlfnt )
      {
        if ( sidprn ) /* For side bars only */
          {
            if ( x == 37 )
              y = 41;
            else if ( x == 38 )
              y = 42;
          }
        else
          {
            if ( x == 37 )
              y = 39;
            else if ( x == 38 )
              y = 40;
          }
      }
    else
      y = x;
    cur_fnt  = y; // NEW 10/5/94 CHECK 
    if ( d_bug == 5 )
      {
        sprintf ( d_bugstr, "sel_fnt: %d, Selected font = %d.", sidprn, y );    
        wrt_debug ();
      }
  } // END - sel_fnt ()

/**** INITIALISE PRINTER ****/
void init_prn ( void ) // Perhaps no need for this at all now...?
  {
    prn_erf = 0;
    cur_fnt = 0;
  }
  

void set_debug ( int x )
{
  if ( x == 0 )
    {
      clos_debug ();
      return;
    }
  fpsrc = fopen ( "debug.dat", "w" );   
  if ( fpsrc == NULL )
    {
      sprintf ( d_bugstr, "Error opening DEBUG file!" );
      Msg ( d_bugstr );
      return;
    }
  d_bug = x;
}

void wrt_debug ( void )
  {
    if ( fpsrc )
      fprintf ( fpsrc, "%s\n", d_bugstr );
  }

void clos_debug ( void )
  {
    if ( fpsrc )
      fclose ( fpsrc );
    d_bug = 0;
  }

// This prints Tibetan page number.
void prn_num ( int n, int x1, int x2, int y, int z )
  {
    int hx, tx, ux, j, i, prnsav;
    int numscnt;    
    jussav = juson;
    juson = 0;
    jusdo = 0;
    numscnt = 0;
    ux = n % 10;
    j = n / 10;
    tx = j % 10;
    hx = j / 10;
    j = hx * 100 + tx * 10 + ux;
    if ( hx )
      {
        if ( hx > 1 )
          numStr[numscnt++] = unitStr[hx - 1];
        if ( tx || ux )
          numStr[numscnt++] = "rgya";
        else
          {
            numStr[numscnt++] = "brgya";
            numStr[numscnt++] = "tham";
            numStr[numscnt++] = "pa";
          }
      }
    if ( tx )
      {
        if ( ux )
          {
            if ( tx == 1 && ( ux == 5 || ux == 8 ) )
              numStr[numscnt++] = "bco";
            else
              numStr[numscnt++] = tencStr[tx - 1];
          }
        else
          {
            numStr[numscnt++] = ten1Str[tx - 1];
            if ( tx == 2 )
              numStr[numscnt++] = "shu";
            else if ( tx == 3 || tx == 7 )
              numStr[numscnt++] = "cu";
            else if ( tx == 4 || tx == 5 || tx == 6 || tx == 8 || tx == 9 )
              numStr[numscnt++] = "bcu";
          }
      }
    if ( ux )
      numStr[numscnt++] = unitStr[ux - 1];
    if ( z && !nobyon )
      numStr[numscnt++] = "byon";
    if ( d_bug )
      {
        sprintf ( d_bugstr, "prn_num: numscnt: %d.", numscnt );
        wrt_debug ();
      }  
/*** FILL UP THE QUEUE ***/
    nqu_ptr = 0;
    for ( i = 0; i < numscnt; ++i )
      {
        for ( j = 0; j < 12; ++j )
          wrd_tst[j] = '\0';
        strcpy ( wrd_tst, numStr[i] );
        word_an ();
        put_nque (); /** Put data into queue, increments queue pointers **/
      }
/*** ANALYSE STRING ***/
    prnsav = noprint;
    noprint = 1;  /** Turn off actual printing **/
     if ( d_bug ) // Change back to == 4 )
      {
        sprintf ( d_bugstr, "Call 1 to setcurpos: X: %d, Y: %d.", x1, y );
        wrt_debug ();
      }
    setcurpos ( x1, y );
    for ( i = 0; i < numscnt; ++i )
      {
        get_nque (i);
        prn_wrd ();
      }
    noprint = prnsav;
/*** NOW, SET UP AND PRINT ***/
    if ( d_bug )
      {
        sprintf ( d_bugstr, "prn_num: pcur_y: %d, x1: %d.", pcur_y, x1 );
        wrt_debug ();
      }  
    hx = pcur_y; // This, presumably, is the position of the end of the string
    if ( d_bug ) // Change back to == 4 )
      {
        sprintf ( d_bugstr, "hx set to: %d.", hx );
        wrt_debug ();
      }    
    hx = hx - x1;
    tx = ( x2 - x1 - hx ) / 2;
    if ( d_bug ) // Change back to == 4 )
      {
        sprintf ( d_bugstr, "Call 2 to setcurpos: X: %d, Y: %d.", x1 + tx, y );
        wrt_debug ();
      }    
    setcurpos ( x1 + tx, y ); /** This centres the text in the bar **/
    for ( i = 0; i < numscnt; ++i )
      {
        get_nque (i);
        prn_wrd ();
      }
    juson = jussav;
  } // END - prn_num ()

/** ROUTINE TO ANALYSE SIDE TITLE STRING **/
void an_titb ( void )
  {
    int more, i, j, x;
    int prnsav, fntsav;
    jussav = juson;
    juson = 0;
    more = 1;
    tqu_ptr = 0;
    do
      {
        j = gtnxwrd (); /** gets word pointed by 'j' into wrd_tst **/
        j = word_an ();
        if ( j == 1 )
           put_tque (); /** Put into queue, increments queue pointers **/
         else if ( j == 2 ) /** Found a side title **/
           more = 0;
      } while ( more );
/*** ANALYSE STRING ***/
    prnsav = noprint;
    noprint = 1;  /** Turn off actual printing **/
    fntsav = smlfnt;
    smlfnt = 1;
    setcurpos ( 100, 100 );
    for ( i = 0; i < tqu_ptr; ++i )
      {
        get_tque (i);
        prn_wrd ();
      }
/*** NOW, SET UP ***/
    x = logicx;
    tbrwdth = x - 100;
    tbrpos = ( boxhgt - tbrwdth ) / 2;
    noprint = prnsav;
    smlfnt = fntsav;
    juson = jussav;
  }

// This prints the short title in reverse page side bar:
void prn_titb ( int x1, int x2, int y )
  {
    int i;
    jussav = juson;
    juson = 0;
    setcurpos ( x1 + tbrpos, y ); /** This centres the text in the bar **/
    for ( i = 0; i < tqu_ptr; ++i )
      {
        get_tque (i);
        prn_wrd ();
      }
    juson = jussav;
  } // END - prn_titb ()

/**** Routine to put word information into end of queue  ****/
/**** 'nqu_ptr' always points to first available position ****/
void put_nque ( void )
  {
    unsigned i;
    if ( nqu_ptr + 1 >= 12 ) // Make this a define ??? 
      {
        sprintf ( d_bugstr, "put_nque: ERROR - QUEUE BLOWN" );
        wrt_debug ();
      }
    nwd_qu[nqu_ptr].lng = lng;   /** Language - 1 = Tib, 2 = Skt **/
    nwd_qu[nqu_ptr].flgs = flgs; /** Flags **/
    nwd_qu[nqu_ptr].prf = prf;   /** Prefix **/
    nwd_qu[nqu_ptr].sup = sup;   /** Superjoined letter **/
    nwd_qu[nqu_ptr].rot = rot;   /** Root letter **/
    nwd_qu[nqu_ptr].sub = sub;   /** Subjoined letter **/
    nwd_qu[nqu_ptr].sub2 = sub2; /** Subjoined letter **/
    nwd_qu[nqu_ptr].vw1 = vw1;   /** First vowel **/
    nwd_qu[nqu_ptr].vw2 = vw2;   /** Second vowel **/
    nwd_qu[nqu_ptr].vw3 = vw3;   /** Third vowel **/
    nwd_qu[nqu_ptr].suf1 = suf1; /** First suffix **/
    nwd_qu[nqu_ptr].suf2 = suf2; /** Second suffix **/
    nwd_qu[nqu_ptr].shd = shd;   /** Shad required **/
    nwd_qu[nqu_ptr].hm = hm;     /** Total horizontal length in dots **/
    i = 0;
    do
      {
        nwd_qu[nqu_ptr].wrd[i] = wrd_tst[i];
      } while ( wrd_tst[i++] != 0 );
    nwd_qu[nqu_ptr].wrd[i] = '\0';
    if ( lng == 2 )  /** If Sanskrit **/
      {
        for ( i = 0; i < 16; ++i )
          nwd_qu[nqu_ptr].sktwrd[i] = sktstr[i];
      }
    ++nqu_ptr; /*** Update top of queue pointer ***/
  }

/**** 'nqu_ptr' always points to first available position ****/
void get_nque ( int n )
  {
     unsigned i;
     lng = nwd_qu[n].lng;
     flgs = nwd_qu[n].flgs;
     prf = nwd_qu[n].prf;
     sup = nwd_qu[n].sup;
     rot = nwd_qu[n].rot;
     sub = nwd_qu[n].sub;
     sub2 = nwd_qu[n].sub2;
     vw1 = nwd_qu[n].vw1;
     vw2 = nwd_qu[n].vw2;
     vw3 = nwd_qu[n].vw3;
     suf1 = nwd_qu[n].suf1;
     suf2 = nwd_qu[n].suf2;
     shd = nwd_qu[n].shd;
     hm = nwd_qu[n].hm;
     i = 0;
     do
       {
         wrd_tst[i] = nwd_qu[n].wrd[i];
       } while ( nwd_qu[n].wrd[i++] != 0 );
     if ( lng == 2 )  /** If Sanskrit **/
       {
         for ( i = 0; i < 16; ++i )
           sktstr[i] = nwd_qu[n].sktwrd[i];
       }
  }

/**** Routine to put word information into end of queue  ****/
/**** 'tqu_ptr' always points to first available position ****/
void put_tque ( void )
  {
     unsigned i;
     if ( tqu_ptr + 1 >= 12 )
       {
         sprintf ( d_bugstr, "put_tque: ERROR - QUEUE BLOWN!" );
         wrt_debug ();
       }
     twd_qu[tqu_ptr].lng = lng;   /** Language - 1 = Tib, 2 = Skt **/
     twd_qu[tqu_ptr].flgs = flgs; /** Flags **/
     twd_qu[tqu_ptr].prf = prf;   /** Prefix **/
     twd_qu[tqu_ptr].sup = sup;   /** Superjoined letter **/
     twd_qu[tqu_ptr].rot = rot;   /** Root letter **/
     twd_qu[tqu_ptr].sub = sub;   /** Subjoined letter **/
     twd_qu[tqu_ptr].sub2 = sub2; /** Subjoined letter **/
     twd_qu[tqu_ptr].vw1 = vw1;   /** First vowel **/
     twd_qu[tqu_ptr].vw2 = vw2;   /** Second vowel **/
     twd_qu[tqu_ptr].vw3 = vw3;   /** Third vowel **/
     twd_qu[tqu_ptr].suf1 = suf1; /** First suffix **/
     twd_qu[tqu_ptr].suf2 = suf2; /** Second suffix **/
     twd_qu[tqu_ptr].shd = shd;   /** Shad required **/
     twd_qu[tqu_ptr].hm = hm;     /** Total horizontal length in dots **/
     i = 0;
     do
       {
         twd_qu[tqu_ptr].wrd[i] = wrd_tst[i];
       } while ( wrd_tst[i++] != 0 );
     twd_qu[tqu_ptr].wrd[i] = '\0';
     if ( lng == 2 )  /** If Sanskrit **/
       {
         for ( i = 0; i < 16; ++i )
           twd_qu[tqu_ptr].sktwrd[i] = sktstr[i];
       }
     ++tqu_ptr; /*** Update top of queue pointer ***/
  }

/**** 'tqu_ptr' always points to first available position ****/
void get_tque ( int n )
  {
     unsigned i;
     lng = twd_qu[n].lng;
     flgs = twd_qu[n].flgs;
     prf = twd_qu[n].prf;
     sup = twd_qu[n].sup;
     rot = twd_qu[n].rot;
     sub = twd_qu[n].sub;
     sub2 = twd_qu[n].sub2;
     vw1 = twd_qu[n].vw1;
     vw2 = twd_qu[n].vw2;
     vw3 = twd_qu[n].vw3;
     suf1 = twd_qu[n].suf1;
     suf2 = twd_qu[n].suf2;
     shd = twd_qu[n].shd;
     hm = twd_qu[n].hm;
     i = 0;
     do
       {
         wrd_tst[i] = twd_qu[n].wrd[i];
       } while ( twd_qu[n].wrd[i++] != 0 );
     if ( lng == 2 )  /** If Sanskrit **/
       {
         for ( i = 0; i < 16; ++i )
           sktstr[i] = twd_qu[n].sktwrd[i];
       }
  }
  