/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
#define INCLUDE_COMMDLG_H       1
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <dos.h>
#include <string.h>
#include <malloc.h>
#include "tibw.h"
#include "tform.h"
#include "tibform.ext"

char fname[10];
char tf_buf[60000];

LPVOID bmBits;

char *tf37buf;
int tf37_xpt, tf37_ypt, tf37_fc, tf37_lc;
HBITMAP tf37_hbm[94];

char *tf38buf;
int tf38_xpt, tf38_ypt, tf38_fc, tf38_lc;
HBITMAP tf38_hbm[60];

char *tf39buf;
int tf39_xpt, tf39_ypt, tf39_fc, tf39_lc;
HBITMAP tf39_hbm[94];

char *tf40buf;
int tf40_xpt, tf40_ypt, tf40_fc, tf40_lc;
HBITMAP tf40_hbm[60];

char *tf41buf;
int tf41_xpt, tf41_ypt, tf41_fc, tf41_lc;
HBITMAP tf41_hbm[94];

char *tf42buf;
int tf42_xpt, tf42_ypt, tf42_fc, tf42_lc;
HBITMAP tf42_hbm[60];

static char spt_msk[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };

unsigned  chhed, fstchr, lstchr, byt_grd;
int x_lin, x_pt, byt_lin, y_pt;

void open_portrait_tfonts ( void )
  {
    int i, Bytes;
    HANDLE h;
    int fil_len = 0;
    int buf_siz;
    unsigned j, x; 
    float xdiv, ydiv;
    int x1, x2, r, c, tgt_grd, tgt_lin;
    int tnch; 

    SetCurrentDirectory( cur_dir );
    if ( d_bug > 1 )
      {
        if ( scaling_needed )      
          sprintf ( d_bugstr, "Creating scaled fonts." );
        else
          sprintf ( d_bugstr, "Creating 300 dpi fonts." );                    
        wrt_debug ();  
      }
  // First, tf37, with t1a.hpf
    strcpy ( fname, "t1a.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T1A, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ();              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;   
    tf37_fc = fstchr; 
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ();   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ();      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ();   
      }
    lstchr = fstchr + tnch - 1;
    tf37_lc = lstchr; 
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd );
        wrt_debug ();   
        sprintf ( d_bugstr, "The source characters are %d x %d.", y_pt, x_pt );
        wrt_debug ();   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ();   
      }                             

    if ( scaling_needed == 0 )
      {
        buf_siz = tnch * byt_grd;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "1. Buffer size = %ld.", buf_siz );
            wrt_debug ( );          
          }
        tf37buf = malloc ( (size_t) buf_siz );
        if ( tf37buf == NULL )
           {
             Msg ( "1. Memory allocation failed!" );
             exit (0);
           }
        memset( tf37buf, 0, buf_siz );        
        tf37_xpt = x_pt;
        tf37_ypt = y_pt;                   
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            x = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed; 
            for ( j = 0; j < byt_grd; ++j )
               *( tf37buf + ( i - fstchr ) * byt_grd + j ) = ~tf_buf[ x + j ];
            bmBits = tf37buf + ( i - fstchr ) * byt_grd;            
            tf37_hbm[i-fstchr] = CreateBitmap ( tf37_xpt, tf37_ypt, 1, 1, bmBits );                    
          }            
      }
    else // If scaling is needed:
      {
        tf37_xpt = x_pt * pr_scalex;
        tgt_lin = tf37_xpt / 8; // What if not a multiple???
        tf37_ypt = y_pt * pr_scaley;
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "New X width: %d, new Y height: %d.", tf37_xpt, tf37_ypt );
            wrt_debug ( );          
          }
        tgt_grd = (int) ( (float) byt_grd * pr_scalex * pr_scaley );
        buf_siz = tgt_grd * tnch;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "2. Buffer size = %ld.", buf_siz );
            wrt_debug ( );                  
          }
        tf37buf = malloc ( (size_t) buf_siz );   
        if ( tf37buf == NULL )
          Msg ( "2. Memory allocation failed!" );      
        memset( tf37buf, 0, buf_siz );        
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
            for ( r = 0; r < tf37_ypt; ++r )
              for ( c = 0; c < tf37_xpt; ++c )
                {
                  if ( !tst_buf ( &tf_buf[j], r * ydiv, c * xdiv, x_pt, y_pt ) )
                    wrt_buf ( ( tf37buf + ( i - fstchr ) * tgt_grd ), r, c, tf37_xpt, tf37_ypt );
                }
            bmBits = ( tf37buf + ( i - fstchr ) * tgt_grd );               
            tf37_hbm[i-fstchr] = CreateBitmap ( tf37_xpt, tf37_ypt, 1, 1, bmBits );
          }            
      }  
// Next is tf38, with t2a.hpf      
    strcpy ( fname, "t2a.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T2A, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;    
    tf38_fc = fstchr;     
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf38_lc = lstchr;     
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd );
        wrt_debug ( );   
        sprintf ( d_bugstr, "The source characters are %d x %d.", y_pt, x_pt );
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      {
        buf_siz = tnch * byt_grd;
        if ( d_bug > 1 )
          {
            sprintf ( d_bugstr, "1. Buffer size = %ld.", buf_siz );
            wrt_debug ( );          
          }
        tf38buf = malloc ( (size_t) buf_siz );
        if ( tf38buf == NULL )
           {
             Msg ( "1. Memory allocation failed!" );
             exit (0);
           }
        memset( tf38buf, 0, buf_siz );                
        tf38_xpt = x_pt;
        tf38_ypt = y_pt;                   
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            x = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed; 
            for ( j = 0; j < byt_grd; ++j )
               *( tf38buf + ( i - fstchr ) * byt_grd + j ) = ~tf_buf[ x + j ];
            bmBits = tf38buf + ( i - fstchr ) * byt_grd;
            tf38_hbm[i-fstchr] = CreateBitmap ( tf38_xpt, tf38_ypt, 1, 1, bmBits );                                
          }            
      }
    else // If scaling is needed:
      {
        tf38_xpt = x_pt * pr_scalex;
        tgt_lin = tf38_xpt / 8; // What if not a multiple???
        tf38_ypt = y_pt * pr_scaley;
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "New X width: %d, new Y height: %d.", tf38_xpt, tf38_ypt );
            wrt_debug ( );          
          }
        tgt_grd = (int) ( (float) byt_grd * pr_scalex * pr_scaley );
        buf_siz = tgt_grd * tnch;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "2. Buffer size = %ld.", buf_siz );
            wrt_debug ( );                  
          }
        tf38buf = malloc ( (size_t) buf_siz );        
        if ( tf38buf == NULL )
          Msg ( "2. Memory allocation failed!" );      
        memset( tf38buf, 0, buf_siz );        
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
            for ( r = 0; r < tf38_ypt; ++r )
              for ( c = 0; c < tf38_xpt; ++c )
                {
                  if ( !tst_buf ( &tf_buf[j], r * ydiv, c * xdiv, x_pt, y_pt ) )
                    wrt_buf ( ( tf38buf + ( i - fstchr ) * tgt_grd ), r, c, tf38_xpt, tf38_ypt );
                }
            bmBits = ( tf38buf + ( i - fstchr ) * tgt_grd );   
            tf38_hbm[i-fstchr] = CreateBitmap ( tf38_xpt, tf38_ypt, 1, 1, bmBits );            
          }            
      }         
// Next is tf39, with t1b.hpf          
    strcpy ( fname, "t1b.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T1B, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;    
    tf39_fc = fstchr;     
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf39_lc = lstchr;     
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd );
        wrt_debug ( );   
        sprintf ( d_bugstr, "The source characters are %d x %d.", y_pt, x_pt );
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      {
        buf_siz = tnch * byt_grd;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "1. Buffer size = %ld.", buf_siz );
            wrt_debug ( );          
          }
        tf39buf = malloc ( (size_t) buf_siz );
        if ( tf39buf == NULL )
           {
             Msg ( "1. Memory allocation failed!" );
             exit (0);
           }
        tf39_xpt = x_pt;
        tf39_ypt = y_pt;                   
        memset( tf39buf, 0, buf_siz );                           
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            x = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed; 
            for ( j = 0; j < byt_grd; ++j )
               *( tf39buf + ( i - fstchr ) * byt_grd + j ) = ~tf_buf[ x + j ];
            bmBits = tf39buf + ( i - fstchr ) * byt_grd;
            tf39_hbm[i-fstchr] = CreateBitmap ( tf39_xpt, tf39_ypt, 1, 1, bmBits );
          }            
      }
    else // If scaling is needed:
      {
        tf39_xpt = x_pt * pr_scalex;
        tgt_lin = tf39_xpt / 8; // What if not a multiple???
        tf39_ypt = y_pt * pr_scaley;
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "New X width: %d, new Y height: %d.", tf39_xpt, tf39_ypt );
            wrt_debug ( );          
          }
        tgt_grd = (int) ( (float) byt_grd * pr_scalex * pr_scaley );
        buf_siz = tgt_grd * tnch;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "2. Buffer size = %ld.", buf_siz );
            wrt_debug ( );                  
          }
        tf39buf = malloc ( (size_t) buf_siz );        
        if ( tf39buf == NULL )
          Msg ( "2. Memory allocation failed!" );      
        memset( tf39buf, 0, buf_siz );                           
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
            for ( r = 0; r < tf39_ypt; ++r )
              for ( c = 0; c < tf39_xpt; ++c )
                {
                  if ( !tst_buf ( &tf_buf[j], r * ydiv, c * xdiv, x_pt, y_pt ) )
                    wrt_buf ( ( tf39buf + ( i - fstchr ) * tgt_grd ), r, c, tf39_xpt, tf39_ypt );
                }
            bmBits = ( tf39buf + ( i - fstchr ) * tgt_grd );   
            tf39_hbm[i-fstchr] = CreateBitmap ( tf39_xpt, tf39_ypt, 1, 1, bmBits );
          }            
      }         
// Next is tf40, with t2b.hpf          
    strcpy ( fname, "t2b.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T2B, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;    
    tf40_fc = fstchr;     
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf40_lc = lstchr;     
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd );
        wrt_debug ( );   
        sprintf ( d_bugstr, "The source characters are %d x %d.", y_pt, x_pt );
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      {
        buf_siz = tnch * byt_grd;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "1. Buffer size = %ld.", buf_siz );
            wrt_debug ( );          
          }
        tf40buf = malloc ( (size_t) buf_siz );
        if ( tf40buf == NULL )
           {
             Msg ( "1. Memory allocation failed!" );
             exit (0);
           }
        tf40_xpt = x_pt;
        tf40_ypt = y_pt;                   
        memset( tf40buf, 0, buf_siz );                                      
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            x = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed; 
            for ( j = 0; j < byt_grd; ++j )
               *( tf40buf + ( i - fstchr ) * byt_grd + j ) = ~tf_buf[ x + j ];
            bmBits = tf40buf + ( i - fstchr ) * byt_grd;
            tf40_hbm[i-fstchr] = CreateBitmap ( tf40_xpt, tf40_ypt, 1, 1, bmBits );
          }            
      }
    else // If scaling is needed:
      {
        tf40_xpt = x_pt * pr_scalex;
        tgt_lin = tf40_xpt / 8; // What if not a multiple???
        tf40_ypt = y_pt * pr_scaley;
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "New X width: %d, new Y height: %d.", tf40_xpt, tf40_ypt );
            wrt_debug ( );          
          }
        tgt_grd = (int) ( (float) byt_grd * pr_scalex * pr_scaley );
        buf_siz = tgt_grd * tnch;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "2. Buffer size = %ld.", buf_siz );
            wrt_debug ( );                  
          }
        tf40buf = malloc ( (size_t) buf_siz );        
        if ( tf40buf == NULL )
          Msg ( "2. Memory allocation failed!" );      
        memset( tf40buf, 0, buf_siz );                                      
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
            for ( r = 0; r < tf40_ypt; ++r )
              for ( c = 0; c < tf40_xpt; ++c )
                {
                  if ( !tst_buf ( &tf_buf[j], r * ydiv, c * xdiv, x_pt, y_pt ) )
                    wrt_buf ( ( tf40buf + ( i - fstchr ) * tgt_grd ), r, c, tf40_xpt, tf40_ypt );
                }
            bmBits = ( tf40buf + ( i - fstchr ) * tgt_grd );   
            tf40_hbm[i-fstchr] = CreateBitmap ( tf40_xpt, tf40_ypt, 1, 1, bmBits );            
          }            
      }         
  } // END - open_portrait_tfonts ()

int tst_buf ( char *buf, int r, int c, int xwd, int yht ) // yht - not needed, ever ???
  {
    int i, j, pre_byt, xln;  
    xln = xwd / 8; 
    if ( xln * 8 < xwd )
      ++xln;     
    i = c / 8;
    j = c % 8;
    pre_byt = r * xln + i;
    if ( *(buf + pre_byt) & spt_msk[j] )
      return (1);
    else
      return (0);
  }  
  
/**** ROUTINE TO WRITE SPOT IN CHR_BUF1 ****/

void wrt_buf ( char *buf, int r, int c, int xwd, int yht )
  {
    int pre_byt, xln;
    if ( r < 0 || r >= yht || c < 0 || c >= xwd )
      {
        sprintf ( d_bugstr, "wrt_buf failing: R: %d, C: %d.", r, c );
        wrt_debug ();
        Msg ( d_bugstr );
        return;
      }
    xln = xwd / 8; // Make this a static somewhere...? No, has to be generic
    if ( xln * 8 < xwd )
      ++xln;  
    if ( xln % 2 )
      ++xln; // Should pass this value, just in case.  ??? Only relevant for landscape. 
    pre_byt = r * xln + c / 8;
    *(buf + pre_byt) = *(buf + pre_byt) | spt_msk[ c & 7 ];
  }
  
void clean_portrait (void) // Clean up memory etc.
  {
    int i;
    free ( tf40buf );
    free ( tf39buf );        
    free ( tf38buf );
    free ( tf37buf );
    for ( i = 0; i < 94; ++i )
      {
        DeleteObject ( tf37_hbm[i] );
        DeleteObject ( tf39_hbm[i] );        
      }
    for ( i = 0; i < 60; ++i )
      {
        DeleteObject ( tf38_hbm[i] );
        DeleteObject ( tf40_hbm[i] );        
      }    
  } // END - clean_portrait ()

void clean_landscape (void) // Clean up memory etc.
  {
    int i;
    free ( tf42buf );
    free ( tf41buf );        
    free ( tf40buf );
    free ( tf39buf );
    free ( tf38buf );
    free ( tf37buf );    
    for ( i = 0; i < 94; ++i )
      {
        DeleteObject ( tf37_hbm[i] );
        DeleteObject ( tf39_hbm[i] );        
        DeleteObject ( tf41_hbm[i] );                
      }
    for ( i = 0; i < 60; ++i )
      {
        DeleteObject ( tf38_hbm[i] );
        DeleteObject ( tf40_hbm[i] );        
        DeleteObject ( tf42_hbm[i] );                
      }    
// ANYTHING ELSE...????    
  } // END - clean_landscape ()  
  
void open_landscape_tfonts ( void )   
  {
    int i, Bytes; //, new_xpt, new_ypt;
    HANDLE h;
    int fil_len = 0;
    int buf_siz;
    unsigned j, x; 
    float xdiv, ydiv;
    int x1, x2, tgt_grd, tgt_lin, r, c, r2, c2, r1, c1;
    int tnch, tst;

    SetCurrentDirectory( cur_dir );    
    if ( d_bug > 1 )
      {
        if ( scaling_needed )      
          sprintf ( d_bugstr, "Creating scaled fonts." );
        else
          sprintf ( d_bugstr, "Creating 300 dpi fonts." );                    
        wrt_debug ();  
      }
// First, tf37, with tl1a.hpf - use t1a.hpf
    strcpy ( fname, "t1a.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T1A, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;   
    tf37_fc = fstchr; 
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf37_lc = lstchr; 
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd ); // 13, 624 - 13, 672
        wrt_debug ( );       
        sprintf ( d_bugstr, "The source characters are %d x %d.", x_pt, y_pt ); // 48, 100
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      { 
        tf37_xpt = y_pt;
        tf37_ypt = x_pt;
        xdiv = 1.0;
        ydiv = 1.0;                   
      }
    else
      {
        tf37_xpt = (int) ( (float) y_pt * pr_scaley );
        tf37_ypt = (int) ( (float) x_pt * pr_scalex );  
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;           
      }
    x2 = tf37_xpt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    tgt_lin = x2; 
    tgt_grd = tgt_lin * tf37_ypt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Tgt_lin = %d, Tgt_grd = %d", tgt_lin, tgt_grd ); // 13, 624 - 13, 672
        wrt_debug ( );  
      }
    buf_siz = tgt_grd * tnch;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "X. Buffer size = %ld.", buf_siz );
        wrt_debug ();
      }
    tf37buf = malloc ( (size_t) buf_siz );        
    if ( tf37buf == NULL )
      Msg ( "X. Memory allocation failed!" );      
    memset( tf37buf, 0, buf_siz );                                                 
    
    for ( i = fstchr; i <= lstchr; ++i )
      {                          
        j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
        for ( r2 = 0; r2 < tf37_ypt; ++r2 ) 
          {
            c1 = (int) ((float) (tf37_ypt-r2) * ydiv);              
            for ( c2 = 0; c2 < tf37_xpt; ++c2 ) 
              {
                r1 = (int) ((float) (c2) * xdiv);
                if ( !tst_buf ( &tf_buf[j], r1, c1, x_pt, y_pt ) )
                  {
                  wrt_buf ( ( tf37buf + ( i - fstchr ) * tgt_grd ), r2, c2, tf37_xpt, tf37_ypt );
                  tst = 1;
                  }
                else
                  tst = 0;
              }
          }  
        bmBits = ( tf37buf + ( i - fstchr ) * tgt_grd );   
        tf37_hbm[i-fstchr] = CreateBitmap ( tf37_xpt, tf37_ypt, 1, 1, bmBits );        
      }            
//  Next is: tl2a 38 
    strcpy ( fname, "t2a.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T2A, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;   
    tf38_fc = fstchr; 
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf38_lc = lstchr; 
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd ); // 13, 624 - 13, 672
        wrt_debug ( );       
        sprintf ( d_bugstr, "The source characters are %d x %d.", x_pt, y_pt ); // 48, 100
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      { 
        tf38_xpt = y_pt;
        tf38_ypt = x_pt;
        xdiv = 1.0;
        ydiv = 1.0;                   
      }
    else
      {
        tf38_xpt = (int) ( (float) y_pt * pr_scaley );
        tf38_ypt = (int) ( (float) x_pt * pr_scalex );  
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;           
      }
    x2 = tf38_xpt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    tgt_lin = x2; 
    tgt_grd = tgt_lin * tf38_ypt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Tgt_lin = %d, Tgt_grd = %d", tgt_lin, tgt_grd ); // 13, 624 - 13, 672
        wrt_debug ( );  
      }
    buf_siz = tgt_grd * tnch;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "X. Buffer size = %ld.", buf_siz );
        wrt_debug ();
      }    
    tf38buf = malloc ( (size_t) buf_siz );        
    if ( tf38buf == NULL )
      Msg ( "X. Memory allocation failed!" );      
    memset( tf38buf, 0, buf_siz );                                                 
    for ( i = fstchr; i <= lstchr; ++i )
      {                          
        j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
        for ( r2 = 0; r2 < tf38_ypt; ++r2 ) // 94 - 91 is going wrong !!!
          {
            c1 = (int) ((float) (tf38_ypt-r2) * ydiv);              
            for ( c2 = 0; c2 < tf38_xpt; ++c2 ) // 200
              {
                r1 = (int) ((float) (c2) * xdiv);
                if ( !tst_buf ( &tf_buf[j], r1, c1, x_pt, y_pt ) )
                  {
                  wrt_buf ( ( tf38buf + ( i - fstchr ) * tgt_grd ), r2, c2, tf38_xpt, tf38_ypt );
                  tst = 1;
                  }
                else
                  tst = 0;
              }
          }  
        bmBits = ( tf38buf + ( i - fstchr ) * tgt_grd );   
        tf38_hbm[i-fstchr] = CreateBitmap ( tf38_xpt, tf38_ypt, 1, 1, bmBits );
      }     
//  Next is: tl1b 39 
    strcpy ( fname, "t1b.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T1B, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }    
    fstchr = tf_buf[4] + tf_buf[5] * 256;   
    tf39_fc = fstchr; 
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf39_lc = lstchr; 
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd ); // - 4, 264
        wrt_debug ( );       
        sprintf ( d_bugstr, "The source characters are %d x %d.", x_pt, y_pt ); // 32, 66
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      { 
        tf39_xpt = y_pt;
        tf39_ypt = x_pt;
        xdiv = 1.0;
        ydiv = 1.0;                   
      }
    else
      {
        tf39_xpt = (int) ( (float) y_pt * pr_scaley );
        tf39_ypt = (int) ( (float) x_pt * pr_scalex );  
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;           
      }

    x2 = tf39_xpt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    tgt_lin = x2; 
    if ( tgt_lin % 2 )
      ++tgt_lin;     
    tgt_grd = tgt_lin * tf39_ypt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Tgt_lin = %d, Tgt_grd = %d", tgt_lin, tgt_grd ); // 17, 1088
        wrt_debug ( );  
      }    
    buf_siz = tgt_grd * tnch;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "X. Buffer size = %ld.", buf_siz );
        wrt_debug ();
      }
    tf39buf = malloc ( (size_t) buf_siz );        
    if ( tf39buf == NULL )
      Msg ( "X. Memory allocation failed!" );      
    memset( tf39buf, 0, buf_siz );                                                 
    
    for ( i = fstchr; i <= lstchr; ++i )
      {                          
        j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
        for ( r2 = 0; r2 < tf39_ypt; ++r2 ) // 94 - 91 is going wrong !!!
          {
            c1 = (int) ((float) (tf39_ypt-r2) * ydiv);              
            for ( c2 = 0; c2 < tf39_xpt; ++c2 ) // 200
              {
                r1 = (int) ((float) (c2) * xdiv);
                if ( !tst_buf ( &tf_buf[j], r1, c1, x_pt, y_pt ) )
                  {
                  wrt_buf ( ( tf39buf + ( i - fstchr ) * tgt_grd ), r2, c2, tf39_xpt, tf39_ypt );
                  tst = 1;
                  }
                else
                  tst = 0;
              }
          }  
        bmBits = ( tf39buf + ( i - fstchr ) * tgt_grd );   
        tf39_hbm[i-fstchr] = CreateBitmap ( tf39_xpt, tf39_ypt, 1, 1, bmBits );        
      }     
//  Next is: tl2b 40  
    strcpy ( fname, "t2b.hpf" );  
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T2B, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;   
    tf40_fc = fstchr; 
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {     
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf40_lc = lstchr; 
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd ); // - 4, 264
        wrt_debug ( );       
        sprintf ( d_bugstr, "The source characters are %d x %d.", x_pt, y_pt ); // 32, 66
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      { 
        tf40_xpt = y_pt;
        tf40_ypt = x_pt;
        xdiv = 1.0;
        ydiv = 1.0;                   
      }
    else
      {
        tf40_xpt = (int) ( (float) y_pt * pr_scaley );
        tf40_ypt = (int) ( (float) x_pt * pr_scalex );  
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;           
      }
    x2 = tf40_xpt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    tgt_lin = x2; 
    if ( tgt_lin % 2 ) // here to here
      ++tgt_lin;     
    tgt_grd = tgt_lin * tf40_ypt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Tgt_lin = %d, Tgt_grd = %d", tgt_lin, tgt_grd ); // 17, 1088
        wrt_debug ( );  
      }
    buf_siz = tgt_grd * tnch;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "X. Buffer size = %ld.", buf_siz );
        wrt_debug ();
      }
    tf40buf = malloc ( (size_t) buf_siz );        
    if ( tf40buf == NULL )
      Msg ( "X. Memory allocation failed!" );      
    memset( tf40buf, 0, buf_siz );                                                 
    for ( i = fstchr; i <= lstchr; ++i )
      {                          
        j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
        for ( r2 = 0; r2 < tf40_ypt; ++r2 ) // 94 - 91 is going wrong !!!
          {
            c1 = (int) ((float) (tf40_ypt-r2) * ydiv);              
            for ( c2 = 0; c2 < tf40_xpt; ++c2 ) // 200
              {
                r1 = (int) ((float) (c2) * xdiv);
                if ( !tst_buf ( &tf_buf[j], r1, c1, x_pt, y_pt ) )
                  {
                  wrt_buf ( ( tf40buf + ( i - fstchr ) * tgt_grd ), r2, c2, tf40_xpt, tf40_ypt );
                  tst = 1;
                  }
                else
                  tst = 0;
              }
          }  
        bmBits = ( tf40buf + ( i - fstchr ) * tgt_grd );   
        tf40_hbm[i-fstchr] = CreateBitmap ( tf40_xpt, tf40_ypt, 1, 1, bmBits );        
      }     
//  Next is: tl1c 41
    strcpy ( fname, "t1b.hpf" );      
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "TL1C, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;   
    tf41_fc = fstchr; 
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf41_lc = lstchr; 
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd ); 
        wrt_debug ( );   
        sprintf ( d_bugstr, "The source characters are %d x %d.", y_pt, x_pt ); 
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      {
        buf_siz = tnch * byt_grd;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "1. Buffer size = %ld.", buf_siz );
            wrt_debug ( );          
          }
        tf41buf = malloc ( (size_t) buf_siz );
        if ( tf41buf == NULL )
           {
             Msg ( "1. Memory allocation failed!" );
             exit (0);
           }
        tf41_xpt = x_pt;
        tf41_ypt = y_pt;
        memset( tf41buf, 0, buf_siz );           
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            x = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed; 
            for ( j = 0; j < byt_grd; ++j )
               *( tf41buf + ( i - fstchr ) * byt_grd + j ) = ~tf_buf[ x + j ];
            bmBits = tf41buf + ( i - fstchr ) * byt_grd;
            tf41_hbm[i-fstchr] = CreateBitmap ( tf41_xpt, tf41_ypt, 1, 1, bmBits );
          }            
      }
    else // If scaling is needed:
      {
        tf41_xpt = (int) ( (float) x_pt * pr_scalex );
        tgt_lin = tf41_xpt / 8; 
        if ( tgt_lin * 8 < tf41_xpt ) // If not a multiple?
          ++tgt_lin;
        if ( tgt_lin % 2 ) // here to here
          ++tgt_lin;          
        if ( d_bug > 1 )
          {          
            sprintf ( d_bugstr, "xpt: %d, tgt_lin: %d.", tf41_xpt, tgt_lin );
            wrt_debug ();          
          }   
        tf41_ypt = (int) ( (float) y_pt * pr_scaley );
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "New X width: %d, new Y height: %d.", tf41_xpt, tf41_ypt ); // 200, 96
            wrt_debug ( );          
          }
        tgt_grd = tgt_lin * tf41_ypt;
        buf_siz = tgt_grd * tnch;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "2. Buffer size = %ld.", buf_siz );
            wrt_debug ( );                  
          }
        tf41buf = malloc ( (size_t) buf_siz );        
        if ( tf41buf == NULL )
          Msg ( "2. Memory allocation failed!" );      
        memset( tf41buf, 0, buf_siz );           
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
            for ( r = 0; r < tf41_ypt; ++r ) // 96
              {
                r2 = (int) ((float) r * ydiv);              
                for ( c = 0; c < tf41_xpt; ++c ) // 200
                  {
                    c2 = (int) ((float) c * xdiv);
                    if ( !tst_buf ( &tf_buf[j], r2, c2, x_pt, y_pt ) )
                      wrt_buf ( ( tf41buf + ( i - fstchr ) * tgt_grd ), r, c, tf41_xpt, tf41_ypt );
                  }
              }  
            bmBits = ( tf41buf + ( i - fstchr ) * tgt_grd );   
            tf41_hbm[i-fstchr] = CreateBitmap ( tf41_xpt, tf41_ypt, 1, 1, bmBits );            
          }            
      }  
//  Next is: tl2c 42
    strcpy ( fname, "t2b.hpf" );      
    h = CreateFile ( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( h == INVALID_HANDLE_VALUE )
      Msg ( "Open font file failed." );         
    fil_len = GetFileSize ( h, NULL );
    sprintf ( d_bugstr, "T2B, file length = %ld", fil_len );
    wrt_debug ( );   
    if ( FALSE == ReadFile( h, tf_buf, fil_len, (LPDWORD) &Bytes, NULL) )
      {
        Msg ( "ReadFile error!" );
        CloseHandle( h );
        sprintf ( d_bugstr, "Bytes read from file: %d of %d", Bytes, fil_len );
        wrt_debug (); 
        return;
      }        
    CloseHandle( h ); 
    chhed = 12; // Total number of header bytes per character 
    tnch = tf_buf[2] + tf_buf[3] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "tnch = %d", tnch ); 
        wrt_debug ( );              
      }
    fstchr = tf_buf[4] + tf_buf[5] * 256;   
    tf42_fc = fstchr; 
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "fstchr = %d", fstchr );
        wrt_debug ( );   
      }
    x_pt = tf_buf[12] + tf_buf[13] * 256;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "x_pt = %d", x_pt );    
        wrt_debug ( );      
      }
    y_pt = tf_buf[14] + tf_buf[15] * 256;    
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "y_pt = %d", y_pt );        
        wrt_debug ( );   
      }
    lstchr = fstchr + tnch - 1;
    tf42_lc = lstchr; 
    x2 = x_pt / 8;
    x1 = x2 * 8;
    if ( x1 != x_pt )
      ++x2;
    if ( x2 % 2 )
      ++x2;       
    byt_lin = x2; 
    byt_grd = byt_lin * y_pt;
    if ( d_bug > 1 )
      {    
        sprintf ( d_bugstr, "Byt_lin = %d, Byt_grd = %d", byt_lin, byt_grd ); // 13, 624 - 13, 672
        wrt_debug ( );   
        sprintf ( d_bugstr, "The source characters are %d x %d.", y_pt, x_pt ); // 48, 100
        wrt_debug ( );   
        sprintf ( d_bugstr, "N = %d, first = %d.", tnch, fstchr );
        wrt_debug ( );                            
      }
    if ( scaling_needed == 0 )
      {
        buf_siz = tnch * byt_grd;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "1. Buffer size = %ld.", buf_siz );
            wrt_debug ( );          
          }
        tf42buf = malloc ( (size_t) buf_siz );
        if ( tf42buf == NULL )
           {
             Msg ( "1. Memory allocation failed!" );
             exit (0);
           }
        tf42_xpt = x_pt;
        tf42_ypt = y_pt;
        memset( tf42buf, 0, buf_siz );                                      
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            x = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed; 
            for ( j = 0; j < byt_grd; ++j )
               *( tf42buf + ( i - fstchr ) * byt_grd + j ) = ~tf_buf[ x + j ];
            bmBits = tf42buf + ( i - fstchr ) * byt_grd;
            tf42_hbm[i-fstchr] = CreateBitmap ( tf42_xpt, tf42_ypt, 1, 1, bmBits );            
          }            
      }
    else // If scaling is needed:
      {
        tf42_xpt = (int) ( (float) x_pt * pr_scalex );
        tgt_lin = tf42_xpt / 8; 
        if ( tgt_lin * 8 < tf42_xpt ) // If not a multiple?
          ++tgt_lin;
        if ( tgt_lin % 2 ) 
          ++tgt_lin;          
        if ( d_bug > 1 )
          {          
            sprintf ( d_bugstr, "xpt: %d, tgt_lin: %d.", tf42_xpt, tgt_lin );
            wrt_debug ();          
          }   
        tf42_ypt = (int) ( (float) y_pt * pr_scaley );
        xdiv = 1.0 / pr_scalex;
        ydiv = 1.0 / pr_scaley;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "New X width: %d, new Y height: %d.", tf42_xpt, tf42_ypt ); // 200, 96
            wrt_debug ( );          
          }
        tgt_grd = tgt_lin * tf42_ypt;
        buf_siz = tgt_grd * tnch;
        if ( d_bug > 1 )
          {        
            sprintf ( d_bugstr, "2. Buffer size = %ld.", buf_siz );
            wrt_debug ( );                  
          }
        tf42buf = malloc ( (size_t) buf_siz );        
        if ( tf42buf == NULL )
          Msg ( "2. Memory allocation failed!" );      
        memset( tf42buf, 0, buf_siz );                                      
        for ( i = fstchr; i <= lstchr; ++i )
          {                          
            j = 40 + (i - fstchr) * ( byt_grd + chhed ) + chhed;             
            for ( r = 0; r < tf42_ypt; ++r ) // 96
              {
                r2 = (int) ((float) r * ydiv);              
                for ( c = 0; c < tf42_xpt; ++c ) // 200
                  {
                    c2 = (int) ((float) c * xdiv);
                    if ( !tst_buf ( &tf_buf[j], r2, c2, x_pt, y_pt ) )
                      wrt_buf ( ( tf42buf + ( i - fstchr ) * tgt_grd ), r, c, tf42_xpt, tf42_ypt );
                  }
              }  
            bmBits = ( tf42buf + ( i - fstchr ) * tgt_grd );   
            tf42_hbm[i-fstchr] = CreateBitmap ( tf42_xpt, tf42_ypt, 1, 1, bmBits );
          }            
      }  
    if ( d_bug > 1 )
      {      
        sprintf ( d_bugstr, "Finished font creation." );
        wrt_debug ();
      }
  } // END - open_landscape_tfonts ()
  