/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
#define MAX_OB_DET 70  // Maximum in object queue for obscuring characters
#define WDQUMX   2048 /*** Array size for word queue ***/
#define A4_PAPER  1
#define A3_PAPER  2
#define Tabloid_PAPER  3
#define LETTER_PAPER  4
#define LEGAL_PAPER  5

struct  wrd_struct
  {
    int     lng;    /** Language - 1 = Tib, 2 = Skt **/
    int     flgs;   /** Flags for special purposes **/
    int     prf;    /** Prefix **/
    int     sup;    /** Superjoined letter **/
    int     rot;    /** Root letter **/
    int     sub;    /** Subjoined letter **/
    int     sub2;   /** 2nd Subjoined letter **/
    int     vw1;    /** First vowel **/
    int     vw2;    /** Second vowel **/
    int     vw3;    /** Third vowel **/
    int     suf1;   /** First suffix **/
    int     suf2;   /** Second suffix **/
    int     shd;    /** Required 'shad' at end **/
    int     hm;     /** Total horizontal length in dots **/
    int     fnt;    /** Flag for small font **/
    int     topx;   /** Reserved **/
    char    wrd[12];
    char    sktwrd[20];
  };
  
struct  c_map
  {
    int   fnt;
    int   chr;
    int   drp;
  };
  
#define FLBUFMX  500000  // was 256  !	Maximum file input buffer 

// ROUTINES IN TIBW.C:

void Msg ( unsigned char * );
void FileEdit ( void );
void DrawInfo ( void );
void DrawConfig ( void );
void SelectPrinter ( void );

// ROUTINES IN TIBW2.C:

void destroy_boxes ( void );

int PrintConfig ( void );
void set_a4_pecha ( void );
void set_a4_port ( void );
void set_a3_pecha ( void );
void set_Tabloid_pecha ( void );
void set_legal_pecha ( void );
void set_legal_port ( void );
void set_letter_pecha ( void );
void set_letter_port ( void );

// ROUTINES IN TF1.C:

void set_pms ( void );
void main_print ( void );
void setpmstop ( void );
void setpmstit ( void );
void setpmsbot ( void );
void sid_barst ( int, int ); // Top, on landscape page
void sid_barsb ( int, int ); // Bottom
void an_pags ( void );
void prn_pag ( int );
void prn_boxt ( int ); // Top, on landscape page
void prn_boxb ( int ); // Bottom
void set_pms_a4 ( void );
void set_pms_Tabloid ( void );
void set_pms_a3 ( void );
void set_pms_legal ( void );
void set_pms_letter ( void );

// ROUTINES IN TF2.C:

void getchrspc ( int, int *, int * );
int chrdrp ( int );
void getbasspc ( void );
void getglbspc ( void );
int gsup_chr ( char );
int sub_chr ( char );
int gssp_chr ( char );
int squsktid ( int );
void sldsktsyl ( int );
void chk_spcskt ( void );
int smlconv ( int );
int vwlshft ( void );
int chk_sktshift ( int, int );
void fixskt ( void );
void prn_syl ( void );
void dissktstr ( void );
void prn_skt ( void );
int chk_anus ( void );
void chk_spctib ( void );
void chk_subdrop ( void );
int chk_rotshift ( void );
void prn_wrd ( void );

// ROUTINES IN TF2A.C:

int anlystitp ( int * );
void anlyshead ( void );
int anlslin ( int );
void prn_nlin ( int, int );
void add2obsq ( int, int, int );
void add2jus ( int, int, int );
int jus ( void );
int compard ( struct obsc_dat *, struct obsc_dat * );
int comparl ( struct obsc_dat *, struct obsc_dat * );
void sort_obs ( int );
int chk_nxto ( int, int, int * );

// ROUTINES IN TF3.C:

int bck_dc ( char * );
int bck_sc ( char );
int chk_vw ( char );
int spec_an ( void );
int word_an ( void );
char get_fch ( void );
void fil_gnbuf ( void );
int gtnxwrd ( void );
void tst_wrd ( int );
void put_que ( void );
void shuf_que ( int );
void get_que ( int );
void str_an ( void );

// ROUTINES IN TF4.C:

int getsktid ( char );
int chk4skt ( void );
int issktvwl ( int );
int detsktid ( char * );
void an_skt ( void );

// ROUTINES IN TF5.C:

void spu_at ( int, int, unsigned );
int kil_blk ( char * );
int get_from_file ( int, long int, char *, unsigned );
void file_er ( void );

// ROUTINES IN TF6.C:

void prn_chr ( int, int, int );
void setcurpos ( int, int );
void backspc ( int );
void sel_fnt ( int );
void init_prn ( void );

// ROUTINES IN TF7.C:

void set_debug ( int );
void wrt_debug ( void );
void clos_debug ( void );
void prn_num ( int, int, int, int, int );
void an_titb ( void );
void prn_titb ( int, int, int );
void put_nque ( void );
void get_nque ( int );
void put_tque ( void );
void get_tque ( int );

// ROUTINES IN TPRINT.C:

void port_rout ( void );
void pn_chr ( int );
int lp_chr ( unsigned char );
void land_rout ( void );
void ver_lin ( int, int, int, int );
void hor_lin ( int, int, int, int );

// ROUTINES IN PREVIEW.C

int wrt_chr ( unsigned char ); // Move from here after testing?
void do_preview ( void );
void start_mempag ( void );
void save_mempag ( int x );
void read_mempag ( int x );

// ROUTINES IN TESTS.C:

void TestP01 ( void );
void TestL01 ( void );
void TestL02 ( void );

// ROUTINES IN TFONTS.C

void open_portrait_tfonts ( void ); // Make two - Portrait and Landscape.
int tst_buf ( char *, int, int, int, int );
void wrt_buf ( char *, int, int, int, int );
void clean_portrait (void);
void open_landscape_tfonts ( void ); 
void clean_landscape (void);
