/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
#define INCLUDE_COMMDLG_H       1
#include <windows.h>
#include <winspool.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <dos.h>
#include <string.h>
#include <malloc.h>
#include "tibw.h"
#include "tform.h"
#include "tibform.ext"

HWND Tibw_hwnd; // Main window
HANDLE Tibw_inst;      // instance handle
HDC Tibw_hdc;    // Main hdc
HFONT Tibw_font;  // Main window font
RECT deskrc;
int csx_on = 0;;

// Main local printer stuff:

int printer_selected = 0;
int file_selected = 0;
int info_drawn = 0;
int go_print = 0;
LPDEVMODE devmode;
LPDEVNAMES devnames;
LPCTSTR pdriver, pdevice, poutput;
int pr_papersize;
char papersize[20];

// Display window:
int scrn_x, scrn_y;
int fxchrwdth, fxchrhgt;

char EditTitle[] = "TibW";

COLORREF BkClr, ForClr;
LONG _EXPORT FAR PASCAL ConfigWindowProc( HWND, unsigned, UINT, LONG );
LONG _EXPORT FAR PASCAL PrevWindowProc( HWND, unsigned, UINT, LONG );

HINSTANCE   MyInstance;
static char TibwClass[10]="TibwClass";
static char confClass[10]="confClass";
static char prvwClass[10]="prvwClass";

HINSTANCE hInst; // The current instance
PRINTDLG prstr;
HFONT CFont;

static BOOL FirstInstance( HINSTANCE );
static BOOL AnyInstance( HINSTANCE, int, LPSTR );

long _EXPORT FAR PASCAL WindowProc( HWND, unsigned, UINT, LONG );

void Msg ( unsigned char *txt )
  {
    MessageBox( NULL, txt, "Message", MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL );
  }

// WinMain - initialization, message loop
int PASCAL WinMain( HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline,
                    int cmdshow )
{
    MSG         msg;
    if ( strcmp ( cmdline, "CSX" ) == 0 || strcmp ( cmdline, "csx" ) == 0 ) 
      {
        csx_on = 255;
        sprintf ( d_bugstr, "Command string: %s, decoding CSX.", cmdline );
        wrt_debug ();
      }
    MyInstance = this_inst;
    hInst = this_inst;
    if( !prev_inst ) {
        if( !FirstInstance( this_inst ) ) return( FALSE );
    }
    if( !AnyInstance( this_inst, cmdshow, cmdline ) ) return( FALSE );

    while( GetMessage( &msg, NULL, 0, 0 ) ) 
      {
        TranslateMessage( &msg );
        DispatchMessage( &msg );
      }
    return( msg.wParam );
  } // END - WinMain ()

// FirstInstance - register window class for the application,
//              and do any other application initialization
static BOOL FirstInstance( HINSTANCE this_inst )
{
    WNDCLASS    wc, wcc, wcc2;    
    BOOL        rc, rcc, rcc2;
// set up and register window classes 
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (LPVOID) WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof( DWORD );
    wc.hInstance = this_inst;
    wc.hIcon = LoadIcon( this_inst, "TibwIcon" );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = GetStockObject( WHITE_BRUSH );
    wc.lpszMenuName = "TibwMenu";
    wc.lpszClassName = TibwClass;
    rc = RegisterClass( &wc );

    wcc.style = CS_HREDRAW | CS_VREDRAW;
    wcc.lpfnWndProc = (LPVOID) ConfigWindowProc; // CHANGE THIS LATER !!! ??? !!!
    wcc.cbClsExtra = 0;
    wcc.cbWndExtra = sizeof( DWORD );
    wcc.hInstance = this_inst;
    wcc.hIcon = LoadIcon( this_inst, "TibwIcon" );
    wcc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wcc.hbrBackground = GetStockObject( WHITE_BRUSH );
    wcc.lpszMenuName = NULL;
    wcc.lpszClassName = confClass;
    rcc = RegisterClass( &wcc );
    
    wcc2.style = CS_HREDRAW | CS_VREDRAW;
    wcc2.lpfnWndProc = (LPVOID) PrevWindowProc; // CHANGE THIS LATER !!! ??? !!!
    wcc2.cbClsExtra = 0;
    wcc2.cbWndExtra = sizeof( DWORD );
    wcc2.hInstance = this_inst;
    wcc2.hIcon = LoadIcon( this_inst, "TibwIcon" );
    wcc2.hCursor = LoadCursor( NULL, IDC_ARROW );
    wcc2.hbrBackground = GetStockObject( WHITE_BRUSH );
    wcc2.lpszMenuName = NULL;
    wcc2.lpszClassName = prvwClass;
    rcc2 = RegisterClass( &wcc2 );  
    set_debug (1);
      
    return( rc );

} /* FirstInstance */

// AnyInstance - do work required for every instance of the application:
//                create the window, initialize data
static BOOL AnyInstance( HINSTANCE this_inst, int cmdshow, LPSTR cmdline )
{
    HWND        hwnd;
    HFONT hf;
    long lfHeight;
    SIZE sz; // For font data
    BOOL res;        
    HFONT g_hfFont = GetStockObject(DEFAULT_GUI_FONT);
    int initx, inity, initwidth, initheight;

    GetCurrentDirectory ( 80, (LPTSTR) cur_dir );
    sprintf ( d_bugstr, "Current directory: %s.", cur_dir );
    wrt_debug ();
    res = SystemParametersInfo( SPI_GETWORKAREA, 0, &deskrc, 0 );
    if ( res == NULL )
      {
        Msg ( "System parameter check failed!" );
        return (0);
      }
    sprintf ( d_bugstr, "Desktop: L: %d, R: %d, T: %d, B: %d", 
              deskrc.left, deskrc.right, deskrc.top, deskrc.bottom );
    wrt_debug ();                  
    deskx = deskrc.right - deskrc.left;
    desky = deskrc.bottom - deskrc.top;
    
    initwidth = 850;
    initheight = 700;

    initx = ( deskx - initwidth ) / 2;
    inity = ( desky - initheight ) / 2;

// create main window
    hwnd = CreateWindow(
        TibwClass,           	// class 
        "Tibw",  		// caption 
        WS_OVERLAPPEDWINDOW,    // style 
        /*initx,*/  CW_USEDEFAULT,   // init. x pos 
        /*inity,*/ CW_USEDEFAULT, // init. y pos
        initwidth,        	// init. x size 
        initheight,       	// init. y size 
        NULL,                   // parent window 
        NULL,                   // menu handle 
        this_inst,              // program handle 
        NULL                    // create parms 
        );

    if( !hwnd ) return( FALSE );

    Tibw_hwnd = hwnd;
    Tibw_hdc = GetDC ( hwnd );
    Tibw_inst = this_inst;
    ForClr = RGB(0,0,0);
    BkClr = RGB(255,255,255);  
    SetTextColor ( Tibw_hdc, ForClr );
    SetBkColor ( Tibw_hdc, BkClr );
    SelectObject( Tibw_hdc, g_hfFont);
    scrn_x = 20;
    scrn_y = 20;
// set up data associated with this window
    lfHeight = -MulDiv(12, GetDeviceCaps(Tibw_hdc, LOGPIXELSY), 72);
    hf = CreateFont(lfHeight, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "Courier New");
    if(hf)
      {
        DeleteObject(g_hfFont);
        g_hfFont = hf;
      }
    else
      {
        MessageBox(hwnd, "Font creation failed!", "Error", MB_OK | MB_ICONEXCLAMATION);
      }          
    SelectObject( Tibw_hdc, hf);        
    Tibw_font = hf;
    GetTextExtentPoint( Tibw_hdc, "MM", 2, &sz );
    fxchrwdth = sz.cx / 2;
    fxchrhgt = sz.cy;

// Set control variables' defaults:    
    file_selected = 0;    
    doall = 1;
    portprn = 0;
    juson = 1;    
    strpnum = 1;
    curwpnum = 1;
    startpage = 1;
    endpage = 999;
    btchmod = 255; // REMOVE THIS LATER, NOT NEEDED FOR WINDOWS VERSION   
    nobyon = 0;      /** Flag for suppression of 'byon' on last page **/    
    revprn = 0;
    noprint = 0;
    stoprint = 0;
    trim_mark = 0;
    num_copies = 1;      

    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_FILEOPEN, MF_GRAYED );
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PREVIEW, MF_GRAYED );
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PRINT, MF_GRAYED );
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTP, MF_GRAYED );            
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL1, MF_GRAYED );            
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL2, MF_GRAYED );             
    
    ShowWindow( hwnd, cmdshow ); // Need to move these back once have fixed the flashing
    UpdateWindow( hwnd );    // after file selection.
    
    return( TRUE );

} /* AnyInstance */

// * AboutDlgProc - processes messages for the about dialog. 
BOOL _EXPORT FAR PASCAL AboutDlgProc( HWND hwnd, unsigned msg, UINT wparam, LONG lparam )
{
    lparam = lparam;                    /* turn off warning */

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} /* AboutDlgProc */

// WindowProc - handle messages for the main application window 
LONG _EXPORT FAR PASCAL WindowProc( HWND hwnd, unsigned msg,
                                     UINT wparam, LONG lparam )
{
    FARPROC     proc;
    PAINTSTRUCT ps;
    switch( msg ) {
    case WM_COMMAND:
        switch( LOWORD( wparam ) ) {
        case MENU_ABOUT:
            proc = MakeProcInstance( (FARPROC)AboutDlgProc, MyInstance );
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)proc );
            FreeProcInstance( proc );
            break;
        case MENU_FILEOPEN:
            file_selected = 0;
            fil_nam[0] = 0;
            EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PRINT, MF_GRAYED );                
            FileEdit();
            if ( file_selected )
              {
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PREVIEW, MF_ENABLED );              
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PRINT, MF_ENABLED );                 
                go_print = 1;
                curwpnum = 1; // Good to reset this when printing more than one file
                DrawInfo ();
              }
            UpdateWindow (Tibw_hwnd);    
            break;
        case MENU_SETPRN: 
            SelectPrinter ();
            if ( printer_selected )
              {
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_FILEOPEN, MF_ENABLED );            
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTP, MF_ENABLED );            
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL1, MF_ENABLED );            
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL2, MF_ENABLED );                         
                DrawInfo ();                   
              }
            break;
	case MENU_PREVIEW:  // Maybe not needed
	    if ( !file_selected && !printer_selected )
	      Msg ( "Please select a file and printer" );
	    else if ( !file_selected )
	      Msg ( "Please select a file" );	      
	    else if ( !printer_selected )
	      Msg ( "Please select a printer" );	      	      
	    else
	      {
                paper_size_save = paper_size;
	        prev_on = 1;
                PrintConfig (); 	
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_FILEOPEN, MF_GRAYED );
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_SETPRN, MF_GRAYED );
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PRINT, MF_GRAYED );
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTP, MF_GRAYED );            
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL1, MF_GRAYED );            
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL2, MF_GRAYED );                     	    
              }                	
            break;              
	case MENU_PRINT:  
	    if ( !file_selected && !printer_selected )
	      Msg ( "Please select a file and printer" );
	    else if ( !file_selected )
	      Msg ( "Please select a file" );	      
	    else if ( !printer_selected )
	      Msg ( "Please select a printer" );	      	      
	    else
	      {
	        prev_on = 0;
                PrintConfig (); // to here	
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_FILEOPEN, MF_GRAYED );
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_SETPRN, MF_GRAYED );
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PREVIEW, MF_GRAYED );
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PRINT, MF_GRAYED );
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTP, MF_GRAYED );            
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL1, MF_GRAYED );            
                EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL2, MF_GRAYED );                     	    
              }
            break;   
        case MENU_TESTP:    
	    if ( !printer_selected )
	      Msg ( "Please select a printer" );	      	      
	    else       
              TestP01 ();
            break;                
        case MENU_TESTL1:
	    if ( !printer_selected )
	      Msg ( "Please select a printer" );	      	      
	    else        
              TestL01 ();              
            break;
        case MENU_TESTL2:
	    if ( !printer_selected )
	      Msg ( "Please select a printer" );	      	      
	    else        
              TestL02 ();              
            break;
	  }
     break;            
    case WM_PAINT:
      {
        BeginPaint (hwnd, &ps);
        if ( !go_print ) // This is set when configuration completed and printer selected
          DrawInfo ();
        else  
          DrawConfig (); // This later becomes main routine !!!
        EndPaint (hwnd, &ps);
      }
      break;
//  case WM_CREATE:
//      ;
//    break;
    case WM_DESTROY:
        clos_debug ();                      
        PostQuitMessage( 0 );
        break;

    default:
        return( DefWindowProc( hwnd, msg, wparam, lparam ) );
    }
    return( 0L );

} // END - WindowProc

// FileEdit - edit a new file
void FileEdit( void )
  {
    char        str[128];
    long int    len=0;
    int         rc;
    int         h;
    OPENFILENAME   of;
    static char   filterList[] = "File (*.txt,*.doc)" "\0" "*.txt;*.doc" "\0\0";

    file_read = 0;
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = Tibw_hwnd;
    of.lpstrFilter = (LPSTR) filterList;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = fil_nam;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.Flags = OFN_HIDEREADONLY;

    rc = GetOpenFileName( &of );
    if ( !rc )
      {
        file_selected = 0;
        return;  
      }
    if( fil_nam[0] != 0 ) 
      {
        h = open( fil_nam, O_RDONLY | O_BINARY );
        if( h < 0 ) 
          {
            sprintf( str,"Could not open file %s", fil_nam );
            MessageBox( Tibw_hwnd, str, EditTitle, MB_OK );
            file_selected = 0;            
            return;
          }
        len = filelength ( h );
        close ( h );
        file_selected = 15;
      }
    else
      file_selected = 0;  
} // END - FileEdit ()

// Routine to draw main screen with current settings.
void DrawInfo ( void )
  {
    WORD     pageheight;
    WORD     pagewidth;
    HBRUSH   hbr;
    RECT rc;
    
    GetClientRect( Tibw_hwnd, &rc);
    pagewidth = rc.right - rc.left;
    pageheight = rc.bottom - rc.top;
  
    hbr = GetStockObject ( WHITE_BRUSH );
    FillRect( Tibw_hdc, &rc, hbr );

    if ( !file_selected && !printer_selected ) // Should never get here now.
      {
        scrn_x = 30;
        scrn_y = 30;
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        "No file or printer selected!", 28, NULL );       
        scrn_y = 90;                        
        strcpy ( msg_str, "First select a printer, and then either select a file to print," );
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );                               
        scrn_y = 120;                        
        strcpy ( msg_str, "or print one of the test pages." );
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );                                                       
      }
    else if ( printer_selected )
      {
        scrn_x = 30;
        scrn_y = 30;
        
        if ( !file_selected )
          {
           ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        "No file selected!", 17, NULL );       
        scrn_y = scrn_y + 30;                        
          }     
        else                       
          {
            sprintf ( msg_str, "File: %s", fil_nam );                  
            ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen ( msg_str ), NULL );             
          }                                          
                        
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Printer selected: %s", pr_name );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL ); 
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Width = %d mms", pr_horzsize );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );        	    
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Height = %d mms", pr_vertsize );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );        	    	    
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Width = %d pixels", pr_horzres );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );        	    	    
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Height = %d pixels", pr_vertres );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL ); 
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "X, pixels per inch = %d", pr_logpixelsx );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );   
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Y, pixels per inch = %d", pr_logpixelsy );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );                                               
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Bits per pixel = %d", pr_bitspixel );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );  
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Physical width = %d pixels", pr_physicalwidth  );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );   
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Physical height = %d pixels", pr_physicalheight );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );    
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Physical offset X = %d pixels", pr_physicaloffsetx  );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );   
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Physical offset Y = %d pixels", pr_physicaloffsety );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );
        scrn_y = scrn_y + 30;                        
        sprintf ( msg_str, "Paper size set to: %s", papersize );        
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );                        
        scrn_y = scrn_y + 30;                                                
        if ( pr_orientation == DMORIENT_PORTRAIT )
          sprintf ( msg_str, "Paper orientation: portrait" ); // 1
        else if ( pr_orientation == DMORIENT_LANDSCAPE )
          sprintf ( msg_str, "Paper orientation: landscape" ); // 2   
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen(msg_str), NULL );                               
        if ( pr_orientation != DMORIENT_PORTRAIT )
          Msg ( "WARNING - printer driver not set to portrait mode!" );
        if ( !file_selected )
          {
            scrn_y = scrn_y + 60;                        
            sprintf ( msg_str, "Please select a file, or print a test page!" );                    
            ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                         msg_str, strlen(msg_str), NULL );
          }                        
      } // END of if ( !file_selected && printer_selected )
   } // DrawInfo

void DrawConfig ( void )
  {
    WORD     pageheight;
    WORD     pagewidth;
    HBRUSH   hbr;
    RECT rc;
    
    GetClientRect( Tibw_hwnd, &rc);
    pagewidth = rc.right - rc.left;
    pageheight = rc.bottom - rc.top;
    hbr = GetStockObject ( WHITE_BRUSH );
    FillRect( Tibw_hdc, &rc, hbr );
    scrn_x = 20;
    scrn_y = 30;
    sprintf ( msg_str, "File: %s", fil_nam );
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL, msg_str, strlen (msg_str), NULL );    
    scrn_y = scrn_y + 40;             
    sprintf ( msg_str, "Tibetan number for first page: %d, Western: %d.", strpnum, curwpnum );
    ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );   
    scrn_y = scrn_y + 40;  
    sprintf ( msg_str, "Tibetan printing to start at page %d, and stop at page %d.", startpage, endpage );
    ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );       
    scrn_y = scrn_y + 40;  
    sprintf ( msg_str, "Trim mark flag: %d, Portrait mode flag: %d.", trim_mark, portprn );
    ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );     
    scrn_y = scrn_y + 40;  
    sprintf ( msg_str, "No byon flag: %d, Reverse printing flag: %d.", nobyon, revprn );
    ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );
    scrn_y = scrn_y + 40;  
    sprintf ( msg_str, "Turn off printing flag: %d, Debug flag value: %d.", stoprint, d_bug );
    ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );
    scrn_y = scrn_y + 40;  
    sprintf ( msg_str, "Number of copies: %d.", num_copies );
    ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL ); 
    if ( !printer_selected )
      {
        scrn_y = scrn_y + 80;  
        sprintf ( msg_str, "Please select a printer." );
        ExtTextOut ( Tibw_hdc, scrn_x, scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );       
      }
  } // END - DrawConfig ()
   
void SelectPrinter ( void)
  {
    BOOL res;
    prstr.lStructSize = sizeof(PRINTDLG);
    prstr.hwndOwner = Tibw_hwnd;
    prstr.hDevMode = NULL; 
    prstr.hDevNames = NULL;
    prstr.hDC = NULL;
    prstr.Flags = PD_RETURNDC | PD_RETURNIC;
    prstr.nFromPage = 1;
    prstr.nToPage = 1000;
    prstr.nMinPage = 1;
    prstr.nMaxPage = 1000;
    prstr.nCopies = 1;
    prstr.hInstance = 0;
    prstr.lpfnPrintHook = NULL;
    prstr.lpfnSetupHook = NULL;
    prstr.lpPrintTemplateName = 0;
    prstr.lpSetupTemplateName = NULL;
    prstr.hPrintTemplate = NULL;
    prstr.hSetupTemplate = 0;            
   
    res = PrintDlg ( &prstr );
    sprintf ( d_bugstr, "Return from PrintDlg = %d", res );            
    wrt_debug ();
    if ( res == NULL )
      {
        printer_selected = 0;  
        return;    
      }
    devnames = (LPDEVNAMES) GlobalLock(prstr.hDevNames); // From PRINTDLG structure 
    pdriver = (LPCTSTR) devnames + devnames->wDeviceOffset;            	    
    pdevice = (LPCTSTR) devnames + devnames->wDeviceOffset;            	    
    poutput = (LPCTSTR) devnames + devnames->wDeviceOffset;            	                
    sprintf ( d_bugstr, "Printer driver: %s", pdriver );
    wrt_debug ();   
    sprintf ( d_bugstr, "Printer name: %s", pdevice );
    wrt_debug ();   	    
    sprintf ( d_bugstr, "Printer output: %s", poutput );
    wrt_debug ();   	 
	    
    strcpy ( pr_name, pdevice );
    GlobalUnlock(prstr.hDevNames);	
    pr_horzsize = GetDeviceCaps ( prstr.hDC, (int) HORZSIZE );
    sprintf ( d_bugstr, "Width = %d mms", pr_horzsize );
    wrt_debug ();
    pr_vertsize = GetDeviceCaps ( prstr.hDC, (int) VERTSIZE );
    sprintf ( d_bugstr, "Height = %d mms", pr_vertsize );
    wrt_debug ();            
    pr_horzres = GetDeviceCaps ( prstr.hDC, (int) HORZRES );
    sprintf ( d_bugstr, "Width = %d pixels", pr_horzres );
    wrt_debug ();
    pr_vertres = GetDeviceCaps ( prstr.hDC, (int) VERTRES );
    sprintf ( d_bugstr, "Height = %d pixels", pr_vertres );
    wrt_debug ();
    pr_logpixelsx = GetDeviceCaps ( prstr.hDC, (int) LOGPIXELSX );
    sprintf ( d_bugstr, "X, pixels per inch = %d", pr_logpixelsx );
    wrt_debug ();
    pr_logpixelsy = GetDeviceCaps ( prstr.hDC, (int) LOGPIXELSY );
    sprintf ( d_bugstr, "Y, pixels per inch = %d", pr_logpixelsy );
    wrt_debug ();
    pr_bitspixel = GetDeviceCaps ( prstr.hDC, (int) BITSPIXEL );
    sprintf ( d_bugstr, "Bits per pixel = %d", pr_bitspixel );
    wrt_debug ();
    pr_physicalwidth = GetDeviceCaps ( prstr.hDC, (int) PHYSICALWIDTH );
    sprintf ( d_bugstr, "Physical width = %d", pr_physicalwidth );
    wrt_debug ();
    pr_physicalheight = GetDeviceCaps ( prstr.hDC, (int) PHYSICALHEIGHT );
    sprintf ( d_bugstr, "Physical height = %d", pr_physicalheight );
    wrt_debug ();
    pr_physicaloffsetx = GetDeviceCaps ( prstr.hDC, (int) PHYSICALOFFSETX );
    sprintf ( d_bugstr, "Physical offset X = %d", pr_physicaloffsetx );
    wrt_debug ();
    pr_physicaloffsety = GetDeviceCaps ( prstr.hDC, (int) PHYSICALOFFSETY );
    sprintf ( d_bugstr, "Physical offset Y = %d", pr_physicaloffsety );
    wrt_debug ();   
      
    devmode = (LPDEVMODE) GlobalLock(prstr.hDevMode);

    sprintf ( d_bugstr, "DevMode, Driver version = %d", devmode->dmDriverVersion );
    wrt_debug ();   	 
    sprintf ( d_bugstr, "DevMode, dmYResolution = %d", devmode->dmYResolution );
    wrt_debug ();   	    
    sprintf ( d_bugstr, "DevMode, dmPrintQuality = %d", devmode->dmPrintQuality );
    wrt_debug ();       
    sprintf ( d_bugstr, "DevMode, dmPaperSize = %d", devmode->dmPaperSize );
    wrt_debug ();       
    sprintf ( d_bugstr, "DevMode, dmOrientation; = %d", devmode->dmOrientation );
    wrt_debug ();           
    pr_duplexflg = devmode->dmDuplex;    
    if ( pr_duplexflg == DMDUP_SIMPLEX )
      sprintf ( d_bugstr, "DevMode, Duplex flag is DMDUP_SIMPLEX" );
    else if ( pr_duplexflg == DMDUP_HORIZONTAL )
      sprintf ( d_bugstr, "DevMode, Duplex flag is DMDUP_HORIZONTAL" );      
    else if ( pr_duplexflg == DMDUP_VERTICAL )
      sprintf ( d_bugstr, "DevMode, Duplex flag is DMDUP_VERTICAL" );            
    wrt_debug ();       
       
    pr_papersize = devmode->dmPaperSize;
    pr_orientation = devmode->dmOrientation;
    if ( pr_papersize == DMPAPER_LETTER ) 
      {
        strcpy ( papersize, "Letter" );                
        paper_size = LETTER_PAPER;
      }       
    else if ( pr_papersize == DMPAPER_LEGAL )       
      {
        strcpy ( papersize, "Legal" );                
        paper_size = LEGAL_PAPER;
      }             
    else if ( pr_papersize == DMPAPER_A4 )   
      {    
        strcpy ( papersize, "A4" );          
        paper_size = A4_PAPER;
      }
    else if ( pr_papersize == DMPAPER_A4_EXTRA )       
      strcpy ( papersize, "A4 Extra" );                
    else if ( pr_papersize == DMPAPER_A4_PLUS )       
      strcpy ( papersize, "A4 Plus" );                      
    else if ( pr_papersize == DMPAPER_A3 )       
      {
        strcpy ( papersize, "A3" );                
        paper_size = A3_PAPER;
      }      
    else if ( pr_papersize == DMPAPER_A3_EXTRA )       
      strcpy ( papersize, "A3 Extra" );  
    else if ( pr_papersize == DMPAPER_11X17 )  
      {     
        strcpy ( papersize, "11x17" );        
        paper_size = Tabloid_PAPER;       
      }
    else 
      sprintf ( papersize, "Paper size unknown, %d", pr_papersize );                    
     
    sprintf ( d_bugstr, "Paper size: %s", papersize );
    wrt_debug (); 
    if ( pr_orientation == DMORIENT_PORTRAIT )
       sprintf ( d_bugstr, "Paper orientation: portrait" ); // 1
    else if ( pr_orientation == DMORIENT_LANDSCAPE )
       sprintf ( d_bugstr, "Paper orientation: landscape" ); // 2   
    else 
       sprintf ( d_bugstr, "Paper orientation unknown" );       
    wrt_debug ();    
    if ( paper_size != A3_PAPER && paper_size != Tabloid_PAPER && paper_size != A4_PAPER 
         && paper_size != LEGAL_PAPER && paper_size != LETTER_PAPER )
      Msg ( "WARNING - unsupported paper size!" );
    if ( pr_orientation != DMORIENT_PORTRAIT )    
      Msg ( "WARNING - printer must be set to portrait mode!" );      
// Tidy up:
    if (prstr.hDevMode)
      GlobalFree (prstr.hDevMode);
    if (prstr.hDevNames)
      GlobalFree (prstr.hDevNames);
    if ( pr_logpixelsx == 300 && pr_logpixelsy == 300 )
      {
        scaling_needed = 0;
        pr_scalex = 1.0; // These two not really needed.
        pr_scaley = 1.0;         
        logicx_max = pr_horzres;
        logicy_max = pr_vertres;        
      }
    else
      {
        scaling_needed = 1;
        pr_scalex = (float) pr_logpixelsx / 300.0;
        pr_scaley = (float) pr_logpixelsy / 300.0;
        logicx_max = (int) ((float) pr_horzres / pr_scalex );
        logicy_max = (int) ((float) pr_vertres / pr_scaley );
      }        
    sprintf ( d_bugstr, "Logicx_max = %d", logicx_max );
    wrt_debug ();
    sprintf ( d_bugstr, "Logicy_max = %d", logicy_max );
    wrt_debug ();    
    printer_selected = 1;
  }// END - SelectPrinter
  