/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
#define INCLUDE_COMMDLG_H       1
#include <windows.h>
#include <winspool.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <dos.h>
#include <string.h>
#include <malloc.h>
#include "tibw.h"
#include "tform.h"
#include "tibform.ext"

#define ID_A3PECHA   76
#define ID_TabloidPECHA  77
#define ID_A4PECHA   78
#define ID_A4PORT    79
#define ID_LEGALPECHA   80
#define ID_LEGALPORT    81
#define ID_LETTERPECHA   82
#define ID_LETTERPORT    83

//#define ID_A3BUTTON   121
#define ID_DOALLBUTTON   122
#define ID_JUSONBUTTON   123

#define ID_TRIM_MARKS  126
#define ID_PORTRAIT   127
#define ID_NO_BYON  129

#define ID_REVPRN 131

#define ID_STOP_PRINT_BUTTON	125
#define ID_PRINT_BUTTON	124			// Button identifier
#define ID_PRINT_BUTTON2 127			// Button identifier

// Control variables:

HWND hWndA3pecha, hWndA4pecha, hWndA4port;
HWND hWndTabloidpecha, hWndLegalpecha, hWndLegalport;
HWND hWndLetterpecha, hWndLetterport;
HWND hWndDoallButton; // doall printing check boxes
HWND hWndjuson; // Justification mode on or off
HWND hWndTstart, hWndTstop; // Tibetan pages to start and stop printing - startpage, endpage;
HWND hWndstrpnum; // Tibetan number for first page - strpnum;
HWND hWndwpnum; // curwpnum; // Western nnumber for first page
HWND hWndTrim;   // Trim marks
HWND hWndPort;   // Portrait mode printing
HWND hWndNoByon;   // Open printing, no "byon"
HWND hWndRevprn;   // Reverse printing - not implemented yet
HWND hWndstopprn; // Flag for stopping printing
HWND hWnddebug;  // Debug flag edit box
HWND hWndcopies; // Window for number of copies, int num_copies;
HWND hWndButton; // Main Print button 

HWND conf_hwnd; // config window
HANDLE conf_inst; // configinstance handle
HDC conf_hdc;    // config hdc
HFONT conf_font;  // config window font  -- do we need this ???
int c_scrn_x, c_scrn_y; // Coordinates for configuration window

// This file is really an extension of tibw.c; the following are "locals":

extern HWND Tibw_hwnd; // Main window
extern HANDLE Tibw_inst; // main instance handle
extern int file_selected;
extern int go_print;
extern COLORREF BkClr, ForClr;
extern int fxchrwdth, fxchrhgt;
extern HINSTANCE hInst; // The current instance

// Configuration dialogue box, from here

void destroy_boxes ( void ) // For configuration dialogue box
  {
    DestroyWindow( hWndA3pecha ); 
    DestroyWindow( hWndTabloidpecha );     
    DestroyWindow( hWndA4pecha ); 
    DestroyWindow( hWndA4port ); 
    DestroyWindow( hWndLegalpecha ); 
    DestroyWindow( hWndLegalport );     
    DestroyWindow( hWndLetterpecha ); 
    DestroyWindow( hWndLetterport );     
    DestroyWindow( hWndDoallButton ); 
    DestroyWindow( hWndTstart );
    DestroyWindow( hWndTstop );
    DestroyWindow( hWndstrpnum );  
    DestroyWindow( hWndwpnum );        
    DestroyWindow( hWndstopprn );          
    DestroyWindow( hWnddebug );
    DestroyWindow( hWndjuson );        
    DestroyWindow( hWndcopies );
    DestroyWindow( hWndButton );          
    DestroyWindow( hWndTrim );          
    DestroyWindow( hWndPort );                    
    DestroyWindow( hWndNoByon );          
    DestroyWindow( hWndRevprn );      
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_FILEOPEN, MF_ENABLED );
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PREVIEW, MF_ENABLED );    
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_SETPRN, MF_ENABLED );
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PRINT, MF_ENABLED );          
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTP, MF_ENABLED );                    
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL1, MF_ENABLED );                    
    EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL2, MF_ENABLED );                            
  }  // END - destroy_boxes ()

LONG _EXPORT FAR PASCAL ConfigWindowProc( HWND hwnd, unsigned msg,
                                     UINT wparam, LONG lparam )
  {
    switch( msg ) {
    case WM_COMMAND:
        switch( LOWORD( wparam ) ) {
	case ID_PRINT_BUTTON:
	  if ( file_selected )
            go_print = 1;
          SendMessage( hWndTstart, WM_GETTEXT, 20, (LPARAM) msg_str);
          startpage = atoi (msg_str);
          SendMessage( hWndTstop, WM_GETTEXT, 20, (LPARAM) msg_str);
          endpage = atoi (msg_str);	  
          SendMessage( hWndstrpnum, WM_GETTEXT, 20, (LPARAM) msg_str);
          strpnum = atoi (msg_str);
          SendMessage( hWndwpnum, WM_GETTEXT, 20, (LPARAM) msg_str);
          curwpnum = atoi (msg_str);  
          SendMessage( hWndcopies, WM_GETTEXT, 20, (LPARAM) msg_str);
          num_copies = atoi (msg_str);  
          SendMessage( hWnddebug, WM_GETTEXT, 20, (LPARAM) msg_str);
          d_bug = atoi (msg_str);              
          if ( prev_on )
            {
              DrawConfig (); // Displays printing information on screen
              if ( a3_pecha )
                paper_size = A3_PAPER;  
              else if ( a4_pecha || a4_port )
                paper_size = A4_PAPER;  
              else if ( legal_pecha || legal_port )
                paper_size = LEGAL_PAPER;  
              else if ( letter_pecha || letter_port )
                paper_size = LETTER_PAPER;  
              else if ( Tabloid_pecha )
                paper_size = Tabloid_PAPER;                                                  
              do_preview ();
            }
          else
            {
              set_pms ();    // This is in TF1.C - does not set A4/A3 parameters       
              DrawConfig (); // Displays printing information on screen
              main_print (); // Sets the A3/A4 parameters - in TF1, calls port_rout            
            }
          DestroyWindow ( conf_hwnd );          
          break;  
          case ID_DOALLBUTTON:
            if ( doall )
              {
                doall = 0;
                SendMessage( hWndDoallButton, BM_SETCHECK, BST_UNCHECKED, 0);                 
              }
            else
              { 
                doall = 1;
                SendMessage( hWndDoallButton, BM_SETCHECK, BST_CHECKED, 0); 
              }
            break;
          case ID_JUSONBUTTON:
            if ( juson )
              {
                juson = 0;
                SendMessage ( hWndjuson, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                juson = 1;
                SendMessage( hWndjuson, BM_SETCHECK, BST_CHECKED, 0); 
              }
            break;
          case ID_STOP_PRINT_BUTTON:
            if ( stoprint )
              {
                stoprint = 0;
                SendMessage ( hWndstopprn, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                stoprint = 1;
                SendMessage( hWndstopprn, BM_SETCHECK, BST_CHECKED, 0); 
              }
            break;      
          case ID_TRIM_MARKS:
            if ( trim_mark )
              {
                trim_mark = 0;
                SendMessage ( hWndTrim, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                trim_mark = 1;
                SendMessage( hWndTrim, BM_SETCHECK, BST_CHECKED, 0); 
              }
            break;      
          case ID_PORTRAIT:
            if ( portprn )
              {
                portprn = 0;
                SendMessage ( hWndPort, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                portprn = 1;
                SendMessage( hWndPort, BM_SETCHECK, BST_CHECKED, 0); 
              }
            break;      
          case ID_NO_BYON:
            if ( nobyon )
              {
                nobyon = 0;
                SendMessage ( hWndNoByon, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                nobyon = 1;
                SendMessage( hWndNoByon, BM_SETCHECK, BST_CHECKED, 0); 
              }
            break;     
          case ID_REVPRN:
            if ( revprn )
              {
                revprn = 0;
                SendMessage ( hWndRevprn, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                revprn = 1;
                SendMessage( hWndRevprn, BM_SETCHECK, BST_CHECKED, 0);                 
              }
            break;                 
          case ID_A3PECHA:
            if ( a3_pecha )
              {
                a3_pecha = 0;
                SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                a3_pecha = 1;
                SendMessage( hWndA3pecha, BM_SETCHECK, BST_CHECKED, 0);                 
                set_a3_pecha ();
              }
            break;  
          case ID_TabloidPECHA:
            if ( Tabloid_pecha )
              {
                Tabloid_pecha = 0;
                SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                Tabloid_pecha = 1;
                SendMessage( hWndTabloidpecha, BM_SETCHECK, BST_CHECKED, 0);                 
                set_Tabloid_pecha ();
              }
            break;              
          case ID_A4PECHA:
            if ( a4_pecha )
              {
                a4_pecha = 0;
                SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                a4_pecha = 1;
                SendMessage( hWndA4pecha, BM_SETCHECK, BST_CHECKED, 0);                 
                set_a4_pecha ();
              }
            break;   
          case ID_A4PORT:
            if ( a4_port )
              {
                a4_port = 0;
                SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                a4_port = 1;
                SendMessage( hWndA4port, BM_SETCHECK, BST_CHECKED, 0);                 
                set_a4_port (); 
              }
            break;  
          case ID_LEGALPECHA:
            if ( legal_pecha )
              {
                legal_pecha = 0;
                SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                legal_pecha = 1;
                SendMessage( hWndLegalpecha, BM_SETCHECK, BST_CHECKED, 0);                 
                set_legal_pecha ();
              }
            break;   
          case ID_LEGALPORT:
            if ( legal_port )
              {
                legal_port = 0;
                SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                legal_port = 1;
                SendMessage( hWndLegalport, BM_SETCHECK, BST_CHECKED, 0);                 
                set_legal_port (); 
              }
            break;                                           
          case ID_LETTERPECHA:
            if ( letter_pecha )
              {
                letter_pecha = 0;
                SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                letter_pecha = 1;
                SendMessage( hWndLetterpecha, BM_SETCHECK, BST_CHECKED, 0);                 
                set_letter_pecha ();
              }
            break;   
          case ID_LETTERPORT:
            if ( letter_port )
              {
                letter_port = 0;
                SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0); 
              }
            else
              { 
                letter_port = 1;
                SendMessage( hWndLetterport, BM_SETCHECK, BST_CHECKED, 0);                 
                set_letter_port (); 
              }
            break;                                                       
      }
    break;
    case WM_DESTROY:
      destroy_boxes ();
      EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_FILEOPEN, MF_ENABLED );
      EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PREVIEW, MF_ENABLED );          
      EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_SETPRN, MF_ENABLED );
      EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_PRINT, MF_ENABLED );          
      EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTP, MF_ENABLED );                    
      EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL1, MF_ENABLED );                    
      EnableMenuItem ( GetMenu (Tibw_hwnd), MENU_TESTL2, MF_ENABLED );                                
      break;
    default:
        return( DefWindowProc( hwnd, msg, wparam, lparam ) );
    }
    return( 0L );
  } // END - ConfigWindowProc

int PrintConfig ( void ) 
  {
    HFONT hf;
    long lfHeight;
    HFONT g_hfFont; 
    PAINTSTRUCT ps;
    WORD     pageheight;
    WORD     pagewidth;
    int initx, inity, initwidth, initheight;    
    RECT rc;    

    curwpnum = 1; // This is to reset it to default if a file has been printed.
    		  // Need a generic method for doing this for other values.
    initwidth = 900;
    initheight = 650;
    initx = ( deskx - initwidth ) / 2;
    inity = ( desky - initheight ) / 2;
    if ( prev_on )
      strcpy ( msg_str, "Preview setup" );
    else  
      strcpy ( msg_str, "Printing setup" );      
    conf_hwnd = CreateWindow (
        "confClass", 	
        msg_str, 
        WS_CAPTION|WS_SYSMENU|WS_VISIBLE|WS_MINIMIZEBOX, // Makes non-resizable
        initx, 		
        inity, 		
        initwidth,    	
        initheight,   
        Tibw_hwnd,   
        NULL,       
        Tibw_inst,   
        NULL  
       );
    if( !conf_hwnd ) 
      {
         Msg ("Creating pop_up failed!");
         return( FALSE );
      }
     
    GetClientRect( conf_hwnd, &rc);
    pagewidth = rc.right - rc.left;
    pageheight = rc.bottom - rc.top;  
    BeginPaint (conf_hwnd, &ps);        
    conf_hdc = ps.hdc;
    SetTextColor ( conf_hdc, ForClr );
    SetBkColor ( conf_hdc, BkClr );      
    g_hfFont = GetStockObject(DEFAULT_GUI_FONT);          
    lfHeight = -MulDiv(12, GetDeviceCaps(conf_hdc, LOGPIXELSY), 72);
    hf = CreateFont( lfHeight, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "Courier New");
    if (hf)
      {
        DeleteObject(g_hfFont);
        g_hfFont = hf;
      }
    else
      MessageBox( conf_hwnd, "Font creation failed!", "Error", MB_OK | MB_ICONEXCLAMATION);
    SelectObject( conf_hdc, hf);        
    conf_font = hf;      
    SelectObject( conf_hdc, g_hfFont);        
    conf_font = g_hfFont;            
    c_scrn_x = 20;
    c_scrn_y = 30; 
    sprintf ( msg_str, "File: %s", fil_nam );
    ExtTextOut ( conf_hdc, c_scrn_x, c_scrn_y, ETO_OPAQUE, NULL, msg_str, strlen (msg_str), NULL );    
    a4_pecha = 0; 
    a3_pecha = 0; 
    legal_pecha = 0;  
    letter_pecha = 0;              
    Tabloid_pecha = 0;      
    if ( paper_size == A4_PAPER )
      a4_pecha = 1; 
    else if ( paper_size == A3_PAPER )
      a3_pecha = 1; 
    else if ( paper_size == LEGAL_PAPER )
      legal_pecha = 1;  
    else if ( paper_size == LETTER_PAPER )
      letter_pecha = 1;              
    else if ( paper_size == Tabloid_PAPER )
      Tabloid_pecha = 1;                    
    c_scrn_y = c_scrn_y + 45;
    MoveToEx ( conf_hdc, 0, c_scrn_y, NULL );
    LineTo ( conf_hdc, pagewidth, c_scrn_y );  
// ------------------- A3 pecha printing 
     strcpy ( msg_str, "A3 pecha:" );
     c_scrn_x = 20;
     c_scrn_y = c_scrn_y + 20; 
     hWndA3pecha = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_A3PECHA, 
                         hInst, NULL );
     SendMessage( hWndA3pecha, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( a3_pecha )
       SendMessage( hWndA3pecha, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0); 
// ------------------- 11x17 Tabloid pecha printing 
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;    
     strcpy ( msg_str, "11x17 pecha:" );
     hWndTabloidpecha = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_TabloidPECHA, 
                         hInst, NULL );
     SendMessage( hWndTabloidpecha, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( Tabloid_pecha )
       SendMessage( hWndTabloidpecha, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);        
// ------------------- A4 pecha printing 
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     strcpy ( msg_str, "A4 pecha:" );
     hWndA4pecha = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_A4PECHA, 
                         hInst, NULL );
     SendMessage( hWndA4pecha, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( a4_pecha )
       SendMessage( hWndA4pecha, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);  
// ------------------- A4 portrait printing 
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     strcpy ( msg_str, "A4 portrait:" );
     hWndA4port = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_A4PORT, 
                         hInst, NULL );

    SendMessage( hWndA4port, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
    if ( a4_port )
      SendMessage( hWndA4port, BM_SETCHECK, BST_CHECKED, 0); 
    else
      SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);  
//------------------------------ Legal pecha 
     c_scrn_x = 20;
     c_scrn_y = c_scrn_y + 40;      
     strcpy ( msg_str, "Legal pecha:" );
     hWndLegalpecha = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_LEGALPECHA, 
                         hInst, NULL );
     SendMessage( hWndLegalpecha, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( legal_pecha )
       SendMessage( hWndLegalpecha, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);  
// ------------------- Legal portrait printing 
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     strcpy ( msg_str, "Legal portrait:" );
     hWndLegalport = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_LEGALPORT, 
                         hInst, NULL );
    SendMessage( hWndLegalport, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
    if ( legal_port )
      SendMessage( hWndLegalport, BM_SETCHECK, BST_CHECKED, 0); 
    else
      SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);  
//------------------------------ letter pecha 
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40; 
     strcpy ( msg_str, "Letter pecha:" );
     hWndLetterpecha = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_LETTERPECHA, 
                         hInst, NULL );
     SendMessage( hWndLetterpecha, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( letter_pecha )
       SendMessage( hWndLetterpecha, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);  
// ------------------- letter portrait printing 
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     strcpy ( msg_str, "Letter portrait:" );
     hWndLetterport = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_LETTERPORT, 
                         hInst, NULL );
    SendMessage( hWndLetterport, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
    if ( letter_port )
      SendMessage( hWndLetterport, BM_SETCHECK, BST_CHECKED, 0); 
    else
      SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);  
    c_scrn_y = c_scrn_y + 50;
    MoveToEx ( conf_hdc, 0, c_scrn_y, NULL );
    LineTo ( conf_hdc, pagewidth, c_scrn_y );         
//-------------------------- Frame printing and Justification
     c_scrn_x = 20;
     c_scrn_y = c_scrn_y + 20; 
     strcpy ( msg_str, "Frame printing:" );
     hWndDoallButton = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_DOALLBUTTON, 
                         hInst, NULL );
     SendMessage(hWndDoallButton, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( doall )
        SendMessage( hWndDoallButton, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndDoallButton, BM_SETCHECK, BST_UNCHECKED, 0);  
       
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     strcpy ( msg_str, "Justification on:" );
     hWndjuson = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_JUSONBUTTON, 
                         hInst, NULL );
     SendMessage( hWndjuson, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( juson )
        SendMessage( hWndjuson, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndjuson, BM_SETCHECK, BST_UNCHECKED, 0);  
//---------------------------------Tibetan and western numbers for first page
    c_scrn_x = 20;
    c_scrn_y = c_scrn_y + 40;                        
    sprintf ( msg_str, "Tibetan number for first page: " );
        ExtTextOut ( conf_hdc, c_scrn_x, c_scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );    
    c_scrn_x = c_scrn_x + strlen ( msg_str ) * fxchrwdth;
    hWndstrpnum = CreateWindowEx(WS_EX_CLIENTEDGE,
		"EDIT",
		"",
        	WS_CHILD|WS_VISIBLE,
		c_scrn_x,   
		c_scrn_y, 
		3 * fxchrwdth + 2,
		fxchrhgt + 2, 
		conf_hwnd,
		NULL, 
		GetModuleHandle(NULL),
		NULL);     
    sprintf ( num_str, "%d", strpnum );
    SendMessage( hWndstrpnum, WM_SETTEXT, (WPARAM) NULL, (LPARAM) num_str ); 
    c_scrn_x = c_scrn_x + 3 * fxchrwdth + 5;
    sprintf ( msg_str, "Western number for first page: " );
        ExtTextOut ( conf_hdc, c_scrn_x, c_scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );    
    c_scrn_x = c_scrn_x + strlen ( msg_str ) * fxchrwdth;
    hWndwpnum = CreateWindowEx( WS_EX_CLIENTEDGE,
		"EDIT",
		"",
        	WS_CHILD|WS_VISIBLE,
		c_scrn_x, 
		c_scrn_y,   
		3 * fxchrwdth + 2, 
		fxchrhgt + 2,      
		conf_hwnd,
		NULL, 
		GetModuleHandle(NULL),
		NULL);     
    sprintf ( num_str, "%d", curwpnum );		
    SendMessage( hWndwpnum, WM_SETTEXT, (WPARAM) NULL, (LPARAM) num_str );     
//--------------------------------------- Tibetan page, start and stop printing
    c_scrn_x = 20;
    c_scrn_y = c_scrn_y + 40;                        
    sprintf ( msg_str, "Printing to start at Tibetan page: " );
        ExtTextOut ( conf_hdc, c_scrn_x, c_scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );    
    c_scrn_x = c_scrn_x + strlen ( msg_str ) * fxchrwdth;
    hWndTstart = CreateWindowEx(WS_EX_CLIENTEDGE,
		"EDIT",
		"",
        	WS_CHILD|WS_VISIBLE,
		c_scrn_x,  c_scrn_y, 
		3 * fxchrwdth + 2, 
		fxchrhgt + 2,      
		conf_hwnd,
		NULL, // (HMENU)IDC_MAIN_EDIT,
		GetModuleHandle(NULL),
		NULL);     
    sprintf ( num_str, "%d", startpage );
    SendMessage( hWndTstart, WM_SETTEXT, (WPARAM) NULL, (LPARAM) num_str ); 
    c_scrn_x = c_scrn_x + 3 * fxchrwdth + 5;
    sprintf ( msg_str, " and finish at page: " );
        ExtTextOut ( conf_hdc, c_scrn_x, c_scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );    
    c_scrn_x = c_scrn_x + strlen ( msg_str ) * fxchrwdth;
    hWndTstop = CreateWindowEx( WS_EX_CLIENTEDGE,
		"EDIT",
		"",
        	WS_CHILD|WS_VISIBLE,
		c_scrn_x, c_scrn_y,
		3 * fxchrwdth + 2, 
		fxchrhgt + 2,      
		conf_hwnd,
		NULL, 
		GetModuleHandle(NULL),
		NULL); 
    sprintf ( num_str, "%d", endpage );		    
    SendMessage( hWndTstop, WM_SETTEXT, (WPARAM) NULL, (LPARAM) num_str );     
//-------------------------------- PORTRAIT, TRIM 
     strcpy ( msg_str, "Print trim marks:" );
     c_scrn_x = 20;
     c_scrn_y = c_scrn_y + 40; 
     hWndTrim = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_TRIM_MARKS, 
                         hInst, NULL );
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     SendMessage( hWndTrim, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( trim_mark )
        SendMessage( hWndTrim, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndTrim, BM_SETCHECK, BST_UNCHECKED, 0); 
     strcpy ( msg_str, "Portrait mode (not A3 or 11x17):" );
     c_scrn_x = c_scrn_x + 2 * fxchrwdth + 5;
     hWndPort = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_PORTRAIT, 
                         hInst, NULL );
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     SendMessage( hWndPort, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( portprn )
        SendMessage( hWndPort, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndPort, BM_SETCHECK, BST_UNCHECKED, 0);        
//-------------------------------- NO BYON
     strcpy ( msg_str, "Open printing, no final \"byon\":" );
     c_scrn_x = 20;
     c_scrn_y = c_scrn_y + 40; 
     hWndNoByon = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_NO_BYON, 
                         hInst, NULL );
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     SendMessage( hWndNoByon, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( nobyon )
        SendMessage( hWndNoByon, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndNoByon, BM_SETCHECK, BST_UNCHECKED, 0);            
//------------------------- REVERSE PRINTING
     strcpy ( msg_str, "Reverse printing, turning each page individually:" );
     c_scrn_x = 20;
     c_scrn_y = c_scrn_y + 40; 
     hWndRevprn = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20,  
                         30, conf_hwnd, 
                         (HMENU) ID_REVPRN, 
                         hInst, NULL );
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     SendMessage( hWndRevprn, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( revprn )
        SendMessage( hWndRevprn, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndRevprn, BM_SETCHECK, BST_UNCHECKED, 0);                
//---------------------------------- STOP PRINTING AND DEBUG 
     strcpy ( msg_str, "Turn off printing (for testing):" );
     c_scrn_x = 20;
     c_scrn_y = c_scrn_y + 40; 
     hWndstopprn = CreateWindowEx( 0, "BUTTON", msg_str, 
                         WS_CHILD | WS_VISIBLE | BS_CHECKBOX | BS_LEFTTEXT, 
                         c_scrn_x, c_scrn_y, 
                         fxchrwdth * strlen (msg_str) + 20, 
                         30, conf_hwnd, 
                         (HMENU) ID_STOP_PRINT_BUTTON, 
                         hInst, NULL );
     c_scrn_x = c_scrn_x + fxchrwdth * strlen (msg_str) + 40;                          
     SendMessage( hWndstopprn, WM_SETFONT, (WPARAM) conf_font, (LPARAM) TRUE);
     if ( stoprint )
        SendMessage( hWndstopprn, BM_SETCHECK, BST_CHECKED, 0); 
     else
       SendMessage ( hWndstopprn, BM_SETCHECK, BST_UNCHECKED, 0); 
    c_scrn_x = c_scrn_x + 2 * fxchrwdth + 5;
    sprintf ( msg_str, "Debug flag value: " );
    ExtTextOut ( conf_hdc, c_scrn_x, c_scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );    
    c_scrn_x = c_scrn_x + strlen ( msg_str ) * fxchrwdth;
    hWnddebug = CreateWindowEx( WS_EX_CLIENTEDGE,
		"EDIT",
		"",
        	WS_CHILD|WS_VISIBLE,
		c_scrn_x, c_scrn_y,
		3 * fxchrwdth + 2, // width
		fxchrhgt + 2,      // height
		conf_hwnd,
		NULL, 
		GetModuleHandle(NULL),
		NULL);     
    sprintf ( num_str, "%d", d_bug );
    SendMessage( hWnddebug, WM_SETTEXT, (WPARAM) NULL, (LPARAM) num_str );  
//-------------------------NUMBER OF COPIES:
    c_scrn_x = 20;
    c_scrn_y = c_scrn_y + 40;                        
    sprintf ( msg_str, "Number of copies: " );
        ExtTextOut ( conf_hdc, c_scrn_x, c_scrn_y, ETO_OPAQUE, NULL,
                        msg_str, strlen (msg_str), NULL );    
    c_scrn_x = c_scrn_x + strlen ( msg_str ) * fxchrwdth;
    hWndcopies = CreateWindowEx(WS_EX_CLIENTEDGE,
		"EDIT",
		"",
        	WS_CHILD|WS_VISIBLE,
		c_scrn_x, c_scrn_y,
		3 * fxchrwdth + 2, // width
		fxchrhgt + 2,      // height
		conf_hwnd,
		NULL, 
		GetModuleHandle(NULL),
		NULL);     
    sprintf ( num_str, "%d", num_copies );		
    SendMessage( hWndcopies, WM_SETTEXT, (WPARAM) NULL, (LPARAM) num_str ); 
//------------------------------- MAIN PRINT BUTTON, closes window
    c_scrn_y = c_scrn_y + 80;
    if ( prev_on )
      strcpy ( msg_str, "Preview" );
    else  
      strcpy ( msg_str, "Print" );          
    hWndButton=CreateWindowEx( 0, 
	"BUTTON",
	msg_str,
	WS_TABSTOP| WS_VISIBLE| WS_CHILD | BS_DEFPUSHBUTTON,
	400, c_scrn_y,
	100,
	40, 
	conf_hwnd,
	(HMENU) ID_PRINT_BUTTON,
	GetModuleHandle(NULL),
	NULL);
    EndPaint (conf_hwnd, &ps);  
    if ( paper_size == A4_PAPER )
      set_a4_pecha ();
    else if ( paper_size == A3_PAPER )
      set_a3_pecha ();
    else if ( paper_size == LEGAL_PAPER )
      set_legal_pecha ();      
    else if ( paper_size == LETTER_PAPER )
      set_letter_pecha ();            
    else if ( paper_size == Tabloid_PAPER )
      set_Tabloid_pecha ();            
    return (1);   
  } // END - PrintConfig 

void set_a4_pecha ( void )
  {
    if ( !doall )
      {
        doall = 1;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_CHECKED, 0);     
      }
    if ( portprn )
      {  
        portprn = 0;
        SendMessage ( hWndPort, BM_SETCHECK, BST_UNCHECKED, 0);         
      }
    if ( !juson )
      {  
        juson = 1;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( nobyon )
      {
        nobyon = 0;   
        SendMessage ( hWndNoByon, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( trim_mark )
      {  
        trim_mark = 0;
        SendMessage ( hWndTrim, BM_SETCHECK, BST_UNCHECKED, 0);   
      }
    if ( Tabloid_pecha )
      {
        Tabloid_pecha = 0;
        SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }        
    else if ( a3_pecha )
      {
        a3_pecha = 0;
        SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( a4_port )
      {
        a4_port = 0;
        SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);              
      }             
    else if ( legal_pecha )
      {
        legal_pecha = 0;
        SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }       
    else if ( legal_port )
      {
        legal_port = 0;
        SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                                          
   else if ( letter_pecha )
      {
        letter_pecha = 0;
        SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }           
   else if ( letter_port )
      {
        letter_port = 0;
        SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                       
  } // END - set_a4_pecha ()

void set_a3_pecha ( void )
  {
    if ( !doall )
      {
        doall = 1;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_CHECKED, 0);     
      }
    if ( portprn )
      {  
        portprn = 0;
        SendMessage ( hWndPort, BM_SETCHECK, BST_UNCHECKED, 0);         
      }
    if ( !juson )
      {  
        juson = 1;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( nobyon )
      {
        nobyon = 0;   
        SendMessage ( hWndNoByon, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( !trim_mark )
      {  
        trim_mark = 1;
        SendMessage ( hWndTrim, BM_SETCHECK, BST_CHECKED, 0);   
      }      
    if ( Tabloid_pecha )
      {
        Tabloid_pecha = 0;
        SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }     
    else if ( a4_pecha )
      {
        a4_pecha = 0;
        SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( a4_port )
      {
        a4_port = 0;
        SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);              
      }         
    else if ( legal_pecha )
      {
        legal_pecha = 0;
        SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }       
    else if ( legal_port )
      {
        legal_port = 0;
        SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                                          
   else if ( letter_pecha )
      {
        letter_pecha = 0;
        SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }           
   else if ( letter_port )
      {
        letter_port = 0;
        SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                       
  } // END - set_a3_pecha ()
  
void set_Tabloid_pecha ( void )  
  {
    if ( !doall )
      {
        doall = 1;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_CHECKED, 0);     
      }
    if ( portprn )
      {  
        portprn = 0;
        SendMessage ( hWndPort, BM_SETCHECK, BST_UNCHECKED, 0);         
      }
    if ( !juson )
      {  
        juson = 1;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( nobyon )
      {
        nobyon = 0;   
        SendMessage ( hWndNoByon, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( !trim_mark )
      {  
        trim_mark = 1;
        SendMessage ( hWndTrim, BM_SETCHECK, BST_CHECKED, 0);   
      }
    if ( a3_pecha )
      {
        a3_pecha = 0;
        SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }     
    else if ( a4_pecha )
      {
        a4_pecha = 0;
        SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( a4_port )
      {
        a4_port = 0;
        SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);              
      }         
    else if ( legal_pecha )
      {
        legal_pecha = 0;
        SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }       
    else if ( legal_port )
      {
        legal_port = 0;
        SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                                          
   else if ( letter_pecha )
      {
        letter_pecha = 0;
        SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }           
   else if ( letter_port )
      {
        letter_port = 0;
        SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                         
  } // set_Tabloid_pecha ()
  
void set_a4_port ( void )
  {
    if ( doall )
      {
        doall = 0;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( !portprn )
      {  
        portprn = 1;
        SendMessage ( hWndPort, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( juson )
      {  
        juson = 0;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_UNCHECKED, 0);         
      }     
    if ( a3_pecha )
      {
        a3_pecha = 0;
        SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( Tabloid_pecha )
      {
        Tabloid_pecha = 0;
        SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }           
    else if ( a4_pecha )
      {
        a4_pecha = 0;
        SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }  
    else if ( legal_pecha )
      {
        legal_pecha = 0;
        SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }       
    else if ( legal_port )
      {
        legal_port = 0;
        SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                                    
   else if ( letter_pecha )
      {
        letter_pecha = 0;
        SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }           
   else if ( letter_port )
      {
        letter_port = 0;
        SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                       
  }  

void set_legal_pecha ( void )
  {
    if ( !doall )
      {
        doall = 1;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_CHECKED, 0);     
      }
    if ( portprn )
      {  
        portprn = 0;
        SendMessage ( hWndPort, BM_SETCHECK, BST_UNCHECKED, 0);         
      }
    if ( !juson )
      {  
        juson = 1;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( nobyon )
      {
        nobyon = 0;   
        SendMessage ( hWndNoByon, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( trim_mark )
      {  
        trim_mark = 0;
        SendMessage ( hWndTrim, BM_SETCHECK, BST_UNCHECKED, 0);   
      }
    if ( a3_pecha )
      {
        a3_pecha = 0;
        SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0);        
      }        
    else if ( Tabloid_pecha )
      {
        Tabloid_pecha = 0;
        SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                 
    else if ( a4_pecha )
      {
        a4_pecha = 0;
        SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( a4_port )
      {
        a4_port = 0;
        SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);              
      }       
    else if ( legal_port )
      {
        legal_port = 0;
        SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }   
   else if ( letter_pecha )
      {
        letter_pecha = 0;
        SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }           
   else if ( letter_port )
      {
        letter_port = 0;
        SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                                   
  } // END - set_legal_pecha ()
  
void set_legal_port ( void )
  {
    if ( doall )
      {
        doall = 0;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( !portprn )
      {  
        portprn = 1;
        SendMessage ( hWndPort, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( juson )
      {  
        juson = 0;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_UNCHECKED, 0);         
      }          
    if ( a3_pecha )
      {
        a3_pecha = 0;
        SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0);        
      }        
    else if ( Tabloid_pecha )
      {
        Tabloid_pecha = 0;
        SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                 
    else if ( a4_pecha )
      {
        a4_pecha = 0;
        SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( a4_port )
      {
        a4_port = 0;
        SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);              
      }         
    else if ( legal_pecha )
      {
        legal_pecha = 0;
        SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }    
   else if ( letter_pecha )
      {
        letter_pecha = 0;
        SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }           
   else if ( letter_port )
      {
        letter_port = 0;
        SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                 
  } // END - set_legal_port ()

void set_letter_pecha ( void )
  {
    if ( !doall )
      {
        doall = 1;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_CHECKED, 0);     
      }
    if ( portprn )
      {  
        portprn = 0;
        SendMessage ( hWndPort, BM_SETCHECK, BST_UNCHECKED, 0);         
      }
    if ( !juson )
      {  
        juson = 1;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( nobyon )
      {
        nobyon = 0;   
        SendMessage ( hWndNoByon, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( trim_mark )
      {  
        trim_mark = 0;
        SendMessage ( hWndTrim, BM_SETCHECK, BST_UNCHECKED, 0);   
      }
    if ( a3_pecha )
      {
        a3_pecha = 0;
        SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0);        
      }        
    else if ( Tabloid_pecha )
      {
        Tabloid_pecha = 0;
        SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                 
    else if ( a4_pecha )
      {
        a4_pecha = 0;
        SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( a4_port )
      {
        a4_port = 0;
        SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);              
      }       
    else if ( legal_pecha )
      {
        legal_pecha = 0;
        SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( legal_port )
      {
        legal_port = 0;
        SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                     
   else if ( letter_port )
      {
        letter_port = 0;
        SendMessage ( hWndLetterport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }        
  } // END - set_letter_pecha ()
  
void set_letter_port ( void )
  {
    if ( doall )
      {
        doall = 0;
        SendMessage ( hWndDoallButton, BM_SETCHECK, BST_UNCHECKED, 0);     
      }
    if ( !portprn )
      {  
        portprn = 1;
        SendMessage ( hWndPort, BM_SETCHECK, BST_CHECKED, 0);         
      }
    if ( juson )
      {  
        juson = 0;    
        SendMessage ( hWndjuson, BM_SETCHECK, BST_UNCHECKED, 0);         
      }  
    if ( a3_pecha )
      {
        a3_pecha = 0;
        SendMessage ( hWndA3pecha, BM_SETCHECK, BST_UNCHECKED, 0);        
      }        
    else if ( Tabloid_pecha )
      {
        Tabloid_pecha = 0;
        SendMessage ( hWndTabloidpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }                 
    else if ( a4_pecha )
      {
        a4_pecha = 0;
        SendMessage ( hWndA4pecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      } 
    else if ( a4_port )
      {
        a4_port = 0;
        SendMessage ( hWndA4port, BM_SETCHECK, BST_UNCHECKED, 0);              
      }         
    else if ( legal_pecha )
      {
        legal_pecha = 0;
        SendMessage ( hWndLegalpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }       
    else if ( legal_port )
      {
        legal_port = 0;
        SendMessage ( hWndLegalport, BM_SETCHECK, BST_UNCHECKED, 0);              
      }             
    else if ( letter_pecha )
      {
        letter_pecha = 0;
        SendMessage ( hWndLetterpecha, BM_SETCHECK, BST_UNCHECKED, 0);              
      }             
  } // END - set_letter_port ()
  