/*********************************************************************************
Licence for Tibw - Tibetan printing software

Copyright (c) 1985-2012 Edward Henning

Permission is hereby granted, free of charge, to any person  obtaining a copy of
this software and associated documentation files (the "Software"), to deal in the
Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
************************************************************************************/
#define INCLUDE_COMMDLG_H       1
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <dos.h>
#include <string.h>
#include <malloc.h>
#include <strsafe.h>
#include "tibw.h"
#include "tform.h"
#include "tibform.ext"

void set_print_font ( HDC );
void DrawBitmapXY ( HDC, HBITMAP, int, int );

extern HANDLE Tibw_inst;  

extern char fname[10];
//extern char tf_buf[58000];

extern char *tf37buf;
extern int tf37_xpt, tf37_ypt, tf37_fc, tf37_lc;
extern HBITMAP tf37_hbm[94];

extern char *tf38buf;
extern int tf38_xpt, tf38_ypt, tf38_fc, tf38_lc;
extern HBITMAP tf38_hbm[60];

extern char *tf39buf;
extern int tf39_xpt, tf39_ypt, tf39_fc, tf39_lc;
extern HBITMAP tf39_hbm[94];

extern char *tf40buf;
extern int tf40_xpt, tf40_ypt, tf40_fc, tf40_lc;
extern HBITMAP tf40_hbm[60];

extern char *tf41buf;
extern int tf41_xpt, tf41_ypt, tf41_fc, tf41_lc;
extern HBITMAP tf41_hbm[94];

extern char *tf42buf;
extern int tf42_xpt, tf42_ypt, tf42_fc, tf42_lc;
extern HBITMAP tf42_hbm[60];

extern HDC pag_DC; 
extern int cur_mem_pag;
extern int max_mem_pag;  // Maybe not needed here
extern int tgt_cx, tgt_cy;

extern int dis_cenx, dis_ceny;                               

LOGFONT plf;
HGDIOBJ hfnt, hfntPrev; 
HRESULT hr; 
int pcch;
HANDLE fl_hndl;

extern char EditTitle[];
extern PRINTDLG prstr;
extern HFONT CFont;

void set_print_font ( HDC hdc )
  {
    hr = StringCchCopy ( plf.lfFaceName, 12, TEXT("Courier New"));
    if ( FAILED (hr) )
      {
        sprintf ( d_bugstr, "Courier New failed...!" );
        Msg (d_bugstr);
        wrt_debug ();      
        hr = StringCchCopy ( plf.lfFaceName, 12, TEXT("Courier"));      
        if ( FAILED (hr) )
          {
            sprintf ( d_bugstr, "Major problem: could not access font for pages numbers!" );
            Msg (d_bugstr);
            wrt_debug ();
          }  
      }    
    plf.lfWeight = FW_NORMAL;  
// We want the height in logical units to be 60, so:
    if ( !prev_on )   
      plf.lfHeight = (int)((float) 60 * pr_scalex ); 
    else
      plf.lfHeight = 55; // 60 seems too large in preview mode
    plf.lfEscapement = 0; 
    hfnt = CreateFontIndirect( &plf ); 
    if ( hfnt == NULL )
      Msg ( "Font creation failed!" );
    hfntPrev = SelectObject( hdc, hfnt );
  } // END - set_print_font

/*** Portrait routine ***/
void port_rout ( void ) // This is for portrait A4 printing. No frames.
  {
    int str_ptr;
    int n, litpnum; 
    open_portrait_tfonts ();
    endfilfnd = 0;
    gen_buf[0] = '\0';
    pasteof = 0;   /** Not yet reached end of file **/
    prn_erf = 0;
    str_ptr = 0;
    init_prn ();
    portprn = 1;  // Added - needed ???
    pcurly = txtrct.top + bas2top; 

    if ( d_bug )
      {
        sprintf ( d_bugstr, "Printing file: %s.", fil_nam );
        wrt_debug ();
        sprintf ( d_bugstr, "pcurly = %d, txtrct.top = %d, bas2top = %d.", 
                  pcurly, txtrct.top, bas2top );
        wrt_debug ();
      }
    sml_fnt = 0;  // Set flags for normal font 
    getglbspc (); // Set up global spacing tables 
    flbftptr = 0; // Pointer into top of file transfer buffer 
    flbfcptr = 0; // Pointer to current place in transfer buffer
    gnbufptr = 0; // Start byte to fill from, 1st available
    gcbufptr = 0; // Pointer to current byte in gen_buf 
    luflptr = 0L; // Pointer into file 
    qu_ptr = 0;   // Pointer into word queue 
    file_read = 0;
    fl_hndl = CreateFile ( fil_nam, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( fl_hndl == INVALID_HANDLE_VALUE )
      Msg ( "Open file failed." );         
    fil_gnbuf ();
    CloseHandle( fl_hndl );     
    obptr = 0;
    litpnum = 1;
    if ( !stoprint && !prev_on )  
      Escape( prstr.hDC, STARTDOC, 4, NULL, NULL );    
    else if ( prev_on )
      start_mempag ();  
    if ( curwpnum ) // Print first western page number.
      {
        if ( d_bug )
          {
            sprintf ( d_bugstr, "Curwpnum: %d, Right: %d", curwpnum, porrct.right );            
            wrt_debug ( );      
          }
// First, print western page number, if needed, for first page:
        setcurpos ( porrct.right, porrct.top + 50 );                 
        if ( prev_on )
          set_print_font ( pag_DC ); // Sets for Courier New. 
        else
          set_print_font ( prstr.hDC ); // Sets for Courier New. 
        sprintf ( num_str, "%d", curwpnum );
        if ( !prev_on )
          TextOut( prstr.hDC, dev_cur_x, dev_cur_y, num_str, strlen (num_str) );  
        else
          TextOut( pag_DC, dev_cur_x, dev_cur_y, num_str, strlen (num_str) );            
      } // End of first western page number.
    if ( d_bug )
      {
        sprintf ( d_bugstr, "Text = %s", fil_nam );
        wrt_debug ( );
      }
    if ( d_bug )
      {
        sprintf ( d_bugstr, "Starting Page: %d\n", litpnum );
        wrt_debug ( );    
      }  
    do // successive pages
      {
        if ( pcurly + linspc + 20 > txtrct.bottom ) // Check for new page
          {
            if ( !stoprint && !prev_on )              
              Escape ( prstr.hDC, NEWFRAME, 0, NULL, NULL );  
// Will need code here for starting a new preview page
            pcurly = txtrct.top + bas2top;
            obptr = 0;
            prvobptr = 0;
            linnum = 1;
            if ( curwpnum )   // Only increase it if it has been set.
              curwpnum += 1;
            litpnum += 1;
            if ( prev_on )
              {
                save_mempag ( cur_mem_pag );              
                ++cur_mem_pag;
                tgt_cx = dis_cenx;
                tgt_cy = dis_ceny;                 
                start_mempag ();
              }
            if ( curwpnum )
              {
                if ( prev_on )
                  set_print_font ( pag_DC ); // Sets for Courier New. 
                else
                  set_print_font ( prstr.hDC ); // Sets for Courier New.               
                sprintf ( num_str, "%d", curwpnum );
                setcurpos ( porrct.right, porrct.top + 50 );                                
                TextOut( prstr.hDC, dev_cur_x, dev_cur_y, num_str, strlen (num_str) );  
                if ( !prev_on )
                  TextOut( prstr.hDC, dev_cur_x, dev_cur_y, num_str, strlen (num_str) );  
                else
                  TextOut( pag_DC, dev_cur_x, dev_cur_y, num_str, strlen (num_str) );                 
              }
          }
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Line Count = %d", linnum );
            wrt_debug ( );
            sprintf ( d_bugstr, "Call to str_an." );
            wrt_debug ( );
          }
        str_an ();
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Finished str_an." );
            wrt_debug ( );        
          }
        obblck = 0; // Stop adding more to obscure queue 
        nxtobj = 0; // Next that will be encountered along the line
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Call to anlslin" );
            wrt_debug ( );
          }        
        n = anlslin ( 0 ); // Returns number of words to print
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "Finished anlslin, words: %d", n );
            wrt_debug ( );
          }                
        setcurpos ( txtrct.left, pcurly );
        if ( d_bug )
          {
            sprintf ( d_bugstr, "Line: %d, Current font = %d", linnum, cur_fnt );
            wrt_debug ( );        
          }
        obblck = 1; // Allow adding more to obscure queue 
        nxtobj = 0;
        if ( d_bug > 0 )
          {
            sprintf ( d_bugstr, "Call to prn_nlin" );
            wrt_debug ( );
          }        
        prn_nlin ( 0, n );
        if ( d_bug > 0 )
          {
            sprintf ( d_bugstr, "Finished prn_nlin" );
            wrt_debug ( );
          }                
        ++linnum;
        pcurly += linspc; // Effective line feed  
        sort_obs ( pcurly );
        shuf_que ( n + 1 );
      } while ( !endfilfnd );
    if ( !stoprint && !prev_on )        
      Escape ( prstr.hDC, ENDDOC, 0, NULL, NULL );
    if ( prev_on )
      save_mempag ( cur_mem_pag );      
    clean_portrait (); 
  } // END - port_rout ()
  
VOID DrawBitmapXY (HDC hdc, HBITMAP hbm, int x, int y)
  {
    BOOL f;
    HDC hdcBits;
    BITMAP bm;
    hdcBits = CreateCompatibleDC(hdc);
    GetObject (hbm, sizeof(BITMAP), &bm);
    SelectObject(hdcBits,hbm);
    f = BitBlt(hdc,x,y,bm.bmWidth, bm.bmHeight, hdcBits, 0, 0, SRCAND /*SRCINVERT*/ );
    DeleteDC(hdcBits);
  }
  
void pn_chr ( int x )
  {
    if ( noprint == 0 && stoprint == 0 )
      {
        if ( lp_chr ( x ) ) // Only returns zero now!
          stoprint = 1;
      }
  }
  
// Routine to print character
int lp_chr ( unsigned char c ) // Always returns zero - could change this
  {
    int x, y;
    if ( c < 33 )  // Remove this later, or keep as error check.
      {
        sprintf ( d_bugstr, "Spurious character: %d.", (int) c );
        wrt_debug ();
        return (0);
      }
    if ( d_bug == 4 )
      {  
        sprintf ( d_bugstr, "Printing, \"%s\", font: %d, chr: %d, Logic x: %d, y: %d, Device X: %d, Y: %d", 
                             wrd_tst, cur_fnt, (int) c, logicx, logicy, dev_cur_x, dev_cur_y );
        wrt_debug ( );          
      }
    if ( prev_on )
      {
        wrt_chr ( c );
        return (0);
      }        
    if ( portprn )
      {
        if ( smlfnt )
          {
            if ( scaling_needed )    
              y = dev_cur_y + (int) ( 7.0 * pr_scaley );         
            else  
              y = dev_cur_y + 7;
          }
        else
          y = dev_cur_y;
        if ( cur_fnt == 37 )
          DrawBitmapXY ( prstr.hDC, tf37_hbm[(int) c - tf37_fc], dev_cur_x, y );        
        else if ( cur_fnt == 38 )
          DrawBitmapXY ( prstr.hDC, tf38_hbm[(int) c - tf38_fc], dev_cur_x, y );         
        else if ( cur_fnt == 39 )
          DrawBitmapXY ( prstr.hDC, tf39_hbm[(int) c - tf39_fc], dev_cur_x, y );         
        else if ( cur_fnt == 40 )
          DrawBitmapXY ( prstr.hDC, tf40_hbm[(int) c - tf40_fc], dev_cur_x, y );                       
      }
    else // if landscape
      {
        if ( smlfnt )  
          {
            if ( scaling_needed )    
              x = dev_cur_x + (int) ( 7.0 * pr_scalex );         
            else  
              x = dev_cur_x + 7;
          }
        else
          x = dev_cur_x;
        y = dev_cur_y;
        if ( cur_fnt == 37 )
          DrawBitmapXY ( prstr.hDC, tf37_hbm[(int) c - tf37_fc], x, y - tf37_ypt );
        else if ( cur_fnt == 38 )
          DrawBitmapXY ( prstr.hDC, tf38_hbm[(int) c - tf38_fc], x, y - tf38_ypt );         
        else if ( cur_fnt == 39 )
          DrawBitmapXY ( prstr.hDC, tf39_hbm[(int) c - tf39_fc], x, y - tf39_ypt );         
        else if ( cur_fnt == 40 )
          DrawBitmapXY ( prstr.hDC, tf40_hbm[(int) c - tf40_fc], x, y - tf40_ypt );                       
        else if ( cur_fnt == 41 ) 
          DrawBitmapXY ( prstr.hDC, tf41_hbm[(int) c - tf41_fc], x, y - tf41_ypt );         
        else if ( cur_fnt == 42 )
          DrawBitmapXY ( prstr.hDC, tf42_hbm[(int) c - tf42_fc], x, y - tf42_ypt );                                 
      }
    return (0);
  }  // END lp_chr ()

// General page routine for landscape printing
void land_rout ( void )   // A4 and A3 
  {
    int str_ptr;
    int lincnt;
    int pg;
    int endfnd;
    int jussav = 0;
    int bas_pag = 0;
    endfnd = 0;
    sidprn = 0; // Make sure sideways printing is off
    portprn = 0;
    endfilfnd = 0;
    prn_erf = 0;
    gen_buf[0] = '\0';
    str_ptr = 0;
    init_prn ();
    sml_fnt = 0;  // Set flags for normal font 
    getglbspc (); // Set up global spacing tables 
    flbftptr = 0; // Pointer into top of file transfer buffer 
    flbfcptr = 0; // Pointer to current place in transfer buffer
    gnbufptr = 0; // Start byte to fill from, 1st available
    gcbufptr = 0; // Pointer to current byte in gen_buf
    luflptr = 0L; // Pointer into file
    qu_ptr = 0;   // Pointer into word queue
    pasteof = 0;  // Not yet reached end of file - maybe redundant now
    file_read = 0;
    fl_hndl = CreateFile ( fil_nam, GENERIC_READ, FILE_SHARE_READ, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );     
    if ( fl_hndl == INVALID_HANDLE_VALUE )
      Msg ( "Open file failed." );         
    fil_gnbuf ();
    CloseHandle( fl_hndl );     
    lincnt = 0;
    if ( d_bug )
      {
        sprintf ( d_bugstr, "Landscape mode, Text = %s", fil_nam );
        wrt_debug ( );
      }
    fstistit = 0;
    titpon = 0;
    titpfnd = 0;
    titpend = 0; // Clear title page flags 
    setpmstop (); // This is only for landscape mode - in tf1.c: sets box, txtrct & sidebr rectangles
    pg = 0;
    pagenum = strpnum;
    if ( pagenum < startpage ) // NEW - 3/8/1998
      stoprint = 1; 
    open_landscape_tfonts ();
    if ( revprn )
      Msg ( "Place a new page in printer, and press Enter when ready." );         
    if ( !stoprint && !prev_on )       
      Escape( prstr.hDC, STARTDOC, 4, NULL, NULL );
// Prepare font for printing page numbers:
    hr = StringCchCopy ( plf.lfFaceName, 12, TEXT("Courier New"));
    if ( FAILED (hr) )
      {
        hr = StringCchCopy ( plf.lfFaceName, 12, TEXT("Courier"));      
        if ( FAILED (hr) )
          {
            sprintf ( d_bugstr, "Major problem: could not access font for pages numbers!" );
            Msg (d_bugstr);
            wrt_debug ();
          }  
      }    
    plf.lfWeight = FW_NORMAL; 
// We want the height in logical units to be 60, so:
    if ( !prev_on )   
      plf.lfHeight = (int)((float) 60 * pr_scalex ); // XXXXXXXXXX!!!
    else
      plf.lfHeight = 60; 
    plf.lfEscapement = 900; 
    hfnt = CreateFontIndirect( &plf ); 
    if ( hfnt == NULL )
      Msg ( "Font creation failed!" );
    if ( !prev_on )     
      hfntPrev = SelectObject( prstr.hDC, hfnt );
// This completes making the font and rotating it
    do // MAIN PRINTING LOOP
      {
        if ( pagenum == startpage )
          stoprint = 0;
        else if ( pagenum > endpage )
          stoprint = 1;
        if ( skiptop || skipbot )
          {
            skiptop = 0; skipbot = 0;
          }
        if ( d_bug == 4 )
          {
            sprintf ( d_bugstr, "MAINLOOP:" );
            wrt_debug ();
            sprintf ( d_bugstr, "FSTISTIT = %d", fstistit );
            wrt_debug ();
            sprintf ( d_bugstr, "txtrct.LEFT = %d, txtrct.RIGHT = %d",
                                       txtrct.left, txtrct.right );
            wrt_debug ();
          }
        str_an ();  // This fills up the queue 
        an_pags (); // Analyses all four pages, TF1.C
        prnpg0 = 1; prnpg1 = 1; prnpg2 = 1; prnpg3 = 1;

        if ( lastpg < 4 )
          {
            endfnd = 1;
            if ( lastpg < 3 )
              prnpg3 = 0;
            if ( lastpg < 2 )
              prnpg2 = 0;
            if ( lastpg < 1 )
              prnpg1 = 0;
          }
// PRINT FIRST TWO PAGES 
        setpmstop (); // In tf1.c
        if ( fstistit )
          {
            titpon = 1;
            setpmstit (); // In tf1.c
            jussav = juson;
            juson = 0;
          }
        if ( prnpg2 == 0 )
          skipbot = 1;
        if ( prev_on )
          {
            cur_mem_pag = bas_pag;
            start_mempag (); // This value no longer used
            hfntPrev = SelectObject( pag_DC, hfnt );
          }
        prn_pag ( 0 ); // in tf1.c; // This prints the text, frames later
        prn_boxt ( 0 ); 
        if ( curwpnum )
          {
            if ( d_bug == 4 )
              {
                sprintf ( d_bugstr, "Call 11 to setcurpos: X: %d, Y: %d.", 
                          lndrct.right + 20, midtop - boxhgt / 2 + 20 );                          
                wrt_debug ();
              } 
            setcurpos ( lndrct.right + 20, midtop - boxhgt / 2 + 20);            
            sprintf ( num_str, "%d", pd[0].wstpnum );
            pcch = strlen (num_str);
            if ( prnpg0 ) 
              {
                if ( prev_on )
                  TextOut ( pag_DC, dev_cur_x, dev_cur_y, num_str, pcch );         
                else
                  TextOut ( prstr.hDC, dev_cur_x, dev_cur_y, num_str, pcch );         
              }
          } // END - if ( curwpnum )     
        sid_barst ( pagenum, 1 ); // In tf1.c                
        if ( prev_on )
          {
            save_mempag ( cur_mem_pag );            
            ++cur_mem_pag;
          }
        if ( fstistit )
          {
            juson = jussav;
            titpend = 0;
            titpfnd = 0;
            titpageon = 0;
          }
        setpmsbot (); // In tf1.c, sets box, txtrct & sidbr rectangles
        if ( prev_on )
          {
            cur_mem_pag = bas_pag + 2;
            start_mempag ();          
            hfntPrev = SelectObject( pag_DC, hfnt );    // to here                        
          }
        if ( prnpg2 )
          {
            prn_pag ( 2 ); // This prints the text
            prn_boxb ( 0 ); // This prints the frames - 0 for topside
            if ( curwpnum )
              {
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "Call 12 to setcurpos: X: %d, Y: %d.", 
                              lndrct.right + 20, midbot - boxhgt / 2 + 20 );                          
                    wrt_debug ();
                  } 
                setcurpos ( lndrct.right + 20, midbot - boxhgt / 2 + 20);            
                sprintf ( num_str, "%d", pd[2].wstpnum );
                pcch = strlen (num_str);
                if ( prev_on )
                  TextOut ( pag_DC, dev_cur_x, dev_cur_y, num_str, pcch );         
                else
                  TextOut ( prstr.hDC, dev_cur_x, dev_cur_y, num_str, pcch );    
              } // END - if ( curwpnum )      
            sid_barsb ( pagenum + 1, 1 ); // In tf1.c      
          } // END - if ( prnpg2 )
        if ( prev_on )
          {
            save_mempag ( cur_mem_pag );            
            ++cur_mem_pag;
          }          
        if ( d_bug )
          {
            sprintf ( d_bugstr, "Pages: %d and %d", pagenum, pagenum + 1 );
            wrt_debug ( );
          }
        skiptop = 0; skipbot = 0;
        if ( titpon )
          fstistit = 0;
// PRINT NEXT TWO PAGES
        if ( !stoprint )  
          {
            if ( revprn )
              {
                Escape ( prstr.hDC, ENDDOC, 0, NULL, NULL );                  
                Msg ( "When the page has printed, replace the paper, and press Enter..." );          
                Escape( prstr.hDC, STARTDOC, 4, NULL, NULL );              
              }
            else
              Escape ( prstr.hDC, NEWFRAME, 0, NULL, NULL );                       
          }
        if ( prnpg3 == 0 || page3blank )
          {
            skiptop = 1;
            if ( prnpg1 == 0 )
              skipbot = 1;
          }
        if ( page1blank )
          skipbot = 1;
        setpmstop ();
        if ( prev_on )
          {
            cur_mem_pag = bas_pag + 3;        
            start_mempag ();            
            hfntPrev = SelectObject( pag_DC, hfnt );             
          }
        if ( prnpg3 && !page3blank )
          {
            prn_pag ( 3 );
            prn_boxt ( 1 ); 
            if ( curwpnum )
              {
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "Call 13 to setcurpos: X: %d, Y: %d.", 
                              lndrct.right + 20, midbot - boxhgt / 2 + 20 );                          
                    wrt_debug ();
                  } 
                setcurpos ( lndrct.right + 20, midtop - boxhgt / 2 + 20);            
                sprintf ( num_str, "%d", pd[3].wstpnum );
                pcch = strlen (num_str);
                if ( prev_on )
                  TextOut ( pag_DC, dev_cur_x, dev_cur_y, num_str, pcch );         
                else
                  TextOut ( prstr.hDC, dev_cur_x, dev_cur_y, num_str, pcch );    
              }          
            sid_barst ( pagenum, 0 ); // In tf1.c                              
          }
        if ( prev_on )
          {
            save_mempag ( cur_mem_pag );            
            ++cur_mem_pag;
          }          
        setpmsbot ();
        if ( prev_on )
          {
            cur_mem_pag = bas_pag + 1;                
            start_mempag ();            
            hfntPrev = SelectObject( pag_DC, hfnt );             
          }
        if ( prnpg1 && !page1blank )
          {
            prn_pag ( 1 );
            prn_boxb ( 1 );   
            if ( curwpnum )
              {
                if ( d_bug == 4 )
                  {
                    sprintf ( d_bugstr, "Call 14 to setcurpos: X: %d, Y: %d.", 
                      lndrct.right + 20, midbot - boxhgt / 2 + 20 );                      
                    wrt_debug ();
                  } 
                setcurpos ( lndrct.right + 20, midbot - boxhgt / 2 + 20);            
                sprintf ( num_str, "%d", pd[1].wstpnum );
                pcch = strlen (num_str);
                if ( prev_on )
                  TextOut ( pag_DC, dev_cur_x, dev_cur_y, num_str, pcch );         
                else
                  TextOut ( prstr.hDC, dev_cur_x, dev_cur_y, num_str, pcch );   
              }                              
            sid_barsb ( pagenum, 0 ); // In tf1.c                                            
          }
        if ( prev_on )
          {
            save_mempag ( cur_mem_pag );            
            ++cur_mem_pag;
          }          
        pagenum += 2;
        page1blank = 0;
        page3blank = 0;
        shuf_que ( que_mxprn + 1 );
        if ( titpon )
          {
            titpon = 0;
            titpend = 0;
            fstistit = 0;
            titpfnd = 0;
          }

        if ( !stoprint )  
          {
            if ( revprn )
              {
                Escape ( prstr.hDC, ENDDOC, 0, NULL, NULL );                  
                Msg ( "Place a new page in printer, and press Enter when ready." );                         
                Escape( prstr.hDC, STARTDOC, 4, NULL, NULL );              
              }
            else
              Escape ( prstr.hDC, NEWFRAME, 0, NULL, NULL );                       
          }
        bas_pag = bas_pag + 4;
      } while ( !endfnd );
    Escape ( prstr.hDC, ENDDOC, 0, NULL, NULL );      
    SelectObject( prstr.hDC, hfntPrev ); // Could get rid of this
    DeleteObject( hfnt );
    clean_landscape ();                  
  } // END - land_rout ( void )
  
void hor_lin ( int x1, int y1, int x2, int n )
  {
    HDC hdc;
    if ( d_bug == 6 )
      {
        sprintf ( d_bugstr, "Horizontal line: %d,%d to %d,%d", x1, y1, x2, y1 );
        wrt_debug ();           
      }
    if ( x1 == x2 ) // Nothing to do
      return;     
    if ( prev_on )
      hdc = pag_DC;
    else
      hdc = prstr.hDC;      
    setcurpos ( x1, y1 );
    MoveToEx ( hdc, dev_cur_x, dev_cur_y, NULL );
    setcurpos ( x2, y1 );    
    LineTo ( hdc, dev_cur_x, dev_cur_y );
    if ( n ==3 )
      {
        setcurpos ( x1, y1-1 );
        MoveToEx ( hdc, dev_cur_x, dev_cur_y, NULL );
        setcurpos ( x2, y1-1 );    
        LineTo ( hdc, dev_cur_x, dev_cur_y );      
        setcurpos ( x1, y1+1 );
        MoveToEx ( hdc, dev_cur_x, dev_cur_y, NULL );
        setcurpos ( x2, y1+1 );    
        LineTo ( hdc, dev_cur_x, dev_cur_y );      
      }
   } // END - hor_lin ()  
  
void ver_lin ( int y1, int y2, int x1, int xr )
  {
    HDC hdc;
    if ( d_bug == 6 )
      {
        sprintf ( d_bugstr, "Vertical line: %d,%d to %d,%d", x1, y1, x1, y2 );
        wrt_debug ();           
      }    
    if ( y1 == y2 ) // Nothing to do
      return;
    if ( prev_on )
      hdc = pag_DC;
    else
      hdc = prstr.hDC;
    setcurpos ( x1, y1 );
    MoveToEx ( hdc, dev_cur_x, dev_cur_y, NULL );
    setcurpos ( x1, y2 );    
    LineTo ( hdc, dev_cur_x, dev_cur_y );       
    if ( xr == 3 )
      {
        setcurpos ( x1-1, y1 );
        MoveToEx ( hdc, dev_cur_x, dev_cur_y, NULL );
        setcurpos ( x1-1, y2 );          
        LineTo ( hdc, dev_cur_x, dev_cur_y );               
        setcurpos ( x1+1, y1 );
        MoveToEx ( hdc, dev_cur_x, dev_cur_y, NULL );
        setcurpos ( x1+1, y2 );          
        LineTo ( hdc, dev_cur_x, dev_cur_y );                       
      }    
  } // END - ver_lin ()
  